@@ -41,14 +41,19 @@ enum {
 	SEARCH_SUMMARY_CONTAINS,
 	SEARCH_DESCRIPTION_CONTAINS,
 	SEARCH_COMMENT_CONTAINS,
+	SEARCH_LOCATION_CONTAINS,
 	SEARCH_CATEGORY_IS,
 };
 
+/* Comments are disabled because they are kind of useless right now, see bug 33247 */
 static ESearchBarItem search_option_items[] = {
 	{ N_("Any field contains"), SEARCH_ANY_FIELD_CONTAINS, NULL },
 	{ N_("Summary contains"), SEARCH_SUMMARY_CONTAINS, NULL },
 	{ N_("Description contains"), SEARCH_DESCRIPTION_CONTAINS, NULL },
+#if 0
 	{ N_("Comment contains"), SEARCH_COMMENT_CONTAINS, NULL },
+#endif
+	{ N_("Location contains"), SEARCH_LOCATION_CONTAINS, NULL },
 	{ N_("Category is"), SEARCH_CATEGORY_IS, NULL },
 	{ NULL, -1, NULL }
 };
@@ -308,6 +313,10 @@ regen_query (CalSearchBar *cal_search)
 		notify_e_cal_view_contains (cal_search, "comment");
 		break;
 
+	case SEARCH_LOCATION_CONTAINS:
+		notify_e_cal_view_contains (cal_search, "location");
+		break;
+
 	case SEARCH_CATEGORY_IS:
 		notify_category_is (cal_search);
 
