@@ -30,9 +30,11 @@
 #include <gtk/gtkbindings.h>
 #include <libgnome/gnome-i18n.h>
 #include <gal/util/e-util.h>
-#include "e-util/e-dialog-utils.h"
+#include <e-util/e-dialog-utils.h>
+#include <e-util/e-icon-factory.h>
 #include "e-calendar-marshal.h"
 #include <libecal/e-cal-time-util.h>
+#include <libecal/e-cal-component.h>
 
 #include "calendar-commands.h"
 #include "calendar-component.h"
@@ -55,7 +57,7 @@
 #include "ea-calendar.h"
 
 /* Used for the status bar messages */
-#define EVOLUTION_CALENDAR_PROGRESS_IMAGE "evolution-calendar-mini.png"
+#define EVOLUTION_CALENDAR_PROGRESS_IMAGE "stock_calendar"
 static GdkPixbuf *progress_icon = NULL;
 
 struct _ECalendarViewPrivate {
@@ -66,6 +68,7 @@ struct _ECalendarViewPrivate {
 	ECalModel *model;
 
 	/* Current activity (for the EActivityHandler, i.e. the status bar).  */
+	EActivityHandler *activity_handler;
 	guint activity_id;
 
 	/* The popup menu */
@@ -528,28 +531,45 @@ e_calendar_view_set_use_24_hour_format (ECalendarView *cal_view, gboolean use_24
 }
 
 void
-e_calendar_view_set_status_message (ECalendarView *cal_view, const gchar *message)
+e_calendar_view_set_activity_handler (ECalendarView *cal_view, EActivityHandler *activity_handler)
 {
-	EActivityHandler *activity_handler = calendar_component_peek_activity_handler (calendar_component_peek ());
-
+	ECalendarViewPrivate *priv;
+	
 	g_return_if_fail (E_IS_CALENDAR_VIEW (cal_view));
 
+	priv = cal_view->priv;
+
+	priv->activity_handler = activity_handler;
+}
+
+void
+e_calendar_view_set_status_message (ECalendarView *cal_view, const gchar *message)
+{
+	ECalendarViewPrivate *priv;
+	
+	g_return_if_fail (E_IS_CALENDAR_VIEW (cal_view));
+
+	priv = cal_view->priv;
+
+	if (!priv->activity_handler)
+		return;
+	
 	if (!message || !*message) {
-		if (cal_view->priv->activity_id != 0) {
-			e_activity_handler_operation_finished (activity_handler, cal_view->priv->activity_id);
-			cal_view->priv->activity_id = 0;
+		if (priv->activity_id != 0) {
+			e_activity_handler_operation_finished (priv->activity_handler, priv->activity_id);
+			priv->activity_id = 0;
 		}
-	} else if (cal_view->priv->activity_id == 0) {
+	} else if (priv->activity_id == 0) {
 		char *client_id = g_strdup_printf ("%p", cal_view);
 
 		if (progress_icon == NULL)
-			progress_icon = gdk_pixbuf_new_from_file (EVOLUTION_IMAGESDIR "/" EVOLUTION_CALENDAR_PROGRESS_IMAGE, NULL);
+			progress_icon = e_icon_factory_get_icon (EVOLUTION_CALENDAR_PROGRESS_IMAGE, E_ICON_SIZE_STATUS);
 
-		cal_view->priv->activity_id = e_activity_handler_operation_started (activity_handler, client_id, progress_icon, message, TRUE);
+		priv->activity_id = e_activity_handler_operation_started (priv->activity_handler, client_id, progress_icon, message, TRUE);
 
 		g_free (client_id);
 	} else {
-		e_activity_handler_operation_progressing (activity_handler, cal_view->priv->activity_id, message, -1.0);
+		e_activity_handler_operation_progressing (priv->activity_handler, priv->activity_id, message, -1.0);
 	}
 }
 
@@ -851,7 +871,7 @@ e_calendar_view_delete_selected_occurrence (ECalendarView *cal_view)
 {
 	ECalendarViewEvent *event;
 	GList *selected;
-	const char *uid, *rid;
+	const char *uid, *rid = NULL;
 	GError *error = NULL;
 	ECalComponent *comp;
 		
@@ -866,20 +886,29 @@ e_calendar_view_delete_selected_occurrence (ECalendarView *cal_view)
 	e_cal_component_get_uid (comp, &uid);
 	if (e_cal_component_is_instance (comp))
 		rid = e_cal_component_get_recurid_as_string (comp);
-	else
-		rid = "";
+	else {
+		ECalComponentDateTime dt;
 
-	if (delete_component_dialog (comp, FALSE, 1, e_cal_component_get_vtype (comp), GTK_WIDGET (cal_view))) {
+		e_cal_component_get_dtstart (comp, &dt);
+		if (dt.value) {
+			rid = icaltime_as_ical_string (icaltime_from_timet (event->start, dt.value->is_date));
+			e_cal_component_free_datetime (&dt);
+		}
+	}
 
-		if (itip_organizer_is_user (comp, event->comp_data->client)
-		    && cancel_component_dialog ((GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (cal_view)),
-						event->comp_data->client,
-						comp, TRUE))
-			itip_send_comp (E_CAL_COMPONENT_METHOD_CANCEL, comp, event->comp_data->client, NULL);
+	if (rid) {
+		if (delete_component_dialog (comp, FALSE, 1, e_cal_component_get_vtype (comp), GTK_WIDGET (cal_view))) {
 
-		e_cal_remove_object_with_mod (event->comp_data->client, uid, rid, CALOBJ_MOD_THIS, &error);
-		delete_error_dialog (error, E_CAL_COMPONENT_EVENT);
-		g_clear_error (&error);
+			if (itip_organizer_is_user (comp, event->comp_data->client)
+			    && cancel_component_dialog ((GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (cal_view)),
+							event->comp_data->client,
+							comp, TRUE))
+				itip_send_comp (E_CAL_COMPONENT_METHOD_CANCEL, comp, event->comp_data->client, NULL);
+
+			e_cal_remove_object_with_mod (event->comp_data->client, uid, rid, CALOBJ_MOD_THIS, &error);
+			delete_error_dialog (error, E_CAL_COMPONENT_EVENT);
+			g_clear_error (&error);
+		}
 	}
 
 	/* free memory */
@@ -1304,7 +1333,8 @@ setup_popup_icons (EPopupMenu *context_menu)
 
 	for (i = 0; context_menu[i].name; i++) {
 		GtkWidget *pixmap_widget = NULL;
-
+		GdkPixbuf *pixbuf;
+		
 		if (!strcmp (context_menu[i].name, _("_Copy")))
 			pixmap_widget = gtk_image_new_from_stock (GTK_STOCK_COPY, GTK_ICON_SIZE_MENU);
 		else if (!strcmp (context_menu[i].name, _("C_ut")))
@@ -1317,14 +1347,26 @@ setup_popup_icons (EPopupMenu *context_menu)
 			pixmap_widget = gtk_image_new_from_stock (GTK_STOCK_HOME, GTK_ICON_SIZE_MENU);
 		else if (!strcmp (context_menu[i].name, _("_Go to Date...")))
 			pixmap_widget = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
-		else if (!strcmp (context_menu[i].name, _("New _Appointment...")))
-			pixmap_widget = gtk_image_new_from_file (EVOLUTION_IMAGESDIR "/new_appointment.xpm");
-		else if (!strcmp (context_menu[i].name, _("New All Day _Event")))
-			pixmap_widget = gtk_image_new_from_file (EVOLUTION_IMAGESDIR "/new_all_day_event.png");
-		else if (!strcmp (context_menu[i].name, _("New Meeting")))
-			pixmap_widget = gtk_image_new_from_file (EVOLUTION_IMAGESDIR "/meeting-request-16.png");
-		else if (!strcmp (context_menu[i].name, _("New Task")))
-			pixmap_widget = gtk_image_new_from_file (EVOLUTION_IMAGESDIR "/new_task-16.png");
+		else if (!strcmp (context_menu[i].name, _("New _Appointment..."))) {
+			pixbuf = e_icon_factory_get_icon ("stock_new-appointment", E_ICON_SIZE_MENU);
+			pixmap_widget = gtk_image_new_from_pixbuf (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
+		}
+		else if (!strcmp (context_menu[i].name, _("New All Day _Event"))) {
+			pixbuf = e_icon_factory_get_icon ("stock_new-24h-appointment", E_ICON_SIZE_MENU);
+			pixmap_widget = gtk_image_new_from_pixbuf (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
+		}
+		else if (!strcmp (context_menu[i].name, _("New Meeting"))) {
+			pixbuf = e_icon_factory_get_icon ("stock_new-meeting", E_ICON_SIZE_MENU);
+			pixmap_widget = gtk_image_new_from_pixbuf (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
+		}
+		else if (!strcmp (context_menu[i].name, _("New Task"))) {
+			pixbuf = e_icon_factory_get_icon ("stock_task", E_ICON_SIZE_MENU);
+			pixmap_widget = gtk_image_new_from_pixbuf (pixbuf);
+			gdk_pixbuf_unref (pixbuf);
+		}
 		else if (!strcmp (context_menu[i].name, _("_Open")))
 			pixmap_widget = gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU);
 		else if (!strcmp (context_menu[i].name, _("_Paste")))
