@@ -55,6 +55,7 @@
 #include "itip-utils.h"
 #include "e-itip-control.h"
 #include "common/authentication.h"
+#include <e-util/e-icon-factory.h>
 
 struct _EItipControlPrivate {
 	GtkWidget *html;
@@ -599,23 +600,14 @@ write_recurrence_piece (EItipControl *itip, ECalComponent *comp,
 
 	switch (r->freq) {
 	case ICAL_DAILY_RECURRENCE:
-		if (r->interval == 1)
-			strcpy (buffer, _("Every day"));
-		else
-			sprintf (buffer, _("Every %d days"), r->interval);
+		sprintf (buffer, ngettext("Every day", "Every %d days", r->interval), r->interval);
 		break;
 
 	case ICAL_WEEKLY_RECURRENCE:
 		if (r->by_day[0] == ICAL_RECURRENCE_ARRAY_MAX) {
-			if (r->interval == 1)
-				strcpy (buffer, _("Every week"));
-			else
-				sprintf (buffer, _("Every %d weeks"), r->interval);
+			sprintf (buffer, ngettext("Every week", "Every %d weeks", r->interval), r->interval);
 		} else {
-			if (r->interval == 1)
-				strcpy (buffer, _("Every week on "));
-			else
-				sprintf (buffer, _("Every %d weeks on "), r->interval);
+			sprintf (buffer, ngettext("Every week on ", "Every %d weeks on ", r->interval), r->interval);
 
 			for (i = 1; i < 8 && r->by_day[i] != ICAL_RECURRENCE_ARRAY_MAX; i++) {
 				if (i > 1)
@@ -647,22 +639,14 @@ write_recurrence_piece (EItipControl *itip, ECalComponent *comp,
 				 nth (pos), get_dayname (r, 0));
 		}
 
-		if (r->interval == 1)
-			strcat (buffer, _("every month"));
-		else {
-			len = strlen (buffer);
-			buffer += len;
-			size -= len;
-			sprintf (buffer, _("every %d months"), r->interval);
-		}
+		len = strlen (buffer);
+		buffer += len;
+		size -= len;
+		sprintf (buffer, ngettext("every month","every %d months", r->interval), r->interval);
 		break;
 
 	case ICAL_YEARLY_RECURRENCE:
-		if (r->interval == 1)
-			strcpy (buffer, _("Every year"));
-		else {
-			sprintf (buffer, _("Every %d years"), r->interval);
-		}
+		sprintf (buffer, ngettext("Every year", "Every %d years", r->interval), r->interval);
 		break;
 
 	default:
@@ -673,7 +657,7 @@ write_recurrence_piece (EItipControl *itip, ECalComponent *comp,
 	buffer += len;
 	size -= len;
 	if (r->count) {
-		sprintf (buffer, _(" a total of %d times"), r->count);
+		sprintf (buffer, ngettext("a total of %d time", " a total of %d times", r->count), r->count);
 	} else if (!icaltime_is_null_time (r->until)) {
 		ECalComponentDateTime dt;
 
@@ -776,6 +760,7 @@ write_error_html (EItipControl *itip, const gchar *itip_err)
 {
 	EItipControlPrivate *priv;
 	GtkHTMLStream *html_stream;
+	gchar *filename;
 
 	priv = itip->priv;
 
@@ -793,7 +778,9 @@ write_error_html (EItipControl *itip, const gchar *itip_err)
 	/* The column for the image */
 	gtk_html_stream_printf (html_stream, "<tr><td width=48 align=\"center\" valign=\"top\" rowspan=\"8\">");
 	/* The image */
-	gtk_html_stream_printf (html_stream, "<img src=\"/meeting-request.png\"></td>");
+	filename = e_icon_factory_get_icon_filename ("stock_new-meeting", E_ICON_SIZE_DIALOG);
+	gtk_html_stream_printf (html_stream, "<img src=\"%s\"></td>", filename);
+	g_free (filename);
 
 	gtk_html_stream_printf (html_stream, "<td align=\"left\" valign=\"top\">");
 
@@ -826,6 +813,7 @@ write_html (EItipControl *itip, const gchar *itip_desc, const gchar *itip_title,
 	const char *string;
 	gchar *html;
 	const gchar *const_html;
+	gchar *filename;
 
 	priv = itip->priv;
 
@@ -849,8 +837,9 @@ write_html (EItipControl *itip, const gchar *itip_desc, const gchar *itip_title,
 	gtk_html_write (GTK_HTML (priv->html), html_stream, const_html, strlen(const_html));
 
 	/* The image */
-	const_html = "<img src=\"/meeting-request.png\"></td>";
-	gtk_html_write (GTK_HTML (priv->html), html_stream, const_html, strlen(const_html));
+	filename = e_icon_factory_get_icon_filename ("stock_new-meeting", E_ICON_SIZE_DIALOG);
+	gtk_html_stream_printf (html_stream, "<img src=\"%s\"></td>", filename);
+	g_free (filename);
 
 	const_html = "<td align=\"left\" valign=\"top\">";
 	gtk_html_write (GTK_HTML (priv->html), html_stream, const_html, strlen(const_html));
@@ -1534,11 +1523,13 @@ e_itip_control_set_data (EItipControl *itip, const gchar *text)
 
 	priv->iter = icalcomponent_begin_component (priv->main_comp, ICAL_ANY_COMPONENT);
 	priv->ical_comp = icalcompiter_deref (&priv->iter);
-	kind = icalcomponent_isa (priv->ical_comp);
-	if (kind != ICAL_VEVENT_COMPONENT
-	    && kind != ICAL_VTODO_COMPONENT
-	    && kind != ICAL_VFREEBUSY_COMPONENT)
-		priv->ical_comp = get_next (&priv->iter);
+	if (priv->ical_comp != NULL) {
+		kind = icalcomponent_isa (priv->ical_comp);
+		if (kind != ICAL_VEVENT_COMPONENT
+		    && kind != ICAL_VTODO_COMPONENT
+		    && kind != ICAL_VFREEBUSY_COMPONENT)
+			priv->ical_comp = get_next (&priv->iter);
+	}
 
 	if (priv->ical_comp == NULL) {
 		write_error_html (itip, _("The attachment has no viewable calendar items"));		
@@ -1749,6 +1740,7 @@ update_item (EItipControl *itip)
 	icalproperty *prop;
 	icalcomponent *clone;
 	GtkWidget *dialog;
+	GError *error = NULL;
 
 	priv = itip->priv;
 
@@ -1770,9 +1762,9 @@ update_item (EItipControl *itip)
 	icalcomponent_add_component (priv->top_level, clone);
 	icalcomponent_set_method (priv->top_level, priv->method);
 
-	/* FIXME Better error dialog */
-	if (!e_cal_receive_objects (priv->current_ecal, priv->top_level, NULL)) {
-		dialog = gnome_warning_dialog (_("Calendar file could not be updated!\n"));
+	if (!e_cal_receive_objects (priv->current_ecal, priv->top_level, &error)) {
+		dialog = gnome_warning_dialog (error->message);
+		g_error_free (error);
 	} else {
 		dialog = gnome_ok_dialog (_("Update complete\n"));
 	}
@@ -2005,13 +1997,10 @@ static void
 url_requested_cb (GtkHTML *html, const gchar *url, GtkHTMLStream *handle, gpointer data)
 {	unsigned char buffer[4096];
 	int len, fd;
-        char *path;
 
-	path = g_strdup_printf ("%s/%s", EVOLUTION_IMAGESDIR, url);
-
-	if ((fd = open (path, O_RDONLY)) == -1) {
+	if ((fd = open (url, O_RDONLY)) == -1) {
 		g_warning ("%s", g_strerror (errno));
-		goto cleanup;
+		return;
 	}
 
        	while ((len = read (fd, buffer, 4096)) > 0) {
@@ -2022,14 +2011,11 @@ url_requested_cb (GtkHTML *html, const gchar *url, GtkHTMLStream *handle, gpoint
 		/* check to see if we stopped because of an error */
 		gtk_html_end (html, handle, GTK_HTML_STREAM_ERROR);
 		g_warning ("%s", g_strerror (errno));
-		goto cleanup;
+		return;
 	}
 	/* done with no errors */
 	gtk_html_end (html, handle, GTK_HTML_STREAM_OK);
 	close (fd);
-
- cleanup:
-	g_free (path);
 }
 
 static gboolean
@@ -2059,8 +2045,6 @@ object_requested_cb (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 		g_free (uid);
 	}
 	if (!source) {
-		ESource *source;
-		
 		/* Try to create a default if there isn't one */
 		source = e_source_list_peek_source_any (priv->source_lists[priv->type]);
 	}
