@@ -22,7 +22,10 @@
  *          Rodrigo Moya <rodrigo@ximian.com>
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
 #include <gnome.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <gal/util/e-util.h>
@@ -30,22 +33,24 @@
 #include <gal/menus/gal-view-instance.h>
 #include <gal/menus/gal-view-factory-etable.h>
 #include <gal/menus/gal-view-etable.h>
-#include <gtkhtml/gtkhtml.h>
-#include <gtkhtml/gtkhtml-stream.h>
 
 #include "e-util/e-categories-config.h"
 #include "e-util/e-time-utils.h"
 #include "e-util/e-url.h"
+#include "shell/e-user-creatable-items-handler.h"
 #include <libecal/e-cal-time-util.h>
 #include "widgets/menus/gal-view-menus.h"
 #include "dialogs/delete-error.h"
 #include "dialogs/task-editor.h"
+#include "e-calendar-marshal.h"
 #include "cal-search-bar.h"
 #include "calendar-config.h"
 #include "calendar-component.h"
 #include "comp-util.h"
 #include "e-calendar-table-config.h"
 #include "misc.h"
+#include "tasks-component.h"
+#include "e-cal-component-preview.h"
 #include "e-tasks.h"
 #include "common/authentication.h"
 
@@ -65,10 +70,13 @@ struct _ETasksPrivate {
 	/* Calendar search bar for tasks */
 	GtkWidget *search_bar;
 
-	/* The HTML widget to display the task's details */
-	GtkWidget *html;
+	/* The preview */
+	GtkWidget *preview;
+	
 	gchar *current_uid;
-
+	char *sexp;
+	guint update_timeout;
+	
 	/* View instance and the view menus handler */
 	GalViewInstance *view_instance;
 	GalViewMenus *view_menus;
@@ -81,12 +89,15 @@ static void e_tasks_class_init (ETasksClass *class);
 static void e_tasks_init (ETasks *tasks);
 static void setup_widgets (ETasks *tasks);
 static void e_tasks_destroy (GtkObject *object);
+static void update_view (ETasks *tasks);
 
 static void backend_error_cb (ECal *client, const char *message, gpointer data);
 
 /* Signal IDs */
 enum {
 	SELECTION_CHANGED,
+	SOURCE_ADDED,
+	SOURCE_REMOVED,
 	LAST_SIGNAL
 };
 
@@ -98,251 +109,45 @@ E_MAKE_TYPE (e_tasks, "ETasks", ETasks,
 	     e_tasks_class_init, e_tasks_init,
 	     GTK_TYPE_TABLE)
 
-/* Converts a time_t to a string, relative to the specified timezone */
-static char *
-timet_to_str_with_zone (time_t t, icaltimezone *zone)
-{
-        struct icaltimetype itt;
-        struct tm tm;
-        char buf[256];
-                                                                                              
-        if (t == -1)
-                return g_strdup (_("invalid time"));
-                                                                                              
-        itt = icaltime_from_timet_with_zone (t, FALSE, zone);
-        tm = icaltimetype_to_tm (&itt);
-                                                                                              
-        e_time_format_date_and_time (&tm, calendar_config_get_24_hour_format (),
-                                     FALSE, FALSE, buf, sizeof (buf));
-        return g_strdup (buf);
-}
-
-static void
-write_html (GtkHTMLStream *stream, ECalComponent *comp)
-{
-	ECalComponentText text;
-	ECalComponentDateTime dt;
-	gchar *buf, *str;
-	icaltimezone *current_zone;
-	GSList *l;
-	icalproperty_status status;
-	int *priority_value;
-
-	g_return_if_fail (E_IS_CAL_COMPONENT (comp));
-
-	str = calendar_config_get_timezone ();
-	if (str && str[0]) {
-		current_zone = icaltimezone_get_builtin_timezone (str);
-	} else
-		current_zone = icaltimezone_get_utc_timezone ();
-
-	/* write document header */
-	e_cal_component_get_summary (comp, &text);
-	gtk_html_stream_printf (stream,
-				"<HTML><BODY><H1>%s</H1>",
-				text.value);
-
-	/* write icons for the categories */
-	e_cal_component_get_categories_list (comp, &l);
-	if (l) {
-		GSList *node;
-
-		for (node = l; node != NULL; node = node->next) {
-			const char *icon_file;
-
-			icon_file = e_categories_config_get_icon_file_for ((const char *) node->data);
-			if (icon_file) {
-				gtk_html_stream_printf (stream, "<IMG ALT=\"%s\" SRC=\"file://%s\">",
-							(const char *) node->data, icon_file);
-			}
-		}
-
-		e_cal_component_free_categories_list (l);
-	}
-	
-	/* write summary */
-	gtk_html_stream_printf (stream,
-				"<BR><BR><BR><TABLE BORDER=\"0\" WIDTH=\"80%%\">"
-				"<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\" WIDTH=\"15%%\"><B>%s</B></TD><TD>%s</TD></TR>",
-				_("Summary:"), text.value);
-
-	/* write start date */
-	e_cal_component_get_dtstart (comp, &dt);
-	if (dt.value != NULL) {
-		buf = timet_to_str_with_zone (icaltime_as_timet (*dt.value), current_zone);
-		str = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-
-		g_free (buf);
-	} else
-		str = g_strdup ("");
-
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD><TD>%s</TD></TR>",
-				_("Start Date:"), str);
-
-	e_cal_component_free_datetime (&dt);
-	g_free (str);
-
-	/* write Due Date */
-	e_cal_component_get_due (comp, &dt);
-	if (dt.value != NULL) {
-		buf = timet_to_str_with_zone (icaltime_as_timet (*dt.value), current_zone);
-		str = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
-
-		g_free (buf);
-	} else
-		str = g_strdup ("");
-
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD><TD>%s</TD></TR>",
-				_("Due Date:"), str);
-
-	e_cal_component_free_datetime (&dt);
-	g_free (str);
-
-	/* write status */
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD>", _("Status:"));
-	e_cal_component_get_status (comp, &status);
-	switch (status) {
-	case ICAL_STATUS_INPROCESS :
-		str = g_strdup (_("In Progress"));
-		break;
-	case ICAL_STATUS_COMPLETED :
-		str = g_strdup (_("Completed"));
-		break;
-	case ICAL_STATUS_CANCELLED :
-		str = g_strdup (_("Cancelled"));
-		break;
-	case ICAL_STATUS_NONE :
-	default :
-		str = g_strdup (_("Not Started"));
-		break;
-	}
-
-	gtk_html_stream_printf (stream, "<TD>%s</TD></TR>", str);
-	g_free (str);
-
-	/* write priority */
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD>", _("Priority:"));
-	e_cal_component_get_priority (comp, &priority_value);
-	if (priority_value) {
-		if (*priority_value == 0)
-			str = g_strdup ("");
-		else if (*priority_value <= 4)
-			str = g_strdup (_("High"));
-		else if (*priority_value == 5)
-			str = g_strdup (_("Normal"));
-		else
-			str = g_strdup (_("Low"));
-
-		gtk_html_stream_printf (stream, "<TD>%s</TD></TR>", str);
-
-		g_free (str);
-		e_cal_component_free_priority (priority_value);
-	} else
-		gtk_html_stream_printf (stream, "<TD></TD></TR>");
-	
-	/* write description and URL */
-	gtk_html_stream_printf (stream, "<TR><TD COLSPAN=\"2\"><HR></TD></TR>");
-
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD>", _("Description:"));
-	e_cal_component_get_description_list (comp, &l);
-	if (l) {
-		GSList *node;
-
-		gtk_html_stream_printf (stream, "<TD>");
-
-		for (node = l; node != NULL; node = node->next) {
-			gint i;
-			GString *str = g_string_new ("");;
-
-			text = * (ECalComponentText *) node->data;
-			for (i = 0; i < strlen (text.value ? text.value : 0); i++) {
-				if (text.value[i] == '\n')
-					str = g_string_append (str, "<BR>");
-				else if (text.value[i] == '<')
-					str = g_string_append (str, "&lt;");
-				else if (text.value[i] == '>')
-					str = g_string_append (str, "&gt;");
-				else
-					str = g_string_append_c (str, text.value[i]);
-			}
-
-			gtk_html_stream_printf (stream, str->str);
-			g_string_free (str, TRUE);
-		}
-
-		gtk_html_stream_printf (stream, "</TD></TR>");
-
-		e_cal_component_free_text_list (l);
-	} else
-		gtk_html_stream_printf (stream, "<TD></TD></TR>");
-
-	/* URL */
-	gtk_html_stream_printf (stream, "<TR><TD VALIGN=\"TOP\" ALIGN=\"RIGHT\"><B>%s</B></TD>", _("Web Page:"));
-	e_cal_component_get_url (comp, (const char **) &str);
-	if (str)
-		gtk_html_stream_printf (stream, "<TD><A HREF=\"%s\">%s</A></TD></TR>", str, str);
-	else
-		gtk_html_stream_printf (stream, "<TD></TD></TR>");
-
-	gtk_html_stream_printf (stream, "</TABLE>");
-
-	/* close document */
-	gtk_html_stream_printf (stream, "</BODY></HTML>");
-}
-
-static void
-on_link_clicked (GtkHTML *html, const char *url, gpointer data)
-{
-        GError *err = NULL;
-
-        gnome_url_show (url, &err);
-
-	if (err) {
-		g_warning ("gnome_url_show: %s", err->message);
-                g_error_free (err);
-        }
-}
-
 /* Callback used when the cursor changes in the table */
 static void
 table_cursor_change_cb (ETable *etable, int row, gpointer data)
 {
 	ETasks *tasks;
 	ETasksPrivate *priv;
+	ECalModel *model;
+	ECalModelComponent *comp_data;
+	ECalComponent *comp;
+	const char *uid;
+
 	int n_selected;
 
 	tasks = E_TASKS (data);
 	priv = tasks->priv;
 
 	n_selected = e_table_selected_count (etable);
-
+	
 	/* update the HTML widget */
-	if (n_selected == 1) {
-		GtkHTMLStream *stream;
-		ECalModel *model;
-		ECalModelComponent *comp_data;
-		ECalComponent *comp;
-		const char *uid;
+	if (n_selected != 1) {
+		e_cal_component_preview_clear (E_CAL_COMPONENT_PREVIEW (priv->preview));
 
-		model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
+		return;
+	}
+	
+	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
+	
+	comp_data = e_cal_model_get_component_at (model, e_table_get_cursor_row (etable));
+	comp = e_cal_component_new ();
+	e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (comp_data->icalcomp));
 
-		stream = gtk_html_begin (GTK_HTML (priv->html));
-
-		comp_data = e_cal_model_get_component_at (model, e_table_get_cursor_row (etable));
-		comp = e_cal_component_new ();
-		e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (comp_data->icalcomp));
-		write_html (stream, comp);
-
-		gtk_html_stream_close (stream, GTK_HTML_STREAM_OK);
-
-		e_cal_component_get_uid (comp, &uid);
-		if (priv->current_uid)
-			g_free (priv->current_uid);
-		priv->current_uid = g_strdup (uid);
-
-		g_object_unref (comp);
-	} else
-		gtk_html_load_empty (GTK_HTML (priv->html));
+	e_cal_component_preview_display (E_CAL_COMPONENT_PREVIEW (priv->preview), comp_data->client, comp);
+	
+	e_cal_component_get_uid (comp, &uid);
+	if (priv->current_uid)
+		g_free (priv->current_uid);
+	priv->current_uid = g_strdup (uid);
+	
+	g_object_unref (comp);
 }
 
 /* Callback used when the selection changes in the table. */
@@ -365,13 +170,16 @@ search_bar_sexp_changed_cb (CalSearchBar *cal_search, const char *sexp, gpointer
 {
 	ETasks *tasks;
 	ETasksPrivate *priv;
-	ECalModel *model;
 
 	tasks = E_TASKS (data);
 	priv = tasks->priv;
 
-	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
-	e_cal_model_set_query (model, sexp);
+	if (priv->sexp)
+		g_free (priv->sexp);
+	
+	priv->sexp = g_strdup (sexp);
+	
+	update_view (tasks);
 }
 
 /* Callback used when the selected category in the search bar changes */
@@ -389,29 +197,6 @@ search_bar_category_changed_cb (CalSearchBar *cal_search, const char *category,
 	e_cal_model_set_default_category (model, category);
 }
 
-/* Callback used when the user selects a URL in the HTML widget */
-static void
-url_requested_cb (GtkHTML *html, const char *url, GtkHTMLStream *stream, gpointer data)
-{
-	if (!strncmp ("file:///", url, strlen ("file:///"))) {
-		GnomeVFSHandle *handle;
-		GnomeVFSResult result;
-		char buffer[4096];
-
-		if (gnome_vfs_open (&handle, url, GNOME_VFS_OPEN_READ) == GNOME_VFS_OK) {
-			do {
-				GnomeVFSFileSize bread;
-
-				result = gnome_vfs_read (handle, buffer, sizeof (buffer), &bread);
-				if (result == GNOME_VFS_OK)
-					gtk_html_stream_write (stream, buffer, bread);
-			} while (result == GNOME_VFS_OK);
-
-			gnome_vfs_close (handle);
-		}
-	}
-}
-
 static gboolean
 vpaned_resized_cb (GtkWidget *widget, GdkEventButton *event, ETasks *tasks)
 {
@@ -424,26 +209,22 @@ static void
 set_timezone (ETasks *tasks) 
 {
 	ETasksPrivate *priv;
-	char *location;
 	icaltimezone *zone;
 	GList *l;
 	
 	priv = tasks->priv;
 	
-	location = calendar_config_get_timezone ();
-	zone = icaltimezone_get_builtin_timezone (location);
-	g_free (location);
-	
-	if (!zone)
-		zone = icaltimezone_get_utc_timezone ();
-
+	zone = calendar_config_get_icaltimezone ();
 	for (l = priv->clients_list; l != NULL; l = l->next) {
 		ECal *client = l->data;
-		
+
 		if (e_cal_get_load_state (client) == E_CAL_LOAD_LOADED)
 			/* FIXME Error checking */
 			e_cal_set_default_timezone (client, zone, NULL);
 	}
+
+	if (priv->preview)
+		e_cal_component_preview_set_default_timezone (E_CAL_COMPONENT_PREVIEW (priv->preview), zone);
 }
 
 static void
@@ -454,6 +235,41 @@ timezone_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer
 	set_timezone (tasks);
 }
 
+static void
+update_view (ETasks *tasks)
+{
+	ETasksPrivate *priv;
+	ECalModel *model;
+	char *real_sexp = NULL;
+	char *new_sexp = NULL;
+	
+	priv = tasks->priv;
+
+	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
+		
+	if ((new_sexp = calendar_config_get_hide_completed_tasks_sexp()) != NULL) {
+		real_sexp = g_strdup_printf ("(and %s %s)", new_sexp, priv->sexp);
+		e_cal_model_set_search_query (model, real_sexp);
+		g_free (new_sexp);
+		g_free (real_sexp);
+	} else
+		e_cal_model_set_search_query (model, priv->sexp);
+}
+
+static gboolean
+update_view_cb (ETasks *tasks)
+{	
+	update_view (tasks);
+
+	return TRUE;
+}
+
+static void
+config_hide_completed_tasks_changed_cb (GConfClient *client, guint id, GConfEntry *entry, gpointer data)
+{
+	update_view (data);
+}
+
 static void
 model_row_changed_cb (ETableModel *etm, int row, gpointer data)
 {
@@ -494,6 +310,18 @@ setup_config (ETasks *tasks)
 	
 	not = calendar_config_add_notification_timezone (timezone_changed_cb, tasks);
 	priv->notifications = g_list_prepend (priv->notifications, GUINT_TO_POINTER (not));
+	
+	not = calendar_config_add_notification_hide_completed_tasks (config_hide_completed_tasks_changed_cb, 
+							      tasks);
+	priv->notifications = g_list_prepend (priv->notifications, GUINT_TO_POINTER (not));
+	
+	not = calendar_config_add_notification_hide_completed_tasks_units (config_hide_completed_tasks_changed_cb, 
+							      tasks);
+	priv->notifications = g_list_prepend (priv->notifications, GUINT_TO_POINTER (not));
+	
+	not = calendar_config_add_notification_hide_completed_tasks_value (config_hide_completed_tasks_changed_cb, 
+							      tasks);
+	priv->notifications = g_list_prepend (priv->notifications, GUINT_TO_POINTER (not));	
 }
 
 #define E_TASKS_TABLE_DEFAULT_STATE					\
@@ -512,7 +340,7 @@ setup_widgets (ETasks *tasks)
 	ETasksPrivate *priv;
 	ETable *etable;
 	ECalModel *model;
-	GtkWidget *paned, *scroll;
+	GtkWidget *paned;
 
 	priv = tasks->priv;
 
@@ -548,25 +376,14 @@ setup_widgets (ETasks *tasks)
 	g_signal_connect (etable, "cursor_change", G_CALLBACK (table_cursor_change_cb), tasks);
 	g_signal_connect (etable, "selection_change", G_CALLBACK (table_selection_change_cb), tasks);
 
+	/* Timeout check to hide completed items */
+	priv->update_timeout = g_timeout_add_full (G_PRIORITY_LOW, 60000, (GSourceFunc) update_view_cb, tasks, NULL);	
+
 	/* create the task detail */
-	priv->html = gtk_html_new ();
-	gtk_html_set_default_content_type (GTK_HTML (priv->html), "charset=utf-8");
-	gtk_html_load_empty (GTK_HTML (priv->html));
-
-	g_signal_connect (G_OBJECT (priv->html), "url_requested",
-			  G_CALLBACK (url_requested_cb), NULL);
-	g_signal_connect (G_OBJECT (priv->html), "link_clicked",
-			  G_CALLBACK (on_link_clicked), tasks);
-
-	gtk_widget_pop_colormap ();
-	scroll = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroll), GTK_SHADOW_IN);
-	gtk_container_add (GTK_CONTAINER (scroll), priv->html);
-	gtk_paned_add2 (GTK_PANED (paned), scroll);
-	gtk_widget_show_all (scroll);
+	priv->preview = e_cal_component_preview_new ();
+	e_cal_component_preview_set_default_timezone (E_CAL_COMPONENT_PREVIEW (priv->preview), calendar_config_get_icaltimezone ());	
+	gtk_paned_add2 (GTK_PANED (paned), priv->preview);
+	gtk_widget_show (priv->preview);
 
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
 	g_signal_connect (G_OBJECT (model), "model_row_changed",
@@ -592,9 +409,33 @@ e_tasks_class_init (ETasksClass *class)
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_INT);
 
+	e_tasks_signals[SOURCE_ADDED] =
+		g_signal_new ("source_added",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ETasksClass, source_added),
+			      NULL, NULL,
+			      e_calendar_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_OBJECT);
+
+	e_tasks_signals[SOURCE_REMOVED] =
+		g_signal_new ("source_removed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ETasksClass, source_removed),
+			      NULL, NULL,
+			      e_calendar_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_OBJECT);
+
 	object_class->destroy = e_tasks_destroy;
 
 	class->selection_changed = NULL;
+	class->source_added = NULL;
+	class->source_removed = NULL;
 }
 
 
@@ -603,7 +444,7 @@ static void
 e_tasks_init (ETasks *tasks)
 {
 	ETasksPrivate *priv;
-
+	
 	priv = g_new0 (ETasksPrivate, 1);
 	tasks->priv = priv;
 
@@ -615,6 +456,9 @@ e_tasks_init (ETasks *tasks)
 	priv->view_instance = NULL;
 	priv->view_menus = NULL;
 	priv->current_uid = NULL;
+	priv->sexp = g_strdup ("#t");
+
+	update_view (tasks);
 }
 
 /* Callback used when the set of categories changes in the calendar client */
@@ -680,7 +524,17 @@ e_tasks_destroy (GtkObject *object)
 			g_free (priv->current_uid);
 			priv->current_uid = NULL;
 		}
+	
+		if (priv->sexp) {
+			g_free (priv->sexp);
+			priv->sexp = NULL;
+		}
 
+		if (priv->update_timeout) {
+			g_source_remove (priv->update_timeout);
+			priv->update_timeout = 0;
+		}
+	
 		if (priv->tasks_view_config) {
 			g_object_unref (priv->tasks_view_config);
 			priv->tasks_view_config = NULL;
@@ -689,7 +543,7 @@ e_tasks_destroy (GtkObject *object)
 		for (l = priv->notifications; l; l = l->next)
 			calendar_config_remove_notification (GPOINTER_TO_UINT (l->data));
 		priv->notifications = NULL;
-		
+
 		g_free (priv);
 		tasks->priv = NULL;
 	}
@@ -733,6 +587,39 @@ backend_error_cb (ECal *client, const char *message, gpointer data)
 	g_free (urinopwd);
 }
 
+/* Callback from the calendar client when the backend dies */
+static void
+backend_died_cb (ECal *client, gpointer data)
+{
+	ETasks *tasks;
+	ETasksPrivate *priv;
+	ESource *source;
+	char *message;
+	GtkWidget *dialog;
+	
+	tasks = E_TASKS (data);
+	priv = tasks->priv;
+
+	source = g_object_ref (e_cal_get_source (client));
+
+	priv->clients_list = g_list_remove (priv->clients_list, client);
+	g_hash_table_remove (priv->clients, e_cal_get_uri (client));
+
+	gtk_signal_emit (GTK_OBJECT (tasks), e_tasks_signals[SOURCE_REMOVED], source);
+
+	e_calendar_table_set_status_message (E_CALENDAR_TABLE (e_tasks_get_calendar_table (tasks)), NULL);
+
+	message = g_strdup_printf (_("The task backend for '%s' has crashed."), e_source_peek_name (source));
+	dialog = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tasks))),
+					 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+					 message);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+	g_free (message);
+
+	g_object_unref (source);
+}
+
 void
 e_tasks_open_task			(ETasks		*tasks)
 {
@@ -773,31 +660,45 @@ e_tasks_new_task			(ETasks		*tasks)
 }
 
 gboolean
-e_tasks_add_todo_uri (ETasks *tasks, const char *str_uri)
+e_tasks_add_todo_source (ETasks *tasks, ESource *source)
 {
 	ETasksPrivate *priv;
 	ECal *client;
 	ECalModel *model;
+	char *str_uri;
 	GError *error = NULL;
 
 	g_return_val_if_fail (tasks != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TASKS (tasks), FALSE);
-	g_return_val_if_fail (str_uri != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
 
 	priv = tasks->priv;
 
+	str_uri = e_source_get_uri (source);
 	client = g_hash_table_lookup (priv->clients, str_uri);
-	if (client)
+	if (client) {
+		g_free (str_uri);
 		return TRUE;
+	}
 
+	/* FIXME Loading should be async */
+	/* FIXME With no event handling here the status message never actually changes */
 	set_status_message (tasks, _("Opening tasks at %s"), str_uri);
 
-	client = auth_new_cal_from_uri (str_uri, E_CAL_SOURCE_TYPE_TODO);
-	g_hash_table_insert (priv->clients, g_strdup (str_uri), client);
+	client = auth_new_cal_from_source (source, E_CAL_SOURCE_TYPE_TODO);
+	if (!client) {
+		g_free (str_uri);
+		return FALSE;
+	}
+
+	g_hash_table_insert (priv->clients, str_uri, client);
 	priv->clients_list = g_list_prepend (priv->clients_list, client);
 	
 	g_signal_connect (G_OBJECT (client), "backend_error", G_CALLBACK (backend_error_cb), tasks);
 	g_signal_connect (G_OBJECT (client), "categories_changed", G_CALLBACK (client_categories_changed_cb), tasks);
+	g_signal_connect (G_OBJECT (client), "backend_died", G_CALLBACK (backend_died_cb), tasks);
+
+	gtk_signal_emit (GTK_OBJECT (tasks), e_tasks_signals[SOURCE_ADDED], source);
 
 	if (!e_cal_open (client, FALSE, &error)) {
 		GtkWidget *dialog;
@@ -809,11 +710,15 @@ e_tasks_add_todo_uri (ETasks *tasks, const char *str_uri)
 						 str_uri, error ? error->message : "");
 
 		g_error_free (error);
-		g_hash_table_remove (priv->clients, str_uri);
 		priv->clients_list = g_list_prepend (priv->clients_list, client);
 		g_signal_handlers_disconnect_matched (client, G_SIGNAL_MATCH_DATA,
 						      0, 0, NULL, NULL, tasks);	
 
+		/* Do this last because it unrefs the client */
+		g_hash_table_remove (priv->clients, str_uri);
+
+		gtk_signal_emit (GTK_OBJECT (tasks), e_tasks_signals[SOURCE_REMOVED], source);
+
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 
@@ -833,23 +738,26 @@ e_tasks_add_todo_uri (ETasks *tasks, const char *str_uri)
 }
 
 gboolean
-e_tasks_remove_todo_uri (ETasks *tasks, const char *str_uri)
+e_tasks_remove_todo_source (ETasks *tasks, ESource *source)
 {
 	ETasksPrivate *priv;
 	ECal *client;
 	ECalModel *model;
+	char *str_uri;
 
 	g_return_val_if_fail (tasks != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TASKS (tasks), FALSE);
-	g_return_val_if_fail (str_uri != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
 
 	priv = tasks->priv;
 
+	str_uri = e_source_get_uri (source);
 	client = g_hash_table_lookup (priv->clients, str_uri);
-	if (!client)
+	if (!client) {
+		g_free (str_uri);
 		return TRUE;
+	}
 
-	g_hash_table_remove (priv->clients, str_uri);
 	priv->clients_list = g_list_remove (priv->clients_list, client);
 	g_signal_handlers_disconnect_matched (client, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, tasks);	
@@ -857,24 +765,32 @@ e_tasks_remove_todo_uri (ETasks *tasks, const char *str_uri)
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
 	e_cal_model_remove_client (model, client);
 
+	g_hash_table_remove (priv->clients, str_uri);
+	g_free (str_uri);
+
+	gtk_signal_emit (GTK_OBJECT (tasks), e_tasks_signals[SOURCE_REMOVED], source);
+
 	return TRUE;
 }
 
 gboolean
-e_tasks_set_default_uri (ETasks *tasks, const char *str_uri)
+e_tasks_set_default_source (ETasks *tasks, ESource *source)
 {
 	ETasksPrivate *priv;
 	ECal *ecal;
 	ECalModel *model;
+	char *str_uri;
 	
 	g_return_val_if_fail (tasks != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TASKS (tasks), FALSE);
-	g_return_val_if_fail (str_uri != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SOURCE (source), FALSE);
 
 	priv = tasks->priv;
 
+	str_uri = e_source_get_uri (source);
 	model = e_calendar_table_get_model (E_CALENDAR_TABLE (priv->tasks_view));
 	ecal = e_cal_model_get_client_for_uri (model, str_uri);
+	g_free (str_uri);
 	if (!ecal)
 		return FALSE;
 
@@ -941,6 +857,8 @@ e_tasks_delete_selected (ETasks *tasks)
 	set_status_message (tasks, _("Deleting selected objects..."));
 	e_calendar_table_delete_selected (cal_table);
 	set_status_message (tasks, NULL);
+
+	e_cal_component_preview_clear (E_CAL_COMPONENT_PREVIEW (priv->preview));
 }
 
 /**
@@ -1041,7 +959,8 @@ e_tasks_setup_view_menus (ETasks *tasks, BonoboUIComponent *uic)
 
 		gal_view_collection_set_title (collection, _("Tasks"));
 
-		dir = gnome_util_prepend_user_home ("/evolution/views/tasks/");
+		dir = g_build_filename (tasks_component_peek_base_directory (tasks_component_peek ()), 
+					"tasks", "views", NULL);		
 		gal_view_collection_set_storage_directories (collection,
 							     EVOLUTION_GALVIEWSDIR "/tasks/",
 							     dir);
@@ -1063,7 +982,7 @@ e_tasks_setup_view_menus (ETasks *tasks, BonoboUIComponent *uic)
 		gal_view_collection_load (collection);
 	}
 
-	priv->view_instance = gal_view_instance_new (collection, e_cal_get_uri (e_tasks_get_default_client (tasks)));
+	priv->view_instance = gal_view_instance_new (collection, NULL);
 
 	priv->view_menus = gal_view_menus_new (priv->view_instance);
 	gal_view_menus_apply (priv->view_menus, uic, NULL);
