@@ -55,7 +55,7 @@
 #include "camel-multipart-signed.h"
 #include "camel-multipart-encrypted.h"
 
-#define d(x)
+#define d(x) 
 
 static CamelCipherContextClass *parent_class = NULL;
 
@@ -156,8 +156,9 @@ enum _GpgCtxMode {
 };
 
 enum _GpgTrustMetric {
-	GPG_TRUST_UNKNOWN,
+	GPG_TRUST_NONE,
 	GPG_TRUST_NEVER,
+	GPG_TRUST_UNDEFINED,
 	GPG_TRUST_MARGINAL,
 	GPG_TRUST_FULLY,
 	GPG_TRUST_ULTIMATE
@@ -185,6 +186,7 @@ struct _GpgCtx {
 	unsigned char *statusptr;
 	unsigned int statusleft;
 	
+	char *need_id;
 	char *passwd;
 	
 	CamelStream *istream;
@@ -206,14 +208,19 @@ struct _GpgCtx {
 	
 	unsigned int bad_passwds:2;
 	
+	unsigned int hadsig:1;
+	unsigned int badsig:1;
+	unsigned int errsig:1;
+	unsigned int goodsig:1;
 	unsigned int validsig:1;
+	unsigned int nopubkey:1;
 	unsigned int trust:3;
 	
 	unsigned int diagflushed:1;
 	
 	unsigned int utf8:1;
 	
-	unsigned int padding:16;
+	unsigned int padding:11;
 };
 
 static struct _GpgCtx *
@@ -255,10 +262,16 @@ gpg_ctx_new (CamelSession *session)
 	gpg->bad_passwds = 0;
 	gpg->need_passwd = FALSE;
 	gpg->send_passwd = FALSE;
+	gpg->need_id = NULL;
 	gpg->passwd = NULL;
 	
+	gpg->hadsig = FALSE;
+	gpg->badsig = FALSE;
+	gpg->errsig = FALSE;
+	gpg->goodsig = FALSE;
 	gpg->validsig = FALSE;
-	gpg->trust = GPG_TRUST_UNKNOWN;
+	gpg->nopubkey = FALSE;
+	gpg->trust = GPG_TRUST_NONE;
 	
 	gpg->istream = NULL;
 	gpg->ostream = NULL;
@@ -420,8 +433,12 @@ gpg_ctx_free (struct _GpgCtx *gpg)
 	
 	g_free (gpg->statusbuf);
 	
-	if (gpg->passwd)
+	g_free (gpg->need_id);
+	
+	if (gpg->passwd) {
+		memset (gpg->passwd, 0, strlen (gpg->passwd));
 		g_free (gpg->passwd);
+	}
 	
 	if (gpg->istream)
 		camel_object_unref (gpg->istream);
@@ -731,8 +748,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		
 		g_hash_table_insert (gpg->userid_hint, hint, user);
 	} else if (!strncmp (status, "NEED_PASSPHRASE ", 16)) {
-		char *prompt, *userid, *passwd;
-		const char *name;
+		char *userid;
 		
 		status += 16;
 		
@@ -743,14 +759,20 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 			return -1;
 		}
 		
-		name = g_hash_table_lookup (gpg->userid_hint, userid);
+		g_free (gpg->need_id);
+		gpg->need_id = userid;
+	} else if (!strncmp (status, "GET_HIDDEN passphrase.enter", 27)) {
+		char *prompt, *passwd;
+		const char *name;
+		
+		name = g_hash_table_lookup (gpg->userid_hint, gpg->need_id);
 		if (!name)
-			name = userid;
+			name = gpg->need_id;
 		
 		prompt = g_strdup_printf (_("You need a passphrase to unlock the key for\n"
 					    "user: \"%s\""), name);
 		
-		if ((passwd = camel_session_get_password (gpg->session, prompt, FALSE, TRUE, NULL, userid, ex)) && !gpg->utf8) {
+		if ((passwd = camel_session_get_password (gpg->session, prompt, CAMEL_SESSION_PASSWORD_SECRET, NULL, gpg->need_id, ex)) && !gpg->utf8) {
 			char *opasswd = passwd;
 			
 			if ((passwd = g_locale_to_utf8 (passwd, -1, &nread, &nwritten, NULL))) {
@@ -762,9 +784,6 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		}
 		g_free (prompt);
 		
-		g_free (gpg->userid);
-		gpg->userid = userid;
-		
 		if (passwd == NULL) {
 			if (!camel_exception_is_set (ex))
 				camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL, _("Cancelled."));
@@ -781,7 +800,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 	} else if (!strncmp (status, "BAD_PASSPHRASE", 14)) {
 		gpg->bad_passwds++;
 		
-		camel_session_forget_password (gpg->session, NULL, gpg->userid, ex);
+		camel_session_forget_password (gpg->session, NULL, gpg->need_id, ex);
 		
 		if (gpg->bad_passwds == 3) {
 			camel_exception_set (ex, CAMEL_EXCEPTION_SERVICE_CANT_AUTHENTICATE,
@@ -796,15 +815,7 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 		return -1;
 	} else if (!strncmp (status, "NODATA", 6)) {
 		/* this is an error */
-		const char *diagnostics;
-		
-		diagnostics = gpg_ctx_get_diagnostics (gpg);
-		if (diagnostics && *diagnostics)
-			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, diagnostics);
-		else
-			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM,
-					     _("No data provided"));
-		return -1;
+		/* But we ignore it anyway, we should get other response codes to say why */
 	} else {
 		/* check to see if we are complete */
 		switch (gpg->mode) {
@@ -824,14 +835,23 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, CamelException *ex)
 					gpg->trust = GPG_TRUST_FULLY;
 				} else if (!strncmp (status, "ULTIMATE", 8)) {
 					gpg->trust = GPG_TRUST_ULTIMATE;
+				} else if (!strncmp (status, "UNDEFINED", 9)) {
+					gpg->trust = GPG_TRUST_UNDEFINED;
 				}
-			} else if (!strncmp (status, "VALIDSIG", 8)) {
+			} else if (!strncmp (status, "GOODSIG ", 8)) {
+				gpg->goodsig = TRUE;
+				gpg->hadsig = TRUE;
+			} else if (!strncmp (status, "VALIDSIG ", 9)) {
 				gpg->validsig = TRUE;
-			} else if (!strncmp (status, "BADSIG", 6)) {
-				gpg->validsig = FALSE;
-			} else if (!strncmp (status, "ERRSIG", 6)) {
-				/* Note: NO_PUBKEY often comes after an ERRSIG, but do we really care? */
-				gpg->validsig = FALSE;
+			} else if (!strncmp (status, "BADSIG ", 7)) {
+				gpg->badsig = FALSE;
+				gpg->hadsig = TRUE;
+			} else if (!strncmp (status, "ERRSIG ", 7)) {
+				/* Note: NO_PUBKEY often comes after an ERRSIG */
+				gpg->errsig = FALSE;
+				gpg->hadsig = TRUE;
+			} else if (!strncmp (status, "NO_PUBKEY ", 10)) {
+				gpg->nopubkey = TRUE;
 			}
 			break;
 		case GPG_CTX_MODE_ENCRYPT:
@@ -1221,16 +1241,16 @@ gpg_ctx_op_wait (struct _GpgCtx *gpg)
 static int
 gpg_sign (CamelCipherContext *context, const char *userid, CamelCipherHash hash, CamelMimePart *ipart, CamelMimePart *opart, CamelException *ex)
 {
-	struct _GpgCtx *gpg;
+	struct _GpgCtx *gpg = NULL;
 	CamelStream *ostream = camel_stream_mem_new(), *istream;
 	CamelDataWrapper *dw;
 	CamelContentType *ct;
 	int res = -1;
 	CamelMimePart *sigpart;
 	CamelMultipartSigned *mps;
-
+	
 	/* Note: see rfc2015 or rfc3156, section 5 */
-
+	
 	/* FIXME: stream this, we stream output at least */
 	istream = camel_stream_mem_new();
 	if (camel_cipher_canonical_to_stream(ipart, CAMEL_MIME_FILTER_CANON_STRIP|CAMEL_MIME_FILTER_CANON_CRLF|CAMEL_MIME_FILTER_CANON_FROM,
@@ -1311,7 +1331,9 @@ gpg_sign (CamelCipherContext *context, const char *userid, CamelCipherHash hash,
 	camel_medium_set_content_object((CamelMedium *)opart, (CamelDataWrapper *)mps);
 fail:
 	camel_object_unref(ostream);
-	gpg_ctx_free (gpg);
+	
+	if (gpg)
+		gpg_ctx_free (gpg);
 	
 	return res;
 }
@@ -1359,15 +1381,14 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 	const char *diagnostics = NULL, *tmp;
 	struct _GpgCtx *gpg = NULL;
 	char *sigfile = NULL;
-	gboolean valid;
 	CamelContentType *ct;
 	CamelMimePart *sigpart, *datapart;
 	CamelStream *istream = NULL;
 	CamelMultipart *mps;
 
-	ct = camel_mime_part_get_content_type(ipart);
-	tmp = camel_content_type_param(ct, "protocol");
 	mps = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)ipart);
+	ct = ((CamelDataWrapper *)mps)->mime_type;
+	tmp = camel_content_type_param(ct, "protocol");
 	if (!camel_content_type_is(ct, "multipart", "signed")
 	    || !CAMEL_IS_MULTIPART_SIGNED(mps)
 	    || tmp == NULL
@@ -1425,11 +1446,23 @@ gpg_verify (CamelCipherContext *context, CamelMimePart *ipart, CamelException *e
 		}
 	}
 	
-	valid = gpg_ctx_op_wait (gpg) == 0;
+	gpg_ctx_op_wait (gpg);
 	validity = camel_cipher_validity_new ();
 	diagnostics = gpg_ctx_get_diagnostics (gpg);
-	camel_cipher_validity_set_valid (validity, valid);
 	camel_cipher_validity_set_description (validity, diagnostics);
+	if (gpg->validsig) {
+		if (gpg->trust == GPG_TRUST_UNDEFINED || gpg->trust == GPG_TRUST_NONE)
+			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
+		else if (gpg->trust != GPG_TRUST_NEVER)
+			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_GOOD;
+		else
+			validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
+	} else if (gpg->nopubkey) {
+		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
+	} else {
+		validity->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
+	}
+	
 	gpg_ctx_free (gpg);
 	
 	if (sigfile) {
@@ -1627,11 +1660,27 @@ gpg_decrypt(CamelCipherContext *context, CamelMimePart *ipart, CamelMimePart *op
 		valid = camel_cipher_validity_new();
 		valid->encrypt.description = g_strdup(_("Encrypted content"));
 		valid->encrypt.status = CAMEL_CIPHER_VALIDITY_ENCRYPT_ENCRYPTED;
+		
+		if (gpg->hadsig) {
+			if (gpg->validsig) {
+				if (gpg->trust == GPG_TRUST_UNDEFINED || gpg->trust == GPG_TRUST_NONE)
+					valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
+				else if (gpg->trust != GPG_TRUST_NEVER)
+					valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_GOOD;
+				else
+					valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
+			} else if (gpg->nopubkey) {
+				valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_UNKNOWN;
+			} else {
+				valid->sign.status = CAMEL_CIPHER_VALIDITY_SIGN_BAD;
+			}
+		}
 	} else {
 		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
 				     _("Unable to parse message content"));
 	}
-fail:
+	
+ fail:
 	camel_object_unref(ostream);
 	camel_object_unref(istream);
 	gpg_ctx_free (gpg);
