@@ -31,6 +31,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include "camel-debug.h"
+
 #include "camel-session.h"
 #include "camel-store.h"
 #include "camel-folder.h"
@@ -50,10 +52,7 @@ static CamelFolder *get_folder (CamelStore *store, const char *folder_name,
 				guint32 flags, CamelException *ex);
 static CamelFolder *get_inbox (CamelStore *store, CamelException *ex);
 
-static void        init_trash (CamelStore *store);
 static CamelFolder *get_trash (CamelStore *store, CamelException *ex);
-
-static void        init_junk (CamelStore *store);
 static CamelFolder *get_junk (CamelStore *store, CamelException *ex);
 
 static CamelFolderInfo *create_folder (CamelStore *store,
@@ -65,7 +64,7 @@ static void delete_folder (CamelStore *store, const char *folder_name,
 static void rename_folder (CamelStore *store, const char *old_name,
 			   const char *new_name, CamelException *ex);
 
-static void store_sync (CamelStore *store, CamelException *ex);
+static void store_sync (CamelStore *store, int expunge, CamelException *ex);
 static CamelFolderInfo *get_folder_info (CamelStore *store, const char *top,
 					 guint32 flags, CamelException *ex);
 static void free_folder_info (CamelStore *store, CamelFolderInfo *tree);
@@ -96,9 +95,7 @@ camel_store_class_init (CamelStoreClass *camel_store_class)
 	camel_store_class->compare_folder_name = g_str_equal;
 	camel_store_class->get_folder = get_folder;
 	camel_store_class->get_inbox = get_inbox;
-	camel_store_class->init_trash = init_trash;
 	camel_store_class->get_trash = get_trash;
-	camel_store_class->init_junk = init_junk;
 	camel_store_class->get_junk = get_junk;
 	camel_store_class->create_folder = create_folder;
 	camel_store_class->delete_folder = delete_folder;
@@ -235,30 +232,55 @@ camel_store_get_folder (CamelStore *store, const char *folder_name, guint32 flag
 	
 	g_return_val_if_fail (folder_name != NULL, NULL);
 	
-	CAMEL_STORE_LOCK(store, folder_lock);
+	/* O_EXCL doesn't make sense if we aren't requesting to also create the folder if it doesn't exist */
+	if (!(flags & CAMEL_STORE_FOLDER_CREATE))
+		flags &= ~CAMEL_STORE_FOLDER_EXCL;
 	
-	if (store->folders)
+	if (store->folders) {
 		/* Try cache first. */
 		folder = camel_object_bag_reserve(store->folders, folder_name);
+		if (folder && (flags & CAMEL_STORE_FOLDER_EXCL)) {
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("Cannot create folder `%s': folder exists"),
+					      folder_name);
+			
+			camel_object_unref (folder);
+			return NULL;
+		}
+	}
 	
 	if (!folder) {
-		folder = CS_CLASS (store)->get_folder (store, folder_name, flags, ex);
-		if (folder) {
-			/* Add the folder to the vTrash/vJunk folder if this store implements it */
-			if (store->vtrash)
-				camel_vee_folder_add_folder (CAMEL_VEE_FOLDER (store->vtrash), folder);
-			if (store->vjunk)
-				camel_vee_folder_add_folder (CAMEL_VEE_FOLDER (store->vjunk), folder);
-			
-			if (store->folders)
-				camel_object_bag_add(store->folders, folder_name, folder);
+		if ((store->flags & CAMEL_STORE_VTRASH) && strcmp(folder_name, CAMEL_VTRASH_NAME) == 0) {
+			folder = CS_CLASS(store)->get_trash(store, ex);
+		} else if ((store->flags & CAMEL_STORE_VJUNK) && strcmp(folder_name, CAMEL_VJUNK_NAME) == 0) {
+			folder = CS_CLASS(store)->get_junk(store, ex);
 		} else {
-			if (store->folders)
+			folder = CS_CLASS (store)->get_folder(store, folder_name, flags, ex);
+			if (folder) {
+				CamelVeeFolder *vfolder;
+				
+				if ((store->flags & CAMEL_STORE_VTRASH)
+				    && (vfolder = camel_object_bag_get(store->folders, CAMEL_VTRASH_NAME))) {
+					camel_vee_folder_add_folder(vfolder, folder);
+					camel_object_unref(vfolder);
+				}
+
+				if ((store->flags & CAMEL_STORE_VJUNK)
+				    && (vfolder = camel_object_bag_get(store->folders, CAMEL_VJUNK_NAME))) {
+					camel_vee_folder_add_folder(vfolder, folder);
+					camel_object_unref(vfolder);
+				}
+			}
+		}
+
+		if (store->folders) {
+			if (folder)
+				camel_object_bag_add(store->folders, folder_name, folder);
+			else
 				camel_object_bag_abort(store->folders, folder_name);
 		}
 	}
 
-	CAMEL_STORE_UNLOCK(store, folder_lock);
 	return folder;
 }
 
@@ -294,6 +316,14 @@ camel_store_create_folder (CamelStore *store, const char *parent_name,
 {
 	CamelFolderInfo *fi;
 
+	if ((parent_name == NULL || parent_name[0] == 0)
+	    && (((store->flags & CAMEL_STORE_VTRASH) && strcmp(folder_name, CAMEL_VTRASH_NAME) == 0)
+		|| ((store->flags & CAMEL_STORE_VJUNK) && strcmp(folder_name, CAMEL_VJUNK_NAME) == 0))) {
+		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_INVALID,
+				     _("Cannot create folder: %s: folder exists"), folder_name);
+		return NULL;
+	}
+
 	CAMEL_STORE_LOCK(store, folder_lock);
 	fi = CS_CLASS (store)->create_folder (store, parent_name, folder_name, ex);
 	CAMEL_STORE_UNLOCK(store, folder_lock);
@@ -301,6 +331,34 @@ camel_store_create_folder (CamelStore *store, const char *parent_name,
 	return fi;
 }
 
+/* deletes folder/removes it from the folder cache, if it's there */
+static void
+cs_delete_cached_folder(CamelStore *store, const char *folder_name)
+{
+	CamelFolder *folder;
+
+	if (store->folders
+	    && (folder = camel_object_bag_get(store->folders, folder_name))) {
+		CamelVeeFolder *vfolder;
+			
+		if ((store->flags & CAMEL_STORE_VTRASH)
+		    && (vfolder = camel_object_bag_get(store->folders, CAMEL_VTRASH_NAME))) {
+			camel_vee_folder_remove_folder(vfolder, folder);
+			camel_object_unref(vfolder);
+		}
+
+		if ((store->flags & CAMEL_STORE_VJUNK)
+		    && (vfolder = camel_object_bag_get(store->folders, CAMEL_VJUNK_NAME))) {
+			camel_vee_folder_remove_folder(vfolder, folder);
+			camel_object_unref(vfolder);
+		}
+
+		camel_folder_delete(folder);
+
+		camel_object_bag_remove(store->folders, folder);
+		camel_object_unref(folder);
+	}
+}
 
 static void
 delete_folder (CamelStore *store, const char *folder_name, CamelException *ex)
@@ -320,32 +378,26 @@ delete_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 void
 camel_store_delete_folder (CamelStore *store, const char *folder_name, CamelException *ex)
 {
-	CamelFolder *folder = NULL;
-	
+	CamelException local;
+
+	/* TODO: should probably be a parameter/bit on the storeinfo */
+	if (((store->flags & CAMEL_STORE_VTRASH) && strcmp(folder_name, CAMEL_VTRASH_NAME) == 0)
+	    || ((store->flags & CAMEL_STORE_VJUNK) && strcmp(folder_name, CAMEL_VJUNK_NAME) == 0)) {
+		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+				     _("Cannot delete folder: %s: Invalid operation"), folder_name);
+		return;
+	}
+
+	camel_exception_init(&local);
+
 	CAMEL_STORE_LOCK(store, folder_lock);
 
-	/* NB: Note similarity of this code to unsubscribe_folder */
-	
-	/* if we deleted a folder, force it out of the cache, and also out of the vtrash/vjunk if setup */
-	if (store->folders) {
-		folder = camel_object_bag_get(store->folders, folder_name);
-		if (folder) {
-			if (store->vtrash)
-				camel_vee_folder_remove_folder((CamelVeeFolder *)store->vtrash, folder);
-			if (store->vjunk)
-				camel_vee_folder_remove_folder((CamelVeeFolder *)store->vjunk, folder);
-			camel_folder_delete (folder);
-		}
-	}
+	CS_CLASS(store)->delete_folder(store, folder_name, &local);
 
-	CS_CLASS (store)->delete_folder (store, folder_name, ex);
-	
-	if (folder) {
-		if (store->folders)
-			camel_object_bag_remove (store->folders, folder);
-		
-		camel_object_unref (folder);
-	}
+	if (!camel_exception_is_set(&local))
+		cs_delete_cached_folder(store, folder_name);
+	else
+		camel_exception_xfer(ex, &local);
 	
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
@@ -367,17 +419,27 @@ rename_folder (CamelStore *store, const char *old_name, const char *new_name, Ca
  * Rename a named folder to a new name.
  **/
 void
-camel_store_rename_folder (CamelStore *store, const char *old_name, const char *new_name, CamelException *ex)
+camel_store_rename_folder (CamelStore *store, const char *old_namein, const char *new_name, CamelException *ex)
 {
 	CamelFolder *folder;
 	int i, oldlen, namelen;
 	GPtrArray *folders;
+	char *old_name;
 
 	d(printf("store rename folder %s '%s' '%s'\n", ((CamelService *)store)->url->protocol, old_name, new_name));
 
-	if (strcmp(old_name, new_name) == 0)
+	if (strcmp(old_namein, new_name) == 0)
 		return;
 
+	if (((store->flags & CAMEL_STORE_VTRASH) && strcmp(old_namein, CAMEL_VTRASH_NAME) == 0)
+	    || ((store->flags & CAMEL_STORE_VJUNK) && strcmp(old_namein, CAMEL_VJUNK_NAME) == 0)) {
+		camel_exception_setv(ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
+				     _("Cannot rename folder: %s: Invalid operation"), old_namein);
+		return;
+	}
+
+	/* need to save this, since old_namein might be folder->full_name, which could go away */
+	old_name = g_strdup(old_namein);
 	oldlen = strlen(old_name);
 
 	CAMEL_STORE_LOCK(store, folder_lock);
@@ -418,15 +480,15 @@ camel_store_rename_folder (CamelStore *store, const char *old_name, const char *
 			folder = folders->pdata[i];
 
 			new = g_strdup_printf("%s%s", new_name, folder->full_name+strlen(old_name));
-			camel_object_bag_remove(store->folders, folder);
-			camel_object_bag_add(store->folders, new, folder);
+			camel_object_bag_rekey(store->folders, folder, new);
 			camel_folder_rename(folder, new);
+			g_free(new);
 
 			CAMEL_FOLDER_UNLOCK(folder, lock);
 			camel_object_unref(folder);
 		}
 
-		/* Emit changed signal */
+		/* Emit renamed signal */
 		if (store->flags & CAMEL_STORE_SUBSCRIPTIONS)
 			flags |= CAMEL_STORE_FOLDER_INFO_SUBSCRIBED;
 		
@@ -448,6 +510,7 @@ camel_store_rename_folder (CamelStore *store, const char *old_name, const char *
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 
 	g_ptr_array_free(folders, TRUE);
+	g_free(old_name);
 }
 
 
@@ -480,118 +543,35 @@ camel_store_get_inbox (CamelStore *store, CamelException *ex)
 	return folder;
 }
 
-static void
-trash_finalize (CamelObject *trash, gpointer event_data, gpointer user_data)
-{
-	CamelStore *store = CAMEL_STORE (user_data);
-	
-	store->vtrash = NULL;
-}
-
-static void
-junk_finalize (CamelObject *junk, gpointer event_data, gpointer user_data)
-{
-	CamelStore *store = CAMEL_STORE (user_data);
-	
-	store->vjunk = NULL;
-}
-
-/* FIXME: derive vjunk folder object from vee_folder */
-#include "camel-vee-store.h"
 static CamelFolder *
-camel_vjunk_folder_new (CamelStore *parent_store, const char *name)
+get_special(CamelStore *store, enum _camel_vtrash_folder_t type)
 {
-	CamelFolder *vjunk;
-	
-	vjunk = (CamelFolder *)camel_object_new (camel_vee_folder_get_type ());
-	vjunk->folder_flags |= CAMEL_FOLDER_IS_JUNK;
-	camel_vee_folder_construct (CAMEL_VEE_FOLDER (vjunk), parent_store, name,
-				    CAMEL_STORE_FOLDER_PRIVATE | CAMEL_STORE_FOLDER_CREATE | CAMEL_STORE_VEE_FOLDER_AUTO);
-	camel_vee_folder_set_expression((CamelVeeFolder *)vjunk, "(match-all (system-flag \"Junk\"))");
+	CamelFolder *folder;
+	GPtrArray *folders;
+	int i;
 
-	return vjunk;
-}
-
-static void
-init_trash_or_junk (CamelStore *store, CamelFolder *folder, void (*finalize) (CamelObject *o, gpointer event_data, gpointer user_data))
-{
-	if (folder) {
-		/* FIXME: this should probably use the object bag or another one ? ... */
-		/* attach to the finalise event of the vtrash/vjunk */
-		camel_object_hook_event (folder, "finalize", finalize, store);
-		
-		/* add all the pre-opened folders to the vtrash/vjunk */
-		if (store->folders) {
-			GPtrArray *folders = camel_object_bag_list(store->folders);
-			int i;
-
-			for (i=0;i<folders->len;i++) {
-				camel_vee_folder_add_folder (CAMEL_VEE_FOLDER (folder), (CamelFolder *)folders->pdata[i]);
-				camel_object_unref(folders->pdata[i]);
-			}
-			g_ptr_array_free(folders, TRUE);
-		}
+	folder = camel_vtrash_folder_new(store, type);
+	folders = camel_object_bag_list(store->folders);
+	for (i=0;i<folders->len;i++) {
+		if (!CAMEL_IS_VTRASH_FOLDER(folders->pdata[i]))
+			camel_vee_folder_add_folder((CamelVeeFolder *)folder, (CamelFolder *)folders->pdata[i]);
+		camel_object_unref(folders->pdata[i]);
 	}
-}
+	g_ptr_array_free(folders, TRUE);
 
-static void
-init_trash (CamelStore *store)
-{
-	if ((store->flags & CAMEL_STORE_VTRASH) == 0)
-		return;
-
-	store->vtrash = camel_vtrash_folder_new (store, CAMEL_VTRASH_NAME);
-	init_trash_or_junk (store, store->vtrash, trash_finalize);
-}
-
-static void
-init_junk (CamelStore *store)
-{
-	if ((store->flags & CAMEL_STORE_VJUNK) == 0)
-		return;
-
-	store->vjunk = camel_vjunk_folder_new (store, CAMEL_VJUNK_NAME);
-	init_trash_or_junk (store, store->vjunk, junk_finalize);
+	return folder;
 }
 
 static CamelFolder *
-get_trash (CamelStore *store, CamelException *ex)
+get_trash(CamelStore *store, CamelException *ex)
 {
-	if (store->vtrash) {
-		camel_object_ref (store->vtrash);
-		return store->vtrash;
-	} else {
-		CS_CLASS (store)->init_trash (store);
-		if (store->vtrash) {
-			/* We don't ref here because we don't want the
-                           store to own a ref on the trash folder */
-			/*camel_object_ref (store->vtrash);*/
-			return store->vtrash;
-		} else {
-			w(g_warning ("This store does not support vTrash."));
-			return NULL;
-		}
-	}
+	return get_special(store, CAMEL_VTRASH_FOLDER_TRASH);
 }
 
 static CamelFolder *
-get_junk (CamelStore *store, CamelException *ex)
+get_junk(CamelStore *store, CamelException *ex)
 {
-	if (store->vjunk) {
-		camel_object_ref (CAMEL_OBJECT (store->vjunk));
-		return store->vjunk;
-	} else {
-		CS_CLASS (store)->init_junk (store);
-		if (store->vjunk) {
-			/* We don't ref here because we don't want the
-                           store to own a ref on the junk folder */
-			/*camel_object_ref (CAMEL_OBJECT (store->vjunk));*/
-			return store->vjunk;
-		} else {
-			w(g_warning ("This store does not support vJunk."));
-			return NULL;
-		}
-	}
+	return get_special(store, CAMEL_VTRASH_FOLDER_JUNK);
 }
 
 /** 
@@ -607,11 +587,22 @@ camel_store_get_trash (CamelStore *store, CamelException *ex)
 {
 	CamelFolder *folder;
 
-	if ((store->flags & CAMEL_STORE_VTRASH) == 0)
+	if ((store->flags & CAMEL_STORE_VTRASH) == 0
+	    || store->folders == NULL)
 		return NULL;
 	
 	CAMEL_STORE_LOCK(store, folder_lock);
-	folder = CS_CLASS (store)->get_trash (store, ex);
+
+	folder = camel_object_bag_reserve(store->folders, CAMEL_VTRASH_NAME);
+	if (!folder) {
+		folder = CS_CLASS(store)->get_trash(store, ex);
+
+		if (folder)
+			camel_object_bag_add(store->folders, CAMEL_VTRASH_NAME, folder);
+		else
+			camel_object_bag_abort(store->folders, CAMEL_VTRASH_NAME);
+	}
+
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 	
 	return folder;
@@ -630,18 +621,29 @@ camel_store_get_junk (CamelStore *store, CamelException *ex)
 {
 	CamelFolder *folder;
 
-	if ((store->flags & CAMEL_STORE_VJUNK) == 0)
+	if ((store->flags & CAMEL_STORE_VJUNK) == 0
+	    || store->folders == NULL)
 		return NULL;
 	
 	CAMEL_STORE_LOCK(store, folder_lock);
-	folder = CS_CLASS (store)->get_junk (store, ex);
+
+	folder = camel_object_bag_reserve(store->folders, CAMEL_VJUNK_NAME);
+	if (!folder) {
+		folder = CS_CLASS(store)->get_junk(store, ex);
+
+		if (folder)
+			camel_object_bag_add(store->folders, CAMEL_VJUNK_NAME, folder);
+		else
+			camel_object_bag_abort(store->folders, CAMEL_VJUNK_NAME);
+	}
+
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 	
 	return folder;
 }
 
 static void
-store_sync (CamelStore *store, CamelException *ex)
+store_sync (CamelStore *store, int expunge, CamelException *ex)
 {
 	if (store->folders) {
 		GPtrArray *folders;
@@ -649,12 +651,15 @@ store_sync (CamelStore *store, CamelException *ex)
 		CamelException x;
 		int i;
 
+		/* we don't sync any vFolders, that is used to update certain vfolder queries mainly,
+		   and we're really only interested in storing/expunging the physical mails */
 		camel_exception_init(&x);
 		folders = camel_object_bag_list(store->folders);
 		for (i=0;i<folders->len;i++) {
 			folder = folders->pdata[i];
-			if (!camel_exception_is_set(&x))
-				camel_folder_sync(folder, FALSE, &x);
+			if (!CAMEL_IS_VEE_FOLDER(folder)
+			    && !camel_exception_is_set(&x))
+				camel_folder_sync(folder, expunge, &x);
 			camel_object_unref(folder);
 		}
 		camel_exception_xfer(ex, &x);
@@ -665,23 +670,22 @@ store_sync (CamelStore *store, CamelException *ex)
 /**
  * camel_store_sync:
  * @store: a CamelStore
+ * @expunge: do we expunge deleted messages too?
  * @ex: a CamelException
  *
  * Syncs any changes that have been made to the store object and its
  * folders with the real store.
  **/
 void
-camel_store_sync (CamelStore *store, CamelException *ex)
+camel_store_sync(CamelStore *store, int expunge, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_STORE (store));
 
-	CS_CLASS (store)->sync (store, ex);
+	CS_CLASS(store)->sync(store, expunge, ex);
 }
 
-
 static CamelFolderInfo *
-get_folder_info (CamelStore *store, const char *top,
-		 guint32 flags, CamelException *ex)
+get_folder_info (CamelStore *store, const char *top, guint32 flags, CamelException *ex)
 {
 	w(g_warning ("CamelStore::get_folder_info not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (store))));
@@ -690,8 +694,7 @@ get_folder_info (CamelStore *store, const char *top,
 }
 
 static void
-add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, const char *full_name,
-		  const char *url_base, gboolean unread_count)
+add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, const char *translated, gboolean unread_count)
 {
 	CamelFolderInfo *fi, *vinfo, *parent;
 	char *uri, *path;
@@ -700,14 +703,14 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
 	g_return_if_fail (info != NULL);
 	
 	parent = NULL;
-	for (fi = info; fi; fi = fi->sibling) {
+	for (fi = info; fi; fi = fi->next) {
 		if (!strcmp (fi->name, name))
 			break;
 		parent = fi;
 	}
 	
 	/* create our vTrash/vJunk URL */
-	url = camel_url_new (info->url, NULL);
+	url = camel_url_new (info->uri, NULL);
 	if (((CamelService *) store)->provider->url_flags & CAMEL_URL_FRAGMENT_IS_PATH) {
 		camel_url_set_fragment (url, name);
 	} else {
@@ -724,7 +727,7 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
 		vinfo = fi;
 		g_free (vinfo->full_name);
 		g_free (vinfo->name);
-		g_free (vinfo->url);
+		g_free (vinfo->uri);
 		g_free (vinfo->path);
 	} else {
 		/* There wasn't a Trash/Junk folder so create a new folder entry */
@@ -735,18 +738,36 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
 		vinfo->flags |= CAMEL_FOLDER_NOINFERIORS | CAMEL_FOLDER_SUBSCRIBED;
 		
 		/* link it into the right spot */
-		vinfo->sibling = parent->sibling;
-		parent->sibling = vinfo;
+		vinfo->next = parent->next;
+		parent->next = vinfo;
 	}
 	
 	/* Fill in the new fields */
-	vinfo->full_name = g_strdup (full_name);
-	vinfo->name = g_strdup (vinfo->full_name);
-	vinfo->url = g_strdup_printf ("%s:%s", url_base, uri);
+	vinfo->flags |= CAMEL_FOLDER_VIRTUAL|CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_VTRASH;
+	vinfo->full_name = g_strdup (name);
+	vinfo->name = g_strdup (translated);
+	vinfo->uri = uri;
 	if (!unread_count)
-		vinfo->unread_message_count = -1;
-	vinfo->path = g_strdup_printf ("/%s", vinfo->name);
-	g_free (uri);
+		vinfo->unread = -1;
+	vinfo->path = g_strdup_printf ("/%s", vinfo->full_name);
+}
+
+static void
+dump_fi(CamelFolderInfo *fi, int depth)
+{
+	char *s;
+
+	s = g_alloca(depth+1);
+	memset(s, ' ', depth);
+	s[depth] = 0;
+
+	while (fi) {
+		printf("%suri: %s\n", s, fi->uri);
+		printf("%sfull_name: %s\n", s, fi->full_name);
+		printf("%sflags: %08x\n", s, fi->flags);
+		dump_fi(fi->child, depth+2);
+		fi = fi->next;
+	}
 }
 
 /**
@@ -766,14 +787,15 @@ add_special_info (CamelStore *store, CamelFolderInfo *info, const char *name, co
  * the immediate subfolders of @top. If @flags includes
  * %CAMEL_STORE_FOLDER_INFO_FAST, the unread_message_count fields of
  * some or all of the structures may be set to -1, if the store cannot
- * determine that information quickly.
+ * determine that information quickly. If @flags includes
+ * %CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL, don't include special virtual
+ * folders (such as vTrash or vJunk).
  * 
  * Return value: a CamelFolderInfo tree, which must be freed with
  * camel_store_free_folder_info.
  **/
 CamelFolderInfo *
-camel_store_get_folder_info (CamelStore *store, const char *top,
-			     guint32 flags, CamelException *ex)
+camel_store_get_folder_info(CamelStore *store, const char *top, guint32 flags, CamelException *ex)
 {
 	CamelFolderInfo *info;
 	
@@ -781,22 +803,27 @@ camel_store_get_folder_info (CamelStore *store, const char *top,
 	g_return_val_if_fail ((store->flags & CAMEL_STORE_SUBSCRIPTIONS) ||
 			      !(flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED),
 			      NULL);
-	
-	CAMEL_STORE_LOCK(store, folder_lock);
+
 	info = CS_CLASS (store)->get_folder_info (store, top, flags, ex);
-	CAMEL_STORE_UNLOCK(store, folder_lock);
 	
-	if (info && (top == NULL || *top == '\0')) {
-		if (info->url && (store->flags & CAMEL_STORE_VTRASH))
-			add_special_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), "vtrash", FALSE);
-		if (info->url && (store->flags & CAMEL_STORE_VJUNK))
-			add_special_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), "vjunk", TRUE);
+	if (info && (top == NULL || *top == '\0') && (flags & CAMEL_STORE_FOLDER_INFO_NO_VIRTUAL) == 0) {
+		if (info->uri && (store->flags & CAMEL_STORE_VTRASH))
+			add_special_info (store, info, CAMEL_VTRASH_NAME, _("Trash"), FALSE);
+		if (info->uri && (store->flags & CAMEL_STORE_VJUNK))
+			add_special_info (store, info, CAMEL_VJUNK_NAME, _("Junk"), TRUE);
+	}
+
+	if (camel_debug_start("store:folder_info")) {
+		char *url = camel_url_to_string(((CamelService *)store)->url, CAMEL_URL_HIDE_ALL);
+		printf("Get folder info(%p:%s, '%s') =\n", store, url, top?top:"<null>");
+		g_free(url);
+		dump_fi(info, 2);
+		camel_debug_end();
 	}
 	
 	return info;
 }
 
-
 static void
 free_folder_info (CamelStore *store, CamelFolderInfo *fi)
 {
@@ -846,7 +873,6 @@ camel_store_free_folder_info_nop (CamelStore *store, CamelFolderInfo *fi)
 	;
 }
 
-
 /**
  * camel_folder_info_free:
  * @fi: the CamelFolderInfo
@@ -857,17 +883,16 @@ void
 camel_folder_info_free (CamelFolderInfo *fi)
 {
 	if (fi) {
-		camel_folder_info_free (fi->sibling);
+		camel_folder_info_free (fi->next);
 		camel_folder_info_free (fi->child);
 		g_free (fi->name);
 		g_free (fi->full_name);
 		g_free (fi->path);
-		g_free (fi->url);
+		g_free (fi->uri);
 		g_free (fi);
 	}
 }
 
-
 /**
  * camel_folder_info_build_path:
  * @fi: folder info
@@ -926,6 +951,9 @@ free_name(void *key, void *data, void *user)
  * %NULL urls to fill in gaps in the tree. The value of @short_names
  * is used in constructing the names of these intermediate folders.
  *
+ * NOTE: This is deprected, do not use this.
+ * FIXME: remove this/move it to imap, which is the only user of it now.
+ *
  * Return value: the top level of the tree of linked folder info.
  **/
 CamelFolderInfo *
@@ -953,6 +981,7 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 			name = fi->full_name;
 		if (*name == separator)
 			name++;
+
 		g_hash_table_insert (hash, g_strdup(name), fi);
 	}
 	
@@ -993,7 +1022,7 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 					pfi->name = g_strdup (pname);
 
 				/* FIXME: url's with fragments should have the fragment truncated, not path */
-				url = camel_url_new (fi->url, NULL);
+				url = camel_url_new (fi->uri, NULL);
 				sep = strrchr (url->path, separator);
 				if (sep)
 					*sep = '\0';
@@ -1004,20 +1033,16 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 
 				/* since this is a "fake" folder node, it is not selectable */
 				camel_url_set_param (url, "noselect", "yes");
-				pfi->url = camel_url_to_string (url, 0);
+				pfi->uri = camel_url_to_string (url, 0);
 				camel_url_free (url);
 
 				g_hash_table_insert (hash, pname, pfi);
 				g_ptr_array_add (folders, pfi);
 			}
-			tail = pfi->child;
-			if (tail == NULL) {
-				pfi->child = fi;
-			} else {
-				while (tail->sibling)
-					tail = tail->sibling;
-				tail->sibling = fi;
-			}
+			tail = (CamelFolderInfo *)&pfi->child;
+			while (tail->next)
+				tail = tail->next;
+			tail->next = fi;
 			fi->parent = pfi;
 		} else if (!top)
 			top = fi;
@@ -1035,7 +1060,7 @@ camel_folder_info_build (GPtrArray *folders, const char *namespace,
 			tail = fi;
 			top = fi;
 		} else {
-			tail->sibling = fi;
+			tail->next = fi;
 			tail = fi;
 		}
 	}
@@ -1049,16 +1074,17 @@ static CamelFolderInfo *folder_info_clone_rec(CamelFolderInfo *fi, CamelFolderIn
 
 	info = g_malloc(sizeof(*info));
 	info->parent = parent;
-	info->url = g_strdup(fi->url);
+	info->uri = g_strdup(fi->uri);
 	info->name = g_strdup(fi->name);
 	info->full_name = g_strdup(fi->full_name);
 	info->path = g_strdup(fi->path);
-	info->unread_message_count = fi->unread_message_count;
-
-	if (fi->sibling)
-		info->sibling = folder_info_clone_rec(fi->sibling, parent);
+	info->unread = fi->unread;
+	info->flags = fi->flags;
+	
+	if (fi->next)
+		info->next = folder_info_clone_rec(fi->next, parent);
 	else
-		info->sibling = NULL;
+		info->next = NULL;
 
 	if (fi->child)
 		info->child = folder_info_clone_rec(fi->child, info);
@@ -1083,9 +1109,8 @@ camel_store_supports_subscriptions (CamelStore *store)
 	return (store->flags & CAMEL_STORE_SUBSCRIPTIONS);
 }
 
-
 static gboolean
-folder_subscribed (CamelStore *store, const char *folder_name)
+folder_subscribed(CamelStore *store, const char *folder_name)
 {
 	w(g_warning ("CamelStore::folder_subscribed not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (store))));
@@ -1100,8 +1125,7 @@ folder_subscribed (CamelStore *store, const char *folder_name)
  * Return value: TRUE if folder is subscribed, FALSE if not.
  **/
 gboolean
-camel_store_folder_subscribed (CamelStore *store,
-			       const char *folder_name)
+camel_store_folder_subscribed(CamelStore *store, const char *folder_name)
 {
 	gboolean ret;
 
@@ -1118,7 +1142,7 @@ camel_store_folder_subscribed (CamelStore *store,
 }
 
 static void
-subscribe_folder (CamelStore *store, const char *folder_name, CamelException *ex)
+subscribe_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	w(g_warning ("CamelStore::subscribe_folder not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (store))));
@@ -1130,9 +1154,7 @@ subscribe_folder (CamelStore *store, const char *folder_name, CamelException *ex
  * @folder_name: the folder to subscribe to.
  **/
 void
-camel_store_subscribe_folder (CamelStore *store,
-			      const char *folder_name,
-			      CamelException *ex)
+camel_store_subscribe_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (store->flags & CAMEL_STORE_SUBSCRIPTIONS);
@@ -1145,57 +1167,39 @@ camel_store_subscribe_folder (CamelStore *store,
 }
 
 static void
-unsubscribe_folder (CamelStore *store, const char *folder_name, CamelException *ex)
+unsubscribe_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 {
 	w(g_warning ("CamelStore::unsubscribe_folder not implemented for `%s'",
 		     camel_type_to_name (CAMEL_OBJECT_GET_TYPE (store))));
 }
 
-
 /**
  * camel_store_unsubscribe_folder: marks a folder as unsubscribed.
  * @store: a CamelStore
  * @folder_name: the folder to unsubscribe from.
  **/
 void
-camel_store_unsubscribe_folder (CamelStore *store,
-				const char *folder_name,
-				CamelException *ex)
+camel_store_unsubscribe_folder(CamelStore *store, const char *folder_name, CamelException *ex)
 {
-	CamelFolder *folder = NULL;
+	CamelException local;
 
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (store->flags & CAMEL_STORE_SUBSCRIPTIONS);
 
+	camel_exception_init(&local);
+
 	CAMEL_STORE_LOCK(store, folder_lock);
 
-	/* NB: Note similarity of this code to delete_folder */
-
-	/* if we deleted a folder, force it out of the cache, and also out of the vtrash/vjunk if setup */
-	if (store->folders) {
-		folder = camel_object_bag_get(store->folders, folder_name);
-		if (folder) {
-			if (store->vtrash)
-				camel_vee_folder_remove_folder((CamelVeeFolder *)store->vtrash, folder);
-			if (store->vjunk)
-				camel_vee_folder_remove_folder((CamelVeeFolder *)store->vjunk, folder);
-			camel_folder_delete (folder);
-		}
-	}
-
 	CS_CLASS (store)->unsubscribe_folder (store, folder_name, ex);
 
-	if (folder) {
-		if (store->folders)
-			camel_object_bag_remove(store->folders, folder);
-
-		camel_object_unref(folder);
-	}
+	if (!camel_exception_is_set(&local))
+		cs_delete_cached_folder(store, folder_name);
+	else
+		camel_exception_xfer(ex, &local);
 
 	CAMEL_STORE_UNLOCK(store, folder_lock);
 }
 
-
 static void
 noop (CamelStore *store, CamelException *ex)
 {
@@ -1203,7 +1207,6 @@ noop (CamelStore *store, CamelException *ex)
 	;
 }
 
-
 /**
  * camel_store_noop:
  * @store: CamelStore
@@ -1218,11 +1221,52 @@ camel_store_noop (CamelStore *store, CamelException *ex)
 }
 
 
-/* Return true if these uri's refer to the same object */
-gboolean
-camel_store_uri_cmp(CamelStore *store, const char *uria, const char *urib)
+/**
+ * camel_store_folder_uri_equal:
+ * @store: CamelStore
+ * @uri0: a uri
+ * @uri1: another uri
+ *
+ * Compares 2 folder uris to check that they are equal.
+ *
+ * Returns %TRUE if they are equal or %FALSE otherwise.
+ **/
+int
+camel_store_folder_uri_equal (CamelStore *store, const char *uri0, const char *uri1)
 {
-	g_assert(CAMEL_IS_STORE(store));
-
-	return CS_CLASS(store)->compare_folder_name(uria, urib);
+	CamelProvider *provider;
+	CamelURL *url0, *url1;
+	int equal;
+	
+	g_return_val_if_fail (CAMEL_IS_STORE (store), FALSE);
+	g_return_val_if_fail (uri0 && uri1, FALSE);
+	
+	provider = ((CamelService *) store)->provider;
+	
+	if (!(url0 = camel_url_new (uri0, NULL)))
+		return FALSE;
+	
+	if (!(url1 = camel_url_new (uri1, NULL))) {
+		camel_url_free (url0);
+		return FALSE;
+	}
+	
+	if ((equal = provider->url_equal (url0, url1))) {
+		const char *name0, *name1;
+		
+		if (provider->url_flags & CAMEL_URL_FRAGMENT_IS_PATH) {
+			name0 = url0->fragment;
+			name1 = url1->fragment;
+		} else {
+			name0 = url0->path[0] == '/' ? url0->path + 1 : url0->path;
+			name1 = url1->path[0] == '/' ? url1->path + 1 : url1->path;
+		}
+		
+		equal = CS_CLASS (store)->compare_folder_name (name0, name1);
+	}
+	
+	camel_url_free (url0);
+	camel_url_free (url1);
+	
+	return equal;
 }
