@@ -65,6 +65,34 @@ static CamelDiscoFolderClass *parent_class = NULL;
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 #define CNNTPS_CLASS(so) CAMEL_STORE_CLASS (CAMEL_OBJECT_GET_CLASS(so))
 
+static void
+nntp_folder_refresh_info_online (CamelFolder *folder, CamelException *ex)
+{
+	CamelNNTPStore *nntp_store;
+	CamelFolderChangeInfo *changes = NULL;
+	CamelNNTPFolder *nntp_folder;
+	
+	nntp_store = (CamelNNTPStore *) folder->parent_store;
+	nntp_folder = (CamelNNTPFolder *) folder;
+	
+	CAMEL_NNTP_STORE_LOCK(nntp_store, command_lock);
+	
+	if (camel_nntp_summary_check ((CamelNNTPSummary *) folder->summary, nntp_folder->changes, ex) != -1)
+		camel_folder_summary_save (folder->summary);
+	
+	if (camel_folder_change_info_changed(nntp_folder->changes)) {
+		changes = nntp_folder->changes;
+		nntp_folder->changes = camel_folder_change_info_new();
+	}
+	
+	CAMEL_NNTP_STORE_UNLOCK(nntp_store, command_lock);
+	
+	if (changes) {
+		camel_object_trigger_event ((CamelObject *) folder, "folder_changed", changes);
+		camel_folder_change_info_free (changes);
+	}
+}
+
 static void
 nntp_folder_sync_online (CamelFolder *folder, CamelException *ex)
 {
@@ -150,18 +178,21 @@ nntp_folder_cache_message (CamelDiscoFolder *disco_folder, const char *uid, Came
 {
 	CamelNNTPStore *nntp_store = (CamelNNTPStore *)((CamelFolder *) disco_folder)->parent_store;
 	CamelStream *stream;
-	const char *msgid;
-	
-	if (!(msgid = strchr (uid, ','))) {
+	char *article, *msgid;
+
+	article = alloca(strlen(uid)+1);
+	strcpy(article, uid);
+	msgid = strchr(uid, ',');
+	if (!msgid) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Internal error: uid in invalid format: %s"), uid);
 		return;
 	}
-	msgid++;
+	*msgid++ = 0;
 	
 	CAMEL_NNTP_STORE_LOCK(nntp_store, command_lock);
 	
-	stream = nntp_folder_download_message ((CamelNNTPFolder *) disco_folder, msgid, ex);
+	stream = nntp_folder_download_message ((CamelNNTPFolder *) disco_folder, article, ex);
 	if (stream) {
 		camel_object_unref (stream);
 	} else {
@@ -182,20 +213,22 @@ nntp_folder_get_message (CamelFolder *folder, const char *uid, CamelException *e
 	CamelNNTPFolder *nntp_folder;
 	CamelStream *stream = NULL;
 	char *line = NULL;
-	const char *msgid;
-	
+	char *article, *msgid;
+
 	nntp_store = (CamelNNTPStore *) folder->parent_store;
 	nntp_folder = (CamelNNTPFolder *) folder;
 	
 	CAMEL_NNTP_STORE_LOCK(nntp_store, command_lock);
-	
+
+	article = alloca(strlen(uid)+1);
+	strcpy(article, uid);
 	msgid = strchr (uid, ',');
 	if (msgid == NULL) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Internal error: uid in invalid format: %s"), uid);
 		goto fail;
 	}
-	msgid++;
+	*msgid++ = 0;
 	
 	/* Lookup in cache, NEWS is global messageid's so use a global cache path */
 	stream = camel_data_cache_get (nntp_store->cache, "cache", msgid, NULL);
@@ -205,8 +238,8 @@ nntp_folder_get_message (CamelFolder *folder, const char *uid, CamelException *e
 					     _("This message is not currently available"));
 			goto fail;
 		}
-		
-		stream = nntp_folder_download_message (nntp_folder, msgid, ex);
+
+		stream = nntp_folder_download_message (nntp_folder, article, ex);
 		if (stream == NULL)
 			goto fail;
 	}
@@ -259,7 +292,7 @@ static GPtrArray*
 nntp_folder_search_by_expression (CamelFolder *folder, const char *expression, CamelException *ex)
 {
 	CamelNNTPFolder *nntp_folder = CAMEL_NNTP_FOLDER (folder);
-	GPtrArray *matches, *summary;
+	GPtrArray *matches;
 	
 	CAMEL_NNTP_FOLDER_LOCK(nntp_folder, search_lock);
 	
@@ -267,15 +300,10 @@ nntp_folder_search_by_expression (CamelFolder *folder, const char *expression, C
 		nntp_folder->search = camel_folder_search_new ();
 	
 	camel_folder_search_set_folder (nntp_folder->search, folder);
-	summary = camel_folder_get_summary (folder);
-	camel_folder_search_set_summary (nntp_folder->search, summary);
-	
-	matches = camel_folder_search_execute_expression (nntp_folder->search, expression, ex);
+	matches = camel_folder_search_search(nntp_folder->search, expression, NULL, ex);
 	
 	CAMEL_NNTP_FOLDER_UNLOCK(nntp_folder, search_lock);
 	
-	camel_folder_free_summary (folder, summary);
-	
 	return matches;
 }
 
@@ -283,22 +311,10 @@ static GPtrArray *
 nntp_folder_search_by_uids (CamelFolder *folder, const char *expression, GPtrArray *uids, CamelException *ex)
 {
 	CamelNNTPFolder *nntp_folder = (CamelNNTPFolder *) folder;
-	GPtrArray *summary, *matches;
-	int i;
-	
-	/* NOTE: could get away without the search lock by creating a new
-	   search object each time */
-	
-	summary = g_ptr_array_new ();
-	for (i = 0; i < uids->len; i++) {
-		CamelMessageInfo *info;
-		
-		if ((info = camel_folder_get_message_info (folder, uids->pdata[i])))
-			g_ptr_array_add (summary, info);
-	}
-	
-	if (summary->len == 0)
-		return summary;
+	GPtrArray *matches;
+
+	if (uids->len == 0)
+		return g_ptr_array_new();
 	
 	CAMEL_NNTP_FOLDER_LOCK(folder, search_lock);
 	
@@ -306,17 +322,10 @@ nntp_folder_search_by_uids (CamelFolder *folder, const char *expression, GPtrArr
 		nntp_folder->search = camel_folder_search_new ();
 	
 	camel_folder_search_set_folder (nntp_folder->search, folder);
-	camel_folder_search_set_summary (nntp_folder->search, summary);
-	
-	matches = camel_folder_search_execute_expression (nntp_folder->search, expression, ex);
+	matches = camel_folder_search_search(nntp_folder->search, expression, uids, ex);
 	
 	CAMEL_NNTP_FOLDER_UNLOCK(folder, search_lock);
 	
-	for (i = 0; i < summary->len; i++)
-		camel_folder_free_message_info (folder, summary->pdata[i]);
-	
-	g_ptr_array_free (summary, TRUE);
-	
 	return matches;
 }
 
@@ -505,6 +514,7 @@ nntp_folder_class_init (CamelNNTPFolderClass *camel_nntp_folder_class)
 	camel_disco_folder_class->transfer_online = nntp_folder_transfer_message;
 	camel_disco_folder_class->transfer_resyncing = nntp_folder_transfer_message;
 	camel_disco_folder_class->transfer_offline = nntp_folder_transfer_message;
+	camel_disco_folder_class->refresh_info_online = nntp_folder_refresh_info_online;
 	
 	camel_folder_class->set_message_flags = nntp_folder_set_message_flags;
 	camel_folder_class->get_message = nntp_folder_get_message;
@@ -602,6 +612,11 @@ camel_nntp_folder_new (CamelStore *parent, const char *folder_name, CamelExcepti
 	nntp_folder->storage_path = g_build_filename (root, folder->full_name, NULL);
 	g_free (root);
 	
+	root = g_strdup_printf ("%s.cmeta", nntp_folder->storage_path);
+	camel_object_set(nntp_folder, NULL, CAMEL_OBJECT_STATE_FILE, root, NULL);
+	camel_object_state_read(nntp_folder);
+	g_free(root);
+
 	folder->summary = (CamelFolderSummary *) camel_nntp_summary_new (nntp_folder);
 	camel_folder_summary_load (folder->summary);
 	
