@@ -21,7 +21,6 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -54,6 +53,8 @@
 #include <camel/camel-mime-part.h>
 
 #include "e-util/e-gui-utils.h"
+#include "e-util/e-icon-factory.h"
+#include "widgets/misc/e-error.h"
 
 #define ICON_WIDTH 64
 #define ICON_SEPARATORS " /-_"
@@ -170,22 +171,15 @@ add_from_file (EMsgComposerAttachmentBar *bar,
 	       const char *disposition)
 {
 	EMsgComposerAttachment *attachment;
-	EMsgComposer *composer;
 	CamelException ex;
-	GtkWidget *dialog;
 	
 	camel_exception_init (&ex);
 	attachment = e_msg_composer_attachment_new (file_name, disposition, &ex);
 	if (attachment) {
 		add_common (bar, attachment);
 	} else {
-		composer = E_MSG_COMPOSER (gtk_widget_get_toplevel (GTK_WIDGET (bar)));
-		dialog = gtk_message_dialog_new(GTK_WINDOW(composer),
-						GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-						"%s", camel_exception_get_description (&ex));
-		gtk_dialog_run(GTK_DIALOG(dialog));
-		gtk_widget_destroy(dialog);
+		e_error_run((GtkWindow *)gtk_widget_get_toplevel((GtkWidget *)bar), "mail-composer:no-attach",
+			    file_name, camel_exception_get_description(&ex), NULL);
 		camel_exception_clear (&ex);
 	}
 }
@@ -232,17 +226,14 @@ update (EMsgComposerAttachmentBar *bar)
 		EMsgComposerAttachment *attachment;
 		CamelContentType *content_type;
 		char *size_string, *label;
-		GdkPixbuf *pixbuf;
-		gboolean image;
+		GdkPixbuf *pixbuf = NULL;
 		const char *desc;
 		
 		attachment = p->data;
 		content_type = camel_mime_part_get_content_type (attachment->body);
 		/* Get the image out of the attachment 
 		   and create a thumbnail for it */
-		image = camel_content_type_is (content_type, "image", "*");
-		
-		if (image && attachment->pixbuf_cache == NULL) {
+		if (camel_content_type_is(content_type, "image", "*") && attachment->pixbuf_cache == NULL) {
 			CamelDataWrapper *wrapper;
 			CamelStreamMem *mstream;
 			GdkPixbufLoader *loader;
@@ -284,9 +275,11 @@ update (EMsgComposerAttachmentBar *bar)
 					 width,
 					 height,
 					 GDK_INTERP_BILINEAR);
+				pixbuf = attachment->pixbuf_cache;
+				g_object_ref(pixbuf);
 			} else {
+				pixbuf = NULL;
 				g_warning ("GdkPixbufLoader Error");
-				image = FALSE;
 			}
 			
 			/* Destroy everything */
@@ -308,17 +301,22 @@ update (EMsgComposerAttachmentBar *bar)
 		} else
 			label = g_strdup (desc);
 		
-		if (image) {
-			gnome_icon_list_append_pixbuf (icon_list, attachment->pixbuf_cache, NULL, label);
-		} else {
+		if (pixbuf == NULL) {
 			char *mime_type;
 			
 			mime_type = camel_content_type_simple (content_type);
 			pixbuf = e_icon_for_mime_type (mime_type, 48);
+			if (pixbuf == NULL) {
+				g_warning("cannot find icon for mime type %s (installation problem?)", mime_type);
+				/* stock_attach would be better, but its fugly scaled up */
+				pixbuf = e_icon_factory_get_icon("stock_unknown", E_ICON_SIZE_DIALOG);
+			}
 			g_free (mime_type);
+		}
+
+		if (pixbuf) {
 			gnome_icon_list_append_pixbuf (icon_list, pixbuf, NULL, label);
-			if (pixbuf)
-				g_object_unref (pixbuf);
+			g_object_unref(pixbuf);
 		}
 		
 		g_free (label);
@@ -743,29 +741,31 @@ e_msg_composer_attachment_bar_new (GtkAdjustment *adj)
 	return GTK_WIDGET (new);
 }
 
-static const char *
+static char *
 get_default_charset (void)
 {
 	GConfClient *gconf;
-	const char *charset;
-	char *buf;
+	const char *locale;
+	char *charset;
 	
 	gconf = gconf_client_get_default ();
-	buf = gconf_client_get_string (gconf, "/apps/evolution/mail/composer/charset", NULL);
-	if (buf && buf[0] == '\0') {
-		g_free (buf);
-		buf = NULL;
+	charset = gconf_client_get_string (gconf, "/apps/evolution/mail/composer/charset", NULL);
+	
+	if (!charset || charset[0] == '\0') {
+		g_free (charset);
+		charset = gconf_client_get_string (gconf, "/apps/evolution/mail/format/charset", NULL);
+		if (charset && charset[0] == '\0') {
+			g_free (charset);
+			charset = NULL;
+		}
 	}
 	
 	g_object_unref (gconf);
 	
-	if (buf != NULL) {
-		charset = e_iconv_charset_name (buf);
-		g_free (buf);
-	} else
-		charset = e_iconv_locale_charset ();
+	if (!charset && (locale = e_iconv_locale_charset ()))
+		charset = g_strdup (locale);
 	
-	return charset;
+	return charset ? charset : g_strdup ("us-ascii");
 }
 
 static void
@@ -786,6 +786,7 @@ attach_to_multipart (CamelMultipart *multipart,
 			CamelMimeFilterBestenc *bestenc;
 			CamelStream *stream;
 			const char *charset;
+			char *buf = NULL;
 			char *type;
 			
 			charset = camel_content_type_param (content_type, "charset");
@@ -808,7 +809,7 @@ attach_to_multipart (CamelMultipart *multipart,
 				default_charset = "us-ascii";
 			} else if (!charset) {
 				if (!default_charset)
-					default_charset = get_default_charset ();
+					default_charset = buf = get_default_charset ();
 				
 				/* FIXME: We should really check that this fits within the
                                    default_charset and if not find one that does and/or
@@ -821,6 +822,7 @@ attach_to_multipart (CamelMultipart *multipart,
 				type = camel_content_type_format (content_type);
 				camel_mime_part_set_content_type (attachment->body, type);
 				g_free (type);
+				g_free (buf);
 			}
 			
 			camel_object_unref (bestenc);
