@@ -35,6 +35,7 @@
 #include <gtk/gtk.h>
 #include <libgnome/gnome-i18n.h>
 #include <gal/util/e-xml-utils.h>
+#include "widgets/misc/e-error.h"
 
 #include "rule-context.h"
 #include "filter-rule.h"
@@ -149,6 +150,8 @@ rule_context_init(RuleContext *rc)
 	
 	rc->part_set_map = g_hash_table_new(g_str_hash, g_str_equal);
 	rc->rule_set_map = g_hash_table_new(g_str_hash, g_str_equal);
+
+	rc->flags = RULE_CONTEXT_GROUPING;
 }
 
 static void
@@ -184,12 +187,7 @@ rule_context_finalise(GObject *obj)
 	g_list_free(rc->parts);
 	g_list_foreach(rc->rules, (GFunc)g_object_unref, NULL);
 	g_list_free(rc->rules);
-	
-	if (rc->system)
-		xmlFreeDoc(rc->system);
-	if (rc->user)
-		xmlFreeDoc(rc->user);
-	
+
 	g_free(rc->priv);
 	
 	G_OBJECT_CLASS(parent_class)->finalize(obj);
@@ -288,6 +286,7 @@ static int
 load(RuleContext *rc, const char *system, const char *user)
 {
 	xmlNodePtr set, rule, root;
+	xmlDocPtr systemdoc, userdoc;
 	struct _part_set_map *part_map;
 	struct _rule_set_map *rule_map;
 	struct stat st;
@@ -296,24 +295,23 @@ load(RuleContext *rc, const char *system, const char *user)
 	
 	d(printf("loading rules %s %s\n", system, user));
 	
-	rc->system = xmlParseFile(system);
-	if (rc->system == NULL) {
+	systemdoc = xmlParseFile(system);
+	if (systemdoc == NULL) {
 		rule_context_set_error(rc, g_strdup_printf("Unable to load system rules '%s': %s",
 							     system, g_strerror(errno)));
 		return -1;
 	}
 
-	root = xmlDocGetRootElement(rc->system);
+	root = xmlDocGetRootElement(systemdoc);
 	if (root == NULL || strcmp(root->name, "filterdescription")) {
 		rule_context_set_error(rc, g_strdup_printf("Unable to load system rules '%s': Invalid format", system));
-		xmlFreeDoc(rc->system);
-		rc->system = NULL;
+		xmlFreeDoc(systemdoc);
 		return -1;
 	}
 	/* doesn't matter if this doens't exist */
-	rc->user = NULL;
+	userdoc = NULL;
 	if (stat (user, &st) != -1 && S_ISREG (st.st_mode))
-		rc->user = xmlParseFile(user);
+		userdoc = xmlParseFile(user);
 	
 	/* now parse structure */
 	/* get rule parts */
@@ -360,8 +358,8 @@ load(RuleContext *rc, const char *system, const char *user)
 	}
 	
 	/* now load actual rules */
-	if (rc->user) {
-		root = xmlDocGetRootElement(rc->user);
+	if (userdoc) {
+		root = xmlDocGetRootElement(userdoc);
 		set = root?root->children:NULL;
 		while (set) {
 			d(printf("set name = %s\n", set->name));
@@ -387,6 +385,9 @@ load(RuleContext *rc, const char *system, const char *user)
 			set = set->next;
 		}
 	}
+
+	xmlFreeDoc(userdoc);
+	xmlFreeDoc(systemdoc);
 	
 	return 0;
 }
@@ -461,7 +462,7 @@ rule_context_revert(RuleContext *rc, const char *user)
 {
 	g_assert(rc);
 	
-	d(printf("rule_context: restoring %s %s\n", user));
+	d(printf("rule_context: restoring %s\n", user));
 	
 	return RULE_CONTEXT_GET_CLASS(rc)->revert(rc, user);
 }
@@ -514,7 +515,7 @@ revert(RuleContext *rc, const char *user)
 	
 	rule_context_set_error(rc, NULL);
 	
-	d(printf("restoring rules %s %s\n", user));
+	d(printf("restoring rules %s\n", user));
 	
 	userdoc = xmlParseFile(user);
 	if (userdoc == NULL)
@@ -538,7 +539,7 @@ revert(RuleContext *rc, const char *user)
 	}
 	
 	/* make what we have, match what we load */
-	set = xmlDocGetRootElement(rc->user);
+	set = xmlDocGetRootElement(userdoc);
 	set = set?set->children:NULL;
 	while (set) {
 		d(printf("set name = %s\n", set->name));
@@ -669,7 +670,7 @@ rule_context_add_rule(RuleContext *rc, FilterRule *new)
 static void
 new_rule_response(GtkWidget *dialog, int button, RuleContext *context)
 {
-	if (button == GTK_RESPONSE_ACCEPT) {
+	if (button == GTK_RESPONSE_OK) {
 		FilterRule *rule = g_object_get_data((GObject *) dialog, "rule");
 		char *user = g_object_get_data((GObject *) dialog, "path");
 		
@@ -679,13 +680,7 @@ new_rule_response(GtkWidget *dialog, int button, RuleContext *context)
 		}
 
 		if (rule_context_find_rule (context, rule->name, rule->source)) {
-			dialog = gtk_message_dialog_new ((GtkWindow *) dialog, GTK_DIALOG_DESTROY_WITH_PARENT,
-							 GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-							 _("Rule name '%s' is not unique, choose another."),
-							 rule->name);
-
-			gtk_dialog_run ((GtkDialog *) dialog);
-			gtk_widget_destroy (dialog);
+			e_error_run((GtkWindow *)dialog, "filter:bad-name-notunique", rule->name, NULL);
 
 			return;
 		}
@@ -716,9 +711,10 @@ rule_context_add_rule_gui(RuleContext *rc, FilterRule *rule, const char *title,
 	
 	dialog =(GtkDialog *) gtk_dialog_new();
 	gtk_dialog_add_buttons(dialog,
-				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
-				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-				NULL);
+			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+			       GTK_STOCK_OK, GTK_RESPONSE_OK,
+			       NULL);
+	gtk_dialog_set_has_separator (dialog, FALSE);
 	
 	gtk_window_set_title((GtkWindow *) dialog, title);
 	gtk_window_set_default_size((GtkWindow *) dialog, 600, 400);
