@@ -31,9 +31,14 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/stat.h>
+
+#include <libxml/parser.h>
 
 #include <e-util/e-mktemp.h>
 
+#include <gal/util/e-xml-utils.h>
+
 #include <camel/camel-file-utils.h>
 
 #include "mail-config.h"
@@ -41,12 +46,18 @@
 #include "mail-tools.h"
 #include "mail-mt.h"
 
+/* sigh, these 2 only needed for outbox total count checking - a mess */
+#include "mail-component.h"
+#include "mail-folder-cache.h"
+
 #include "em-utils.h"
 
+#include <camel/camel-folder.h>
+
 #include "em-marshal.h"
 #include "em-folder-tree-model.h"
 
-
+#define u(x) x			/* unread count debug */
 #define d(x) x
 
 static GType col_types[] = {
@@ -55,11 +66,11 @@ static GType col_types[] = {
 	G_TYPE_STRING,   /* path */
 	G_TYPE_STRING,   /* uri */
 	G_TYPE_UINT,     /* unread count */
+	G_TYPE_UINT,     /* flags */
 	G_TYPE_BOOLEAN,  /* is a store node */
 	G_TYPE_BOOLEAN,  /* has not-yet-loaded subfolders */
 };
 
-
 /* GObject virtual method overrides */
 static void em_folder_tree_model_class_init (EMFolderTreeModelClass *klass);
 static void em_folder_tree_model_init (EMFolderTreeModel *model);
@@ -67,40 +78,20 @@ static void em_folder_tree_model_finalize (GObject *obj);
 
 /* interface init methods */
 static void tree_model_iface_init (GtkTreeModelIface *iface);
-static void tree_drag_dest_iface_init (GtkTreeDragDestIface *iface);
-static void tree_drag_source_iface_init (GtkTreeDragSourceIface *iface);
-
-/* drag & drop iface methods */
-static gboolean model_drag_data_received (GtkTreeDragDest *drag_dest,
-					  GtkTreePath *dest_path,
-					  GtkSelectionData *selection);
-static gboolean model_row_drop_possible  (GtkTreeDragDest *drag_dest,
-					  GtkTreePath *dest_path,
-					  GtkSelectionData *selection);
-static gboolean model_row_draggable      (GtkTreeDragSource *drag_source,
-					  GtkTreePath *src_path);
-static gboolean model_drag_data_get      (GtkTreeDragSource *drag_source,
-					  GtkTreePath *src_path,
-					  GtkSelectionData *selection);
-static gboolean model_drag_data_delete   (GtkTreeDragSource *drag_source,
-					  GtkTreePath *src_path);
+static void tree_sortable_iface_init (GtkTreeSortableIface *iface);
 
+static void account_changed (EAccountList *accounts, EAccount *account, gpointer user_data);
+static void account_removed (EAccountList *accounts, EAccount *account, gpointer user_data);
 
 enum {
 	LOADING_ROW,
-	DRAG_DATA_RECEIVED,
-	ROW_DROP_POSSIBLE,
-	ROW_DRAGGABLE,
-	DRAG_DATA_GET,
-	DRAG_DATA_DELETE,
+	LOADED_ROW,
+	FOLDER_ADDED,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0, };
-
-
-static GtkTreeStore *parent_class = NULL;
-
+static GtkTreeStoreClass *parent_class = NULL;
 
 GType
 em_folder_tree_model_get_type (void)
@@ -124,13 +115,8 @@ em_folder_tree_model_get_type (void)
 			NULL,
 			NULL
 		};
-		static const GInterfaceInfo drag_dest_info = {
-			(GInterfaceInitFunc) tree_drag_dest_iface_init,
-			NULL,
-			NULL
-		};
-		static const GInterfaceInfo drag_source_info = {
-			(GInterfaceInitFunc) tree_drag_source_iface_init,
+		static const GInterfaceInfo sortable_info = {
+			(GInterfaceInitFunc) tree_sortable_iface_init,
 			NULL,
 			NULL
 		};
@@ -139,10 +125,8 @@ em_folder_tree_model_get_type (void)
 		
 		g_type_add_interface_static (type, GTK_TYPE_TREE_MODEL,
 					     &tree_model_info);
-		g_type_add_interface_static (type, GTK_TYPE_TREE_DRAG_DEST,
-					     &drag_dest_info);
-		g_type_add_interface_static (type, GTK_TYPE_TREE_DRAG_SOURCE,
-					     &drag_source_info);
+		g_type_add_interface_static (type, GTK_TYPE_TREE_SORTABLE,
+					     &sortable_info);
 	}
 	
 	return type;
@@ -169,6 +153,74 @@ em_folder_tree_model_class_init (EMFolderTreeModelClass *klass)
 			      G_TYPE_NONE, 2,
 			      G_TYPE_POINTER,
 			      G_TYPE_POINTER);
+	
+	signals[LOADED_ROW] =
+		g_signal_new ("loaded-row",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (EMFolderTreeModelClass, loaded_row),
+			      NULL, NULL,
+			      em_marshal_VOID__POINTER_POINTER,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_POINTER,
+			      G_TYPE_POINTER);
+	
+	signals[FOLDER_ADDED] =
+		g_signal_new ("folder-added",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (EMFolderTreeModelClass, folder_added),
+			      NULL, NULL,
+			      em_marshal_VOID__STRING_STRING,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
+}
+
+static int
+sort_cb (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
+{
+	extern CamelStore *vfolder_store;
+	char *aname, *bname;
+	CamelStore *store;
+	gboolean is_store;
+	
+	gtk_tree_model_get (model, a, COL_BOOL_IS_STORE, &is_store,
+			    COL_POINTER_CAMEL_STORE, &store,
+			    COL_STRING_DISPLAY_NAME, &aname, -1);
+	gtk_tree_model_get (model, b, COL_STRING_DISPLAY_NAME, &bname, -1);
+	
+	if (is_store) {
+		/* On This Computer is always first and VFolders is always last */
+		if (!strcmp (aname, _("On This Computer")))
+			return -1;
+		if (!strcmp (bname, _("On This Computer")))
+			return 1;
+		if (!strcmp (aname, _("VFolders")))
+			return 1;
+		if (!strcmp (bname, _("VFolders")))
+			return -1;
+	} else if (store == vfolder_store) {
+		/* UNMATCHED is always last */
+		if (aname && !strcmp (aname, _("UNMATCHED")))
+			return 1;
+		if (bname && !strcmp (bname, _("UNMATCHED")))
+			return -1;
+	} else {
+		/* Inbox is always first */
+		if (aname && (!strcmp (aname, "INBOX") || !strcmp (aname, _("Inbox"))))
+			return -1;
+		if (bname && (!strcmp (bname, "INBOX") || !strcmp (bname, _("Inbox"))))
+			return 1;
+	}
+	
+	if (aname == NULL) {
+		if (bname == NULL)
+			return 0;
+	} else if (bname == NULL)
+		return 1;
+	
+	return g_utf8_collate (aname, bname);
 }
 
 static void
@@ -176,7 +228,13 @@ em_folder_tree_model_init (EMFolderTreeModel *model)
 {
 	model->store_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 	model->uri_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	model->expanded = g_hash_table_new (g_str_hash, g_str_equal);
+	
+	gtk_tree_sortable_set_default_sort_func ((GtkTreeSortable *) model, sort_cb, NULL, NULL);
+	
+	model->accounts = mail_config_get_accounts ();
+	model->account_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	model->account_changed_id = g_signal_connect (model->accounts, "account-changed", G_CALLBACK (account_changed), model);
+	model->account_removed_id = g_signal_connect (model->accounts, "account-removed", G_CALLBACK (account_removed), model);
 }
 
 static void
@@ -217,28 +275,24 @@ uri_hash_free (gpointer key, gpointer value, gpointer user_data)
 	gtk_tree_row_reference_free (value);
 }
 
-static gboolean
-expanded_free (gpointer key, gpointer value, gpointer user_data)
-{
-	g_free (key);
-	return TRUE;
-}
-
 static void
 em_folder_tree_model_finalize (GObject *obj)
 {
 	EMFolderTreeModel *model = (EMFolderTreeModel *) obj;
 	
+	g_free (model->filename);
+	if (model->expanded)
+		xmlFreeDoc (model->expanded);
+	
 	g_hash_table_foreach (model->store_hash, store_hash_free, NULL);
 	g_hash_table_destroy (model->store_hash);
 	
 	g_hash_table_foreach (model->uri_hash, uri_hash_free, NULL);
 	g_hash_table_destroy (model->uri_hash);
 	
-	g_hash_table_foreach (model->expanded, (GHFunc) expanded_free, NULL);
-	g_hash_table_destroy (model->expanded);
-	
-	g_free (model->filename);
+	g_hash_table_destroy (model->account_hash);
+	g_signal_handler_disconnect (model->accounts, model->account_changed_id);
+	g_signal_handler_disconnect (model->accounts, model->account_removed_id);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
@@ -251,459 +305,36 @@ tree_model_iface_init (GtkTreeModelIface *iface)
 }
 
 static void
-tree_drag_dest_iface_init (GtkTreeDragDestIface *iface)
+tree_sortable_iface_init (GtkTreeSortableIface *iface)
 {
-	iface->drag_data_received = model_drag_data_received;
-	iface->row_drop_possible = model_row_drop_possible;
-}
-
-static void
-tree_drag_source_iface_init (GtkTreeDragSourceIface *iface)
-{
-	iface->row_draggable = model_row_draggable;
-	iface->drag_data_get = model_drag_data_get;
-	iface->drag_data_delete = model_drag_data_delete;
-}
-
-
-static void
-drop_uid_list (CamelFolder *dest, gboolean move, GtkSelectionData *selection, CamelException *ex)
-{
-	CamelFolder *src;
-	GPtrArray *uids;
-	char *src_uri;
-	
-	em_utils_selection_get_uidlist (selection, &src_uri, &uids);
-	
-	if (!(src = mail_tool_uri_to_folder (src_uri, 0, ex))) {
-		em_utils_uids_free (uids);
-		g_free (src_uri);
-		return;
-	}
-	
-	g_free (src_uri);
-	
-	camel_folder_transfer_messages_to (src, uids, dest, NULL, move, ex);
-	em_utils_uids_free (uids);
-	camel_object_unref (src);
-}
-
-static void
-drop_folder (CamelFolder *dest, gboolean move, GtkSelectionData *selection, CamelException *ex)
-{
-	CamelFolder *src;
-	
-	/* get the folder being dragged */
-	if (!(src = mail_tool_uri_to_folder (selection->data, 0, ex)))
-		return;
-	
-	if (src->parent_store == dest->parent_store && move) {
-		/* simple rename() action */
-		char *old_name, *new_name;
-		
-		old_name = g_strdup (src->full_name);
-		new_name = g_strdup_printf ("%s/%s", dest->full_name, src->name);
-		
-		camel_store_rename_folder (dest->parent_store, old_name, new_name, ex);
-		
-		g_free (old_name);
-		g_free (new_name);
-	} else {
-		/* copy the folder to the new location */
-		CamelFolder *folder;
-		char *path;
-		
-		path = g_strdup_printf ("%s/%s", dest->full_name, src->name);
-		if ((folder = camel_store_get_folder (dest->parent_store, path, CAMEL_STORE_FOLDER_CREATE, ex))) {
-			GPtrArray *uids;
-			
-			uids = camel_folder_get_uids (src);
-			camel_folder_transfer_messages_to (src, uids, folder, NULL, FALSE, ex);
-			camel_folder_free_uids (src, uids);
-			
-			camel_object_unref (folder);
-		}
-		
-		g_free (path);
-	}
-	
-	camel_object_unref (src);
-}
-
-static gboolean
-import_message_rfc822 (CamelFolder *dest, CamelStream *stream, gboolean scan_from, CamelException *ex)
-{
-	CamelMimeParser *mp;
-	
-	mp = camel_mime_parser_new ();
-	camel_mime_parser_scan_from (mp, scan_from);
-	camel_mime_parser_init_with_stream (mp, stream);
-	
-	while (camel_mime_parser_step (mp, 0, 0) == CAMEL_MIME_PARSER_STATE_FROM) {
-		CamelMessageInfo *info;
-		CamelMimeMessage *msg;
-		
-		msg = camel_mime_message_new ();
-		if (camel_mime_part_construct_from_parser (CAMEL_MIME_PART (msg), mp) == -1) {
-			camel_object_unref (msg);
-			camel_object_unref (mp);
-			return FALSE;
-		}
-		
-		/* append the message to the folder... */
-		info = g_new0 (CamelMessageInfo, 1);
-		camel_folder_append_message (dest, msg, info, NULL, ex);
-		camel_object_unref (msg);
-		
-		if (camel_exception_is_set (ex)) {
-			camel_object_unref (mp);
-			return FALSE;
-		}
-		
-		/* skip over the FROM_END state */
-		camel_mime_parser_step (mp, 0, 0);
-	}
-	
-	camel_object_unref (mp);
-	
-	return TRUE;
-}
-
-static void
-drop_message_rfc822 (CamelFolder *dest, GtkSelectionData *selection, CamelException *ex)
-{
-	CamelStream *stream;
-	gboolean scan_from;
-	
-	scan_from = selection->length > 5 && !strncmp (selection->data, "From ", 5);
-	stream = camel_stream_mem_new_with_buffer (selection->data, selection->length);
-	
-	import_message_rfc822 (dest, stream, scan_from, ex);
-	
-	camel_object_unref (stream);
-}
-
-static void
-drop_text_uri_list (CamelFolder *dest, GtkSelectionData *selection, CamelException *ex)
-{
-	char **urls, *url, *tmp;
-	CamelStream *stream;
-	CamelURL *uri;
-	int fd, i;
-	
-	tmp = g_strndup (selection->data, selection->length);
-	urls = g_strsplit (tmp, "\n", 0);
-	g_free (tmp);
-	
-	for (i = 0; urls[i] != NULL; i++) {
-		/* get the path component */
-		url = g_strstrip (urls[i]);
-		uri = camel_url_new (url, NULL);
-		g_free (url);
-		
-		if (!uri || strcmp (uri->protocol, "file") != 0) {
-			camel_url_free (uri);
-			continue;
-		}
-		
-		url = uri->path;
-		uri->path = NULL;
-		camel_url_free (uri);
-		
-		if ((fd = open (url, O_RDONLY)) == -1) {
-			g_free (url);
-			continue;
-		}
-		
-		stream = camel_stream_fs_new_with_fd (fd);
-		if (!import_message_rfc822 (dest, stream, TRUE, ex)) {
-			/* FIXME: should we abort now? or continue? */
-			/* for now lets just continue... */
-			camel_exception_clear (ex);
-		}
-		
-		camel_object_unref (stream);
-		g_free (url);
-	}
-	
-	g_free (urls);
-}
-
-
-static gboolean
-model_drag_data_received (GtkTreeDragDest *drag_dest, GtkTreePath *dest_path, GtkSelectionData *selection)
-{
-	EMFolderTreeModel *model = (EMFolderTreeModel *) drag_dest;
-	const char *full_name;
-	CamelFolder *folder;
-	CamelStore *store;
-	CamelException ex;
-	GtkTreeIter iter;
-	char *path;
-	
-	/* this means we are receiving no data */
-	if (!selection->data || selection->length == -1)
-		return FALSE;
-	
-	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, dest_path))
-		return FALSE;
-	
-	gtk_tree_model_get ((GtkTreeModel *) model, &iter,
-			    COL_POINTER_CAMEL_STORE, &store,
-			    COL_STRING_FOLDER_PATH, &path, -1);
-	
-	/* make sure user isn't try to drop on a placeholder row */
-	if (path == NULL)
-		return FALSE;
-	
-	full_name = path[0] == '/' ? path + 1 : path;
-	
-	camel_exception_init (&ex);
-	if ((folder = camel_store_get_folder (store, full_name, 0, &ex))) {
-		/* FIXME: would have been nicer if we could 'move'
-		 * messages and/or folders. but alas, gtktreeview
-		 * drag&drop doesn't give us the context->action to
-		 * check for GDK_ACTION_MOVE, so we can't. Yay. */
-		gboolean move = FALSE;
-		
-		if (selection->target == gdk_atom_intern ("x-uid-list", FALSE)) {
-			/* import a list of uids from another evo folder */
-			drop_uid_list (folder, move, selection, &ex);
-			d(printf ("* dropped a x-uid-list\n"));
-		} else if (selection->target == gdk_atom_intern ("x-folder", FALSE)) {
-			/* copy or move (aka rename) a folder */
-			drop_folder (folder, move, selection, &ex);
-			d(printf ("* dropped a x-folder\n"));
-		} else if (selection->target == gdk_atom_intern ("message/rfc822", FALSE)) {
-			/* import a message/rfc822 stream */
-			drop_message_rfc822 (folder, selection, &ex);
-			d(printf ("* dropped a message/rfc822\n"));
-		} else if (selection->target == gdk_atom_intern ("text/uri-list", FALSE)) {
-			/* import an mbox, maildir, or mh folder? */
-			drop_text_uri_list (folder, selection, &ex);
-			d(printf ("* dropped a text/uri-list\n"));
-		} else {
-			g_assert_not_reached ();
-		}
-	}
-	
-	if (camel_exception_is_set (&ex)) {
-		/* FIXME: error dialog? */
-		camel_exception_clear (&ex);
-		return FALSE;
-	}
-	
-	return TRUE;
-}
-
-static gboolean
-model_row_drop_possible (GtkTreeDragDest *drag_dest, GtkTreePath *dest_path, GtkSelectionData *selection)
-{
-	EMFolderTreeModel *model = (EMFolderTreeModel *) drag_dest;
-	gboolean is_store;
-	GtkTreeIter iter;
-	
-	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, dest_path))
-		return FALSE;
-	
-	gtk_tree_model_get ((GtkTreeModel *) model, &iter, COL_BOOL_IS_STORE, &is_store, -1);
-	
-	if (selection->target == gdk_atom_intern ("x-uid-list", FALSE)) {
-		if (is_store)
-			return FALSE;
-		
-		return TRUE;
-	} else if (selection->target == gdk_atom_intern ("x-folder", FALSE)) {
-		return TRUE;
-	} else if (selection->target == gdk_atom_intern ("message/rfc822", FALSE)) {
-		if (is_store)
-			return FALSE;
-		
-		return TRUE;
-	} else if (selection->target == gdk_atom_intern ("text/uri-list", FALSE)) {
-		if (is_store)
-			return FALSE;
-		
-		return TRUE;
-	} else {
-		g_assert_not_reached ();
-		return FALSE;
-	}
-}
-
-static gboolean
-model_row_draggable (GtkTreeDragSource *drag_source, GtkTreePath *src_path)
-{
-	EMFolderTreeModel *model = (EMFolderTreeModel *) drag_source;
-	gboolean is_store;
-	GtkTreeIter iter;
-	
-	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, src_path))
-		return FALSE;
-	
-	gtk_tree_model_get ((GtkTreeModel *) model, &iter, COL_BOOL_IS_STORE, &is_store, -1);
-	
-	return !is_store;
-}
-
-static void
-drag_text_uri_list (CamelFolder *src, GtkSelectionData *selection, CamelException *ex)
-{
-	CamelFolder *dest;
-	const char *tmpdir;
-	CamelStore *store;
-	GPtrArray *uids;
-	GString *url;
-	
-	if (!(tmpdir = e_mkdtemp ("drag-n-drop-XXXXXX"))) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not create temporary directory: %s"),
-				      g_strerror (errno));
-		return;
-	}
-	
-	url = g_string_new ("mbox:");
-	g_string_append (url, tmpdir);
-	if (!(store = camel_session_get_store (session, url->str, ex))) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not create temporary mbox store: %s"),
-				      camel_exception_get_description (ex));
-		g_string_free (url, TRUE);
-		
-		return;
-	}
-	
-	if (!(dest = camel_store_get_folder (store, "mbox", CAMEL_STORE_FOLDER_CREATE, ex))) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not create temporary mbox folder: %s"),
-				      camel_exception_get_description (ex));
-		
-		camel_object_unref (store);
-		g_string_free (url, TRUE);
-		
-		return;
-	}
-	
-	camel_object_unref (store);
-	uids = camel_folder_get_uids (src);
-	
-	camel_folder_transfer_messages_to (src, uids, dest, NULL, FALSE, ex);
-	if (camel_exception_is_set (ex)) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not copy messages to temporary mbox folder: %s"),
-				      camel_exception_get_description (ex));
-	} else {
-		/* replace "mbox:" with "file:" */
-		memcpy (url->str, "file", 4);
-		g_string_append (url, "\r\n");
-		gtk_selection_data_set (selection, selection->target, 8, url->str, url->len);
-	}
-	
-	camel_folder_free_uids (src, uids);
-	camel_object_unref (dest);
-	g_string_free (url, TRUE);
-}
-
-static gboolean
-model_drag_data_get (GtkTreeDragSource *drag_source, GtkTreePath *src_path, GtkSelectionData *selection)
-{
-	EMFolderTreeModel *model = (EMFolderTreeModel *) drag_source;
-	const char *full_name;
-	CamelFolder *folder;
-	CamelStore *store;
-	CamelException ex;
-	GtkTreeIter iter;
-	char *path, *uri;
-	
-	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, src_path))
-		return FALSE;
-	
-	gtk_tree_model_get ((GtkTreeModel *) model, &iter,
-			    COL_POINTER_CAMEL_STORE, &store,
-			    COL_STRING_FOLDER_PATH, &path,
-			    COL_STRING_URI, &uri, -1);
-	
-	/* make sure user isn't try to drag on a placeholder row */
-	if (path == NULL)
-		return FALSE;
-	
-	full_name = path[0] == '/' ? path + 1 : path;
-	
-	camel_exception_init (&ex);
-	
-	if (selection->target == gdk_atom_intern ("x-folder", FALSE)) {
-		/* dragging to a new location in the folder tree */
-		gtk_selection_data_set (selection, selection->target, 8, uri, strlen (uri) + 1);
-	} else if (selection->target == gdk_atom_intern ("text/uri-list", FALSE)) {
-		/* dragging to nautilus or something, probably */
-		if ((folder = camel_store_get_folder (store, full_name, 0, &ex))) {
-			drag_text_uri_list (folder, selection, &ex);
-			camel_object_unref (folder);
-		}
-	} else {
-		g_assert_not_reached ();
-	}
-	
-	if (camel_exception_is_set (&ex)) {
-		/* FIXME: error dialog? */
-		camel_exception_clear (&ex);
-		return FALSE;
-	}
-	
-	return TRUE;
-}
-
-static gboolean
-model_drag_data_delete (GtkTreeDragSource *drag_source, GtkTreePath *src_path)
-{
-	EMFolderTreeModel *model = (EMFolderTreeModel *) drag_source;
-	const char *full_name;
-	gboolean is_store;
-	CamelStore *store;
-	CamelException ex;
-	GtkTreeIter iter;
-	char *path;
-	
-	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, src_path))
-		return FALSE;
-	
-	gtk_tree_model_get ((GtkTreeModel *) model, &iter,
-			    COL_POINTER_CAMEL_STORE, &store,
-			    COL_STRING_FOLDER_PATH, &path,
-			    COL_BOOL_IS_STORE, &is_store, -1);
-	
-	if (is_store)
-		return FALSE;
-	
-	full_name = path[0] == '/' ? path + 1 : path;
-	
-	camel_exception_init (&ex);
-	camel_store_delete_folder (store, full_name, &ex);
-	if (camel_exception_is_set (&ex)) {
-		/* FIXME: error dialog? */
-		camel_exception_clear (&ex);
-		return FALSE;
-	}
-	
-	return TRUE;
+	;
 }
 
 
 static void
 em_folder_tree_model_load_state (EMFolderTreeModel *model, const char *filename)
 {
-	char *node;
-	FILE *fp;
+	xmlNodePtr root, node;
+	struct stat st;
 	
-	g_hash_table_foreach_remove (model->expanded, expanded_free, NULL);
+	if (model->expanded)
+		xmlFreeDoc (model->expanded);
 	
-	if ((fp = fopen (filename, "r")) == NULL)
+	if (stat (filename, &st) == 0 && (model->expanded = xmlParseFile (filename)))
 		return;
 	
-	while (camel_file_util_decode_string (fp, &node) != -1)
-		g_hash_table_insert (model->expanded, node, GINT_TO_POINTER (TRUE));
+	/* setup some defaults - expand "Local Folders" and "VFolders" */
+	model->expanded = xmlNewDoc ("1.0");
+	root = xmlNewDocNode (model->expanded, NULL, "tree-state", NULL);
+	xmlDocSetRootElement (model->expanded, root);
 	
-	fclose (fp);
+	node = xmlNewChild (root, NULL, "node", NULL);
+	xmlSetProp (node, "name", "local");
+	xmlSetProp (node, "expand", "true");
+	
+	node = xmlNewChild (root, NULL, "node", NULL);
+	xmlSetProp (node, "name", "vfolder");
+	xmlSetProp (node, "expand", "true");
 }
 
 
@@ -715,8 +346,11 @@ em_folder_tree_model_new (const char *evolution_dir)
 	
 	model = g_object_new (EM_TYPE_FOLDER_TREE_MODEL, NULL);
 	gtk_tree_store_set_column_types ((GtkTreeStore *) model, NUM_COLUMNS, col_types);
+	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable *) model,
+					      GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
+					      GTK_SORT_ASCENDING);
 	
-	filename = g_build_filename (evolution_dir, "mail", "config", "folder-tree.state", NULL);
+	filename = g_build_filename (evolution_dir, "mail", "config", "folder-tree-expand-state.xml", NULL);
 	em_folder_tree_model_load_state (model, filename);
 	model->filename = filename;
 	
@@ -724,48 +358,111 @@ em_folder_tree_model_new (const char *evolution_dir)
 }
 
 
+static void
+account_changed (EAccountList *accounts, EAccount *account, gpointer user_data)
+{
+	EMFolderTreeModel *model = user_data;
+	struct _EMFolderTreeModelStoreInfo *si;
+	CamelProvider *provider;
+	CamelStore *store;
+	CamelException ex;
+	char *uri;
+	
+	if (!(si = g_hash_table_lookup (model->account_hash, account)))
+		return;
+	
+	em_folder_tree_model_remove_store (model, si->store);
+	
+	if (!(uri = account->source->url))
+		return;
+	
+	camel_exception_init (&ex);
+	if (!(provider = camel_provider_get(uri, &ex))) {
+		camel_exception_clear (&ex);
+		return;
+	}
+	
+	/* make sure the new store belongs in the tree */
+	if (!(provider->flags & CAMEL_PROVIDER_IS_STORAGE))
+		return;
+	
+	if (!(store = (CamelStore *) camel_session_get_service (session, uri, CAMEL_PROVIDER_STORE, &ex))) {
+		camel_exception_clear (&ex);
+		return;
+	}
+	
+	em_folder_tree_model_add_store (model, store, account->name);
+	camel_object_unref (store);
+}
+
+static void
+account_removed (EAccountList *accounts, EAccount *account, gpointer user_data)
+{
+	EMFolderTreeModel *model = user_data;
+	struct _EMFolderTreeModelStoreInfo *si;
+	
+	if (!(si = g_hash_table_lookup (model->account_hash, account)))
+		return;
+	
+	em_folder_tree_model_remove_store (model, si->store);
+}
+
+
 void
 em_folder_tree_model_set_folder_info (EMFolderTreeModel *model, GtkTreeIter *iter,
 				      struct _EMFolderTreeModelStoreInfo *si,
-				      CamelFolderInfo *fi)
+				      CamelFolderInfo *fi, int fully_loaded)
 {
 	GtkTreeRowReference *uri_row, *path_row;
 	unsigned int unread;
 	GtkTreePath *path;
 	GtkTreeIter sub;
-	gboolean load;
+	gboolean load = FALSE;
+	struct _CamelFolder *folder;
+	gboolean emitted = FALSE;
 	
-	load = fi->child == NULL && !(fi->flags & (CAMEL_FOLDER_NOCHILDREN | CAMEL_FOLDER_NOINFERIORS));
+	if (!fully_loaded)
+		load = fi->child == NULL && !(fi->flags & (CAMEL_FOLDER_NOCHILDREN | CAMEL_FOLDER_NOINFERIORS));
 	
 	path = gtk_tree_model_get_path ((GtkTreeModel *) model, iter);
 	uri_row = gtk_tree_row_reference_new ((GtkTreeModel *) model, path);
 	path_row = gtk_tree_row_reference_copy (uri_row);
 	gtk_tree_path_free (path);
 	
-	g_hash_table_insert (model->uri_hash, g_strdup (fi->url), uri_row);
+	g_hash_table_insert (model->uri_hash, g_strdup (fi->uri), uri_row);
 	g_hash_table_insert (si->path_hash, g_strdup (fi->path), path_row);
 	
-	unread = fi->unread_message_count == -1 ? 0 : fi->unread_message_count;
-	
+	/* HACK: if we have the folder, and its the outbox folder, we need the total count, not unread */
+	/* This is duplicated in mail-folder-cache too, should perhaps be functionised */
+	unread = fi->unread == -1 ? 0 : fi->unread;
+	if (mail_note_get_folder_from_uri(fi->uri, &folder) && folder) {
+		if (folder == mail_component_get_folder(NULL, MAIL_COMPONENT_FOLDER_OUTBOX)) {
+			int total;
+			
+			if ((total = camel_folder_get_message_count (folder)) > 0) {
+				int deleted = camel_folder_get_deleted_message_count (folder);
+				
+				if (deleted != -1)
+					total -= deleted;
+			}
+			
+			unread = total > 0 ? total : 0;
+		}
+		camel_object_unref(folder);
+	}
+		
 	gtk_tree_store_set ((GtkTreeStore *) model, iter,
 			    COL_STRING_DISPLAY_NAME, fi->name,
 			    COL_POINTER_CAMEL_STORE, si->store,
 			    COL_STRING_FOLDER_PATH, fi->path,
-			    COL_STRING_URI, fi->url,
+			    COL_STRING_URI, fi->uri,
 			    COL_UINT_UNREAD, unread,
+			    COL_UINT_FLAGS, fi->flags,
 			    COL_BOOL_IS_STORE, FALSE,
 			    COL_BOOL_LOAD_SUBDIRS, load,
 			    -1);
 	
-	if (fi->child) {
-		fi = fi->child;
-		
-		do {
-			gtk_tree_store_append ((GtkTreeStore *) model, &sub, iter);
-			em_folder_tree_model_set_folder_info (model, &sub, si, fi);
-			fi = fi->sibling;
-		} while (fi);
-	} else if (load) {
+	if (load) {
 		/* create a placeholder node for our subfolders... */
 		gtk_tree_store_append ((GtkTreeStore *) model, &sub, iter);
 		gtk_tree_store_set ((GtkTreeStore *) model, &sub,
@@ -781,6 +478,31 @@ em_folder_tree_model_set_folder_info (EMFolderTreeModel *model, GtkTreeIter *ite
 		path = gtk_tree_model_get_path ((GtkTreeModel *) model, iter);
 		g_signal_emit (model, signals[LOADING_ROW], 0, path, iter);
 		gtk_tree_path_free (path);
+		return;
+	}
+	
+	if (fi->child) {
+		fi = fi->child;
+		
+		do {
+			gtk_tree_store_append ((GtkTreeStore *) model, &sub, iter);
+			
+			if (!emitted) {
+				path = gtk_tree_model_get_path ((GtkTreeModel *) model, iter);
+				g_signal_emit (model, signals[LOADED_ROW], 0, path, iter);
+				gtk_tree_path_free (path);
+				emitted = TRUE;
+			}
+			
+			em_folder_tree_model_set_folder_info (model, &sub, si, fi, fully_loaded);
+			fi = fi->next;
+		} while (fi);
+	}
+	
+	if (!emitted) {
+		path = gtk_tree_model_get_path ((GtkTreeModel *) model, iter);
+		g_signal_emit (model, signals[LOADED_ROW], 0, path, iter);
+		gtk_tree_path_free (path);
 	}
 }
 
@@ -837,7 +559,9 @@ folder_subscribed (CamelStore *store, CamelFolderInfo *fi, EMFolderTreeModel *mo
 	/* append a new node */
 	gtk_tree_store_append ((GtkTreeStore *) model, &iter, &parent);
 	
-	em_folder_tree_model_set_folder_info (model, &iter, si, fi);
+	em_folder_tree_model_set_folder_info (model, &iter, si, fi, TRUE);
+	
+	g_signal_emit (model, signals[FOLDER_ADDED], 0, fi->path, fi->uri);
 	
  done:
 	
@@ -949,10 +673,10 @@ folder_renamed (CamelStore *store, CamelRenameInfo *info, EMFolderTreeModel *mod
 	em_folder_tree_model_remove_folders (model, si, &iter);
 	
 	parent = g_strdup (info->new->path);
-	if ((p = strrchr (parent + 1, '/')))
-		*p = '\0';
-	
-	if (!strcmp (parent, "/")) {
+	p = strrchr(parent, '/');
+	g_assert(p);
+	*p = 0;
+	if (parent == p) {
 		/* renamed to a toplevel folder on the store */
 		path = gtk_tree_row_reference_get_path (si->row);
 	} else {
@@ -978,7 +702,7 @@ folder_renamed (CamelStore *store, CamelRenameInfo *info, EMFolderTreeModel *mod
 	}
 	
 	gtk_tree_store_append ((GtkTreeStore *) model, &iter, &root);
-	em_folder_tree_model_set_folder_info (model, &iter, si, info->new);
+	em_folder_tree_model_set_folder_info (model, &iter, si, info->new, TRUE);
 	
  done:
 	
@@ -1010,29 +734,20 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	GtkTreeRowReference *row;
 	GtkTreeIter root, iter;
 	GtkTreePath *path;
+	EAccount *account;
 	char *uri;
 	
 	g_return_if_fail (EM_IS_FOLDER_TREE_MODEL (model));
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (display_name != NULL);
 	
-	if ((si = g_hash_table_lookup (model->store_hash, store))) {
-		const char *name;
-		
-		path = gtk_tree_row_reference_get_path (si->row);
-		gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, path);
-		gtk_tree_path_free (path);
-		
-		gtk_tree_model_get ((GtkTreeModel *) model, &iter, COL_STRING_DISPLAY_NAME, (char **) &name, -1);
-		
-		g_warning ("the store `%s' is already in the folder tree as `%s'",
-			   display_name, name);
-		
-		return;
-	}
+	if ((si = g_hash_table_lookup (model->store_hash, store)))
+		em_folder_tree_model_remove_store (model, store);
 	
 	uri = camel_url_to_string (((CamelService *) store)->url, CAMEL_URL_HIDE_ALL);
 	
+	account = mail_config_get_account_by_source_url (uri);
+	
 	/* add the store to the tree */
 	gtk_tree_store_append ((GtkTreeStore *) model, &iter, NULL);
 	gtk_tree_store_set ((GtkTreeStore *) model, &iter,
@@ -1045,15 +760,16 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	
 	path = gtk_tree_model_get_path ((GtkTreeModel *) model, &iter);
 	row = gtk_tree_row_reference_new ((GtkTreeModel *) model, path);
-	gtk_tree_path_free (path);
 	
 	si = g_new (struct _EMFolderTreeModelStoreInfo, 1);
 	si->display_name = g_strdup (display_name);
 	camel_object_ref (store);
 	si->store = store;
+	si->account = account;
 	si->row = row;
 	si->path_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	g_hash_table_insert (model->store_hash, store, si);
+	g_hash_table_insert (model->account_hash, account, si);
 	
 	/* each store has folders... but we don't load them until the user demands them */
 	root = iter;
@@ -1077,6 +793,9 @@ em_folder_tree_model_add_store (EMFolderTreeModel *model, CamelStore *store, con
 	si->renamed_id = camel_object_hook_event (store, "folder_renamed", CAMEL_CALLBACK (folder_renamed_cb), model);
 	si->subscribed_id = camel_object_hook_event (store, "folder_subscribed", CAMEL_CALLBACK (folder_subscribed_cb), model);
 	si->unsubscribed_id = camel_object_hook_event (store, "folder_unsubscribed", CAMEL_CALLBACK (folder_unsubscribed_cb), model);
+	
+	g_signal_emit (model, signals[LOADED_ROW], 0, path, &root);
+	gtk_tree_path_free (path);
 }
 
 
@@ -1107,6 +826,7 @@ em_folder_tree_model_remove_store_info (EMFolderTreeModel *model, CamelStore *st
 		return;
 	
 	g_hash_table_remove (model->store_hash, si->store);
+	g_hash_table_remove (model->account_hash, si->account);
 	store_info_free (si);
 }
 
@@ -1157,11 +877,8 @@ em_folder_tree_model_remove_store (EMFolderTreeModel *model, CamelStore *store)
 	g_return_if_fail (EM_IS_FOLDER_TREE_MODEL (model));
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	
-	if (!(si = g_hash_table_lookup (model->store_hash, store))) {
-		g_warning ("the store `%s' is not in the folder tree", si->display_name);
-		
+	if (!(si = g_hash_table_lookup (model->store_hash, store)))
 		return;
-	}
 	
 	path = gtk_tree_row_reference_get_path (si->row);
 	gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, path);
@@ -1172,11 +889,64 @@ em_folder_tree_model_remove_store (EMFolderTreeModel *model, CamelStore *store)
 }
 
 
+static xmlNodePtr
+find_xml_node (xmlNodePtr root, const char *name)
+{
+	xmlNodePtr node;
+	char *nname;
+	
+	node = root->children;
+	while (node != NULL) {
+		if (!strcmp (node->name, "node")) {
+			nname = xmlGetProp (node, "name");
+			if (nname && !strcmp (nname, name)) {
+				xmlFree (nname);
+				return node;
+			}
+			
+			xmlFree (nname);
+		}
+		
+		node = node->next;
+	}
+	
+	return node;
+}
+
 gboolean
 em_folder_tree_model_get_expanded (EMFolderTreeModel *model, const char *key)
 {
-	if (g_hash_table_lookup (model->expanded, key))
-		return TRUE;
+	xmlNodePtr node;
+	const char *name;
+	char *buf, *p;
+	
+	node = model->expanded ? model->expanded->children : NULL;
+	if (!node || strcmp (node->name, "tree-state") != 0)
+		return FALSE;
+	
+	name = buf = g_alloca (strlen (key) + 1);
+	p = g_stpcpy (buf, key);
+	if (p[-1] == '/')
+		p[-1] = '\0';
+	p = NULL;
+	
+	do {
+		if ((p = strchr (name, '/')))
+			*p = '\0';
+		
+		if ((node = find_xml_node (node, name))) {
+			gboolean expanded;
+			
+			buf = xmlGetProp (node, "expand");
+			expanded = buf && !strcmp (buf, "true");
+			xmlFree (buf);
+			
+			if (!expanded || p == NULL)
+				return expanded;
+		}
+		
+		name = p ? p + 1 : NULL;
+	} while (name && node);
 	
 	return FALSE;
 }
@@ -1185,34 +955,55 @@ em_folder_tree_model_get_expanded (EMFolderTreeModel *model, const char *key)
 void
 em_folder_tree_model_set_expanded (EMFolderTreeModel *model, const char *key, gboolean expanded)
 {
-	gpointer okey, oval;
+	xmlNodePtr node, parent;
+	const char *name;
+	char *buf, *p;
 	
-	if (g_hash_table_lookup_extended (model->expanded, key, &okey, &oval)) {
-		g_hash_table_remove (model->expanded, okey);
-		g_free (okey);
+	if (model->expanded == NULL)
+		model->expanded = xmlNewDoc ("1.0");
+	
+	if (!model->expanded->children) {
+		node = xmlNewDocNode (model->expanded, NULL, "tree-state", NULL);
+		xmlDocSetRootElement (model->expanded, node);
+	} else {
+		node = model->expanded->children;
 	}
 	
-	if (expanded)
-		g_hash_table_insert (model->expanded, g_strdup (key), GINT_TO_POINTER (TRUE));
-}
-
-
-static void
-expanded_save (gpointer key, gpointer value, FILE *fp)
-{
-	/* FIXME: don't save stale entries */
-	if (!GPOINTER_TO_INT (value))
-		return;
+	name = buf = g_alloca (strlen (key) + 1);
+	p = g_stpcpy (buf, key);
+	if (p[-1] == '/')
+		p[-1] = '\0';
+	p = NULL;
 	
-	camel_file_util_encode_string (fp, key);
+	do {
+		parent = node;
+		if ((p = strchr (name, '/')))
+			*p = '\0';
+		
+		if (!(node = find_xml_node (node, name))) {
+			if (!expanded) {
+				/* node doesn't exist, so we don't need to set expanded to FALSE */
+				return;
+			}
+			
+			/* node (or parent node) doesn't exist, need to add it */
+			node = xmlNewChild (parent, NULL, "node", NULL);
+			xmlSetProp (node, "name", name);
+		}
+		
+		xmlSetProp (node, "expand", expanded || p ? "true" : "false");
+		
+		name = p ? p + 1 : NULL;
+	} while (name);
 }
 
 void
 em_folder_tree_model_save_expanded (EMFolderTreeModel *model)
 {
-	char *dirname, *tmpname;
-	FILE *fp;
-	int fd;
+	char *dirname;
+	
+	if (model->expanded == NULL)
+		return;
 	
 	dirname = g_path_get_dirname (model->filename);
 	if (camel_mkdir (dirname, 0777) == -1 && errno != EEXIST) {
@@ -1221,44 +1012,54 @@ em_folder_tree_model_save_expanded (EMFolderTreeModel *model)
 	}
 	
 	g_free (dirname);
-	tmpname = g_strdup_printf ("%s~", model->filename);
 	
-	if (!(fp = fopen (tmpname, "w+"))) {
-		g_free (tmpname);
-		return;
-	}
-	
-	g_hash_table_foreach (model->expanded, (GHFunc) expanded_save, fp);
-	
-	if (fflush (fp) != 0)
-		goto exception;
-	
-	if ((fd = fileno (fp)) == -1)
-		goto exception;
-	
-	if (fsync (fd) == -1)
-		goto exception;
-	
-	fclose (fp);
-	fp = NULL;
-	
-	if (rename (tmpname, model->filename) == -1)
-		goto exception;
-	
-	g_free (tmpname);
-	
-	return;
-	
- exception:
-	
-	if (fp != NULL)
-		fclose (fp);
-	
-	unlink (tmpname);
-	g_free (tmpname);
+	e_xml_save_file (model->filename, model->expanded);
 }
 
 
+static void
+expand_foreach_r (EMFolderTreeModel *model, xmlNodePtr parent, const char *dirname, EMFTModelExpandFunc func, void *user_data)
+{
+	xmlNodePtr node = parent->children;
+	char *path, *name, *expand;
+	
+	while (node != NULL) {
+		if (!strcmp (node->name, "node")) {
+			name = xmlGetProp (node, "name");
+			expand = xmlGetProp (node, "expand");
+			
+			if (expand && name && !strcmp (expand, "true")) {
+				if (dirname)
+					path = g_strdup_printf ("%s/%s", dirname, name);
+				else
+					path = g_strdup (name);
+				
+				func (model, path, user_data);
+				if (node->children)
+					expand_foreach_r (model, node, path, func, user_data);
+				g_free (path);
+			}
+			
+			xmlFree (expand);
+			xmlFree (name);
+		}
+		
+		node = node->next;
+	}
+}
+
+void
+em_folder_tree_model_expand_foreach (EMFolderTreeModel *model, EMFTModelExpandFunc func, void *user_data)
+{
+	xmlNodePtr root;
+	
+	root = model->expanded ? model->expanded->children : NULL;
+	if (!root || !root->children || strcmp (root->name, "tree-state") != 0)
+		return;
+	
+	expand_foreach_r (model, root, NULL, func, user_data);
+}
+
 void
 em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *store, const char *path, int unread)
 {
@@ -1270,15 +1071,21 @@ em_folder_tree_model_set_unread_count (EMFolderTreeModel *model, CamelStore *sto
 	g_return_if_fail (EM_IS_FOLDER_TREE_MODEL (model));
 	g_return_if_fail (CAMEL_IS_STORE (store));
 	g_return_if_fail (path != NULL);
-	
+
+	u(printf("set unread count %p '%s' %d\n", store, path, unread));
+
 	if (unread < 0)
 		unread = 0;
 	
-	if (!(si = g_hash_table_lookup (model->store_hash, store)))
+	if (!(si = g_hash_table_lookup (model->store_hash, store))) {
+		u(printf("  can't find store\n"));
 		return;
+	}
 	
-	if (!(row = g_hash_table_lookup (si->path_hash, path)))
+	if (!(row = g_hash_table_lookup (si->path_hash, path))) {
+		u(printf("  can't find row\n"));
 		return;
+	}
 	
 	tree_path = gtk_tree_row_reference_get_path (row);
 	if (!gtk_tree_model_get_iter ((GtkTreeModel *) model, &iter, tree_path)) {
