@@ -40,8 +40,8 @@
 #include "mail-vfolder.h"
 #include "mail-autofilter.h"
 #include "mail-component.h"
-
-#include "camel/camel.h"
+#include "em-utils.h"
+#include "widgets/misc/e-error.h"
 
 #include "filter/vfolder-context.h"
 #include "filter/vfolder-rule.h"
@@ -222,28 +222,6 @@ rule_match_mlist(RuleContext *context, FilterRule *rule, const char *mlist)
 	filter_input_set_value((FilterInput *)element, mlist);
 }
 
-static void
-rule_match_thread (RuleContext *context, FilterRule *rule, const char *msgid)
-{
-	FilterElement *element;
-	FilterPart *part;
-	
-	if (msgid[0] == 0)
-		return;
-	
-	part = rule_context_create_part (context, "header");
-	filter_rule_add_part (rule, part);
-	
-	element = filter_part_find_element (part, "header-type");
-	filter_option_set_current ((FilterOption *) element, "contains");
-
-	element = filter_part_find_element (part, "header-field");
-	filter_input_set_value ((FilterInput *) element, "References");
-	
-	element = filter_part_find_element (part, "word");
-	filter_input_set_value ((FilterInput *) element, msgid);
-}
-
 static void
 rule_from_message (FilterRule *rule, RuleContext *context, CamelMimeMessage *msg, int flags)
 {
@@ -296,41 +274,19 @@ rule_from_message (FilterRule *rule, RuleContext *context, CamelMimeMessage *msg
 		}
 		g_free(mlist);
 	}
-	if (flags & AUTO_THREAD) {
-		const char *msgid = NULL, *refs = NULL;
-		char *name;
-		
-		if (!(refs = camel_medium_get_header ((CamelMedium *) msg, "References"))) {
-			if (!(refs = camel_medium_get_header ((CamelMedium *) msg, "In-Reply-To")))
-				msgid = camel_mime_message_get_message_id (msg);
-		}
-		
-		if (refs || msgid) {
-			struct _camel_header_references *r = NULL;
-			
-			if (refs) {
-				r = camel_header_references_decode (refs);
-				msgid = r->id;
-			}
-			
-			rule_match_thread (context, rule, msgid);
-			name = g_strdup_printf (_("Replies to %s"), msgid);
-			
-			if (r)
-				camel_header_references_list_clear (&r);
-		}
-	}
 }
 
 FilterRule *
 vfolder_rule_from_message (VfolderContext *context, CamelMimeMessage *msg, int flags, const char *source)
 {
 	VfolderRule *rule;
-	
+	char *euri = em_uri_from_camel(source);
+
 	rule = vfolder_rule_new ();
-	vfolder_rule_add_source (rule, source);
+	vfolder_rule_add_source (rule, euri);
 	rule_from_message ((FilterRule *)rule, (RuleContext *)context, msg, flags);
-	
+	g_free(euri);
+
 	return (FilterRule *)rule;
 }
 
@@ -375,17 +331,20 @@ filter_gui_add_from_message (CamelMimeMessage *msg, const char *source, int flag
 void
 mail_filter_rename_uri(CamelStore *store, const char *olduri, const char *newuri)
 {
-	GCompareFunc uri_cmp = CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name;
 	FilterContext *fc;
 	char *user, *system;
 	GList *changed;
-	
+	char *eolduri, *enewuri;
+
+	eolduri = em_uri_from_camel(olduri);
+	enewuri = em_uri_from_camel(newuri);
+
 	fc = filter_context_new ();
 	user = g_strdup_printf ("%s/mail/filters.xml", mail_component_peek_base_directory (mail_component_peek ()));
 	system = EVOLUTION_PRIVDATADIR "/filtertypes.xml";
 	rule_context_load ((RuleContext *)fc, system, user);
 	
-	changed = rule_context_rename_uri((RuleContext *)fc, olduri, newuri, uri_cmp);
+	changed = rule_context_rename_uri((RuleContext *)fc, eolduri, enewuri, g_str_equal);
 	if (changed) {
 		printf("Folder rename '%s' -> '%s' changed filters, resaving\n", olduri, newuri);
 		if (rule_context_save((RuleContext *)fc, user) == -1)
@@ -394,51 +353,52 @@ mail_filter_rename_uri(CamelStore *store, const char *olduri, const char *newuri
 	}
 	
 	g_free(user);
-	g_object_unref (fc);
+	g_object_unref(fc);
+
+	g_free(enewuri);
+	g_free(eolduri);
 }
 
 void
 mail_filter_delete_uri(CamelStore *store, const char *uri)
 {
-	GCompareFunc uri_cmp = CAMEL_STORE_CLASS(CAMEL_OBJECT_GET_CLASS(store))->compare_folder_name;
 	FilterContext *fc;
 	char *user, *system;
 	GList *deleted;
-	
+	char *euri;
+
+	euri = em_uri_from_camel(uri);
+
 	fc = filter_context_new ();
 	user = g_strdup_printf ("%s/mail/filters.xml", mail_component_peek_base_directory (mail_component_peek ()));
 	system = EVOLUTION_PRIVDATADIR "/filtertypes.xml";
 	rule_context_load ((RuleContext *)fc, system, user);
 	
-	deleted = rule_context_delete_uri ((RuleContext *) fc, uri, uri_cmp);
+	deleted = rule_context_delete_uri ((RuleContext *) fc, euri, g_str_equal);
 	if (deleted) {
 		GtkWidget *dialog;
 		GString *s;
 		GList *l;
 		
-		s = g_string_new (_("The following filter rule(s):\n"));
+		s = g_string_new("");
 		l = deleted;
 		while (l) {
 			g_string_append_printf (s, "    %s\n", (char *)l->data);
 			l = l->next;
 		}
-		g_string_append_printf (s, _("Used the removed folder:\n    '%s'\n"
-					     "And have been updated."), uri);
-		
-		dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
-						 GTK_BUTTONS_CLOSE, "%s", s->str);
+
+		dialog = e_error_new(NULL, "mail:filter-updated", s->str, euri, NULL);
 		g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
+		g_string_free(s, TRUE);
+		gtk_widget_show(dialog);
 		
-		g_string_free (s, TRUE);
-		
-		gtk_widget_show (dialog);
-		
-		printf("Folder deleterename '%s' changed filters, resaving\n", uri);
+		printf("Folder deleterename '%s' changed filters, resaving\n", euri);
 		if (rule_context_save ((RuleContext *) fc, user) == -1)
 			g_warning ("Could not write out changed filter rules\n");
 		rule_context_free_uri_list ((RuleContext *) fc, deleted);
 	}
 	
-	g_free (user);
-	g_object_unref (fc);
+	g_free(user);
+	g_object_unref(fc);
+	g_free(euri);
 }
