@@ -19,7 +19,6 @@
  *
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,17 +28,22 @@
 
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkbox.h>
+#include <gtk/gtkcheckbutton.h>
 
-#include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-sound.h>
 
+#include <camel/camel.h>	/* FIXME: this is where camel_init is defined, it shouldn't include everything else */
 #include "camel/camel-filter-driver.h"
+
 #include "filter/filter-context.h"
 #include "filter/filter-filter.h"
-#include "mail.h"
 #include "mail-component.h"
 #include "mail-config.h"
 #include "mail-session.h"
@@ -49,6 +53,7 @@
 #include "e-util/e-passwords.h"
 #include "e-util/e-msgport.h"
 #include "em-junk-filter.h"
+#include "widgets/misc/e-error.h"
 
 #define d(x)
 
@@ -72,11 +77,6 @@ typedef struct _MailSession {
 	EMutex *lock;
 
 	MailAsyncEvent *async;
-
-	/* spamassassin filter options */
-	gboolean sa_local_only;
-	gboolean sa_use_daemon;
-	int sa_daemon_port;
 } MailSession;
 
 typedef struct _MailSessionClass {
@@ -100,8 +100,6 @@ init (MailSession *session)
 {
 	session->lock = e_mutex_new(E_MUTEX_REC);
 	session->async = mail_async_event_new();
-	session->sa_local_only = gconf_client_get_bool (mail_config_get_gconf_client (), "/apps/evolution/mail/junk/sa/local_only", NULL);
-	session->sa_use_daemon = gconf_client_get_bool (mail_config_get_gconf_client (), "/apps/evolution/mail/junk/sa/use_daemon", NULL);
 }
 
 static void
@@ -274,19 +272,16 @@ request_password (struct _pass_msg *m)
 	else
 		title = g_strdup (_("Enter Password"));
 	
-	password_dialog = (GtkDialog *) gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION,
-								GTK_BUTTONS_OK_CANCEL, "%s", m->prompt);
+	password_dialog = (GtkDialog *)e_error_new(NULL, "mail:ask-session-password", m->prompt, NULL);
 	gtk_window_set_title (GTK_WINDOW (password_dialog), title);
-	gtk_dialog_set_default_response (password_dialog, GTK_RESPONSE_OK);
 	g_free (title);
 	
-	gtk_container_set_border_width ((GtkContainer *) password_dialog, 6);
-	
 	m->entry = gtk_entry_new ();
 	gtk_entry_set_visibility ((GtkEntry *) m->entry, !(m->flags & CAMEL_SESSION_PASSWORD_SECRET));
 	g_signal_connect (m->entry, "activate", G_CALLBACK (pass_activate), password_dialog);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (password_dialog)->vbox), m->entry, TRUE, FALSE, 3);
 	gtk_widget_show (m->entry);
+	gtk_widget_grab_focus (m->entry);
 	
 	if ((m->flags & CAMEL_SESSION_PASSWORD_REPROMPT) && m->result) {
 		gtk_entry_set_text ((GtkEntry *) m->entry, m->result);
@@ -459,11 +454,17 @@ user_message_destroy_notify (struct _user_message_msg *m, GObject *deadbeef)
 	message_dialog = NULL;
 }
 
+/* This is kinda ugly/inefficient, but oh well, it works */
+static const char *error_type[] = {
+	"mail:session-message-info", "mail:session-message-warning", "mail:session-message-error",
+	"mail:session-message-info-cancel", "mail:session-message-warning-cancel", "mail:session-message-error-cancel"
+};
+
 static void
 do_user_message (struct _mail_msg *mm)
 {
 	struct _user_message_msg *m = (struct _user_message_msg *)mm;
-	GtkMessageType msg_type;
+	int type;
 	
 	if (!m->ismain && message_dialog != NULL) {
 		e_dlist_addtail (&message_list, (EDListNode *)m);
@@ -472,23 +473,22 @@ do_user_message (struct _mail_msg *mm)
 	
 	switch (m->type) {
 	case CAMEL_SESSION_ALERT_INFO:
-		msg_type = GTK_MESSAGE_INFO;
+		type = 0;
 		break;
 	case CAMEL_SESSION_ALERT_WARNING:
-		msg_type = GTK_MESSAGE_WARNING;
+		type = 1;
 		break;
 	case CAMEL_SESSION_ALERT_ERROR:
-		msg_type = GTK_MESSAGE_ERROR;
+		type = 2;
 		break;
 	default:
-		msg_type = GTK_MESSAGE_INFO;
+		type = 0;
 	}
+
+	if (m->allow_cancel)
+		type += 3;
 	
-	message_dialog = (GtkDialog *) gtk_message_dialog_new (
-		NULL, 0, msg_type,
-		m->allow_cancel ? GTK_BUTTONS_OK_CANCEL : GTK_BUTTONS_OK,
-		"%s", m->prompt);
-	gtk_dialog_set_default_response (message_dialog, m->allow_cancel ? GTK_RESPONSE_CANCEL : GTK_RESPONSE_OK);
+	message_dialog = (GtkDialog *)e_error_new(NULL, error_type[type], m->prompt, NULL);
 	g_object_set ((GObject *) message_dialog, "allow_shrink", TRUE, "allow_grow", TRUE, NULL);
 	
 	/* We only need to wait for the result if we allow cancel otherwise show but send result back instantly */
@@ -637,18 +637,18 @@ main_get_filter_driver (CamelSession *session, const char *type, CamelException
 	if ((!strcmp (type, FILTER_SOURCE_INCOMING) || !strcmp (type, FILTER_SOURCE_JUNKTEST))
 	    && camel_session_check_junk (session)) {
 		/* implicit junk check as 1st rule */
-		camel_filter_driver_add_rule (driver, "Junk check", "(junk-test)", "(begin (set-system-flag \"junk\"))");
+		camel_filter_driver_add_rule (driver, "Junk check", "(junk-test)", "(begin (set-system-flag \"junk\")(set-system-flag \"seen\"))");
 	}
 
-	if (strcmp (type, FILTER_SOURCE_JUNKTEST)) {
+	if (strcmp (type, FILTER_SOURCE_JUNKTEST) != 0) {
 		GString *fsearch, *faction;
-
+		
 		fsearch = g_string_new ("");
 		faction = g_string_new ("");
-
+		
 		if (!strcmp (type, FILTER_SOURCE_DEMAND))
 			type = FILTER_SOURCE_INCOMING;
-
+		
 		/* add the user-defined rules next */
 		while ((rule = rule_context_next_rule (fc, rule, type))) {
 			g_string_truncate (fsearch, 0);
@@ -658,7 +658,7 @@ main_get_filter_driver (CamelSession *session, const char *type, CamelException
 			filter_filter_build_action ((FilterFilter *) rule, faction);
 			camel_filter_driver_add_rule (driver, rule->name, fsearch->str, faction->str);
 		}
-
+		
 		g_string_free (fsearch, TRUE);
 		g_string_free (faction, TRUE);
 	}
@@ -765,42 +765,6 @@ mail_session_forget_password (const char *key)
 	e_passwords_forget_password ("Mail", key);
 }
 
-gboolean
-mail_session_get_sa_local_only ()
-{
-	return MAIL_SESSION (session)->sa_local_only;
-}
-
-void
-mail_session_set_sa_local_only (gboolean value)
-{
-	MAIL_SESSION (session)->sa_local_only = value;
-}
-
-gboolean
-mail_session_get_sa_use_daemon ()
-{
-	return MAIL_SESSION (session)->sa_use_daemon;
-}
-
-void
-mail_session_set_sa_use_daemon (gboolean value)
-{
-	MAIL_SESSION (session)->sa_use_daemon = value;
-}
-
-int
-mail_session_get_sa_daemon_port ()
-{
-	return MAIL_SESSION (session)->sa_daemon_port;
-}
-
-void
-mail_session_set_sa_daemon_port (int value)
-{
-	MAIL_SESSION (session)->sa_daemon_port = value;
-}
-
 static void
 mail_session_check_junk_notify (GConfClient *gconf, guint id, GConfEntry *entry, CamelSession *session)
 {
@@ -814,15 +778,6 @@ mail_session_check_junk_notify (GConfClient *gconf, guint id, GConfEntry *entry,
 		key ++;
 		if (!strcmp (key, "check_incoming"))
 			camel_session_set_check_junk (session, gconf_value_get_bool (gconf_entry_get_value (entry)));
-		else if (!strcmp (key, "check_incoming_imap"))
-			camel_session_set_check_junk_for_imap (session, gconf_value_get_bool (gconf_entry_get_value (entry)));
-		else if (!strcmp (key, "local_only"))
-			mail_session_set_sa_local_only (gconf_value_get_bool (gconf_entry_get_value (entry)));
-		else if (!strcmp (key, "use_daemon"))
-			mail_session_set_sa_use_daemon (gconf_value_get_bool (gconf_entry_get_value (entry)));
-		else if (!strcmp (key, "daemon_port"))
-			mail_session_set_sa_daemon_port (gconf_value_get_int (gconf_entry_get_value (entry)));
-
 	}
 }
 
@@ -843,8 +798,6 @@ mail_session_init (const char *base_directory)
 	gconf = mail_config_get_gconf_client ();
 	gconf_client_add_dir (gconf, "/apps/evolution/mail/junk", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	camel_session_set_check_junk (session, gconf_client_get_bool (gconf, "/apps/evolution/mail/junk/check_incoming", NULL));
-	camel_session_set_check_junk_for_imap (session, gconf_client_get_bool (gconf, "/apps/evolution/mail/junk/check_incoming_imap", NULL));
-	mail_session_set_sa_daemon_port (gconf_client_get_int (gconf, "/apps/evolution/mail/junk/sa/daemon_port", NULL));
 	session_check_junk_notify_id = gconf_client_notify_add (gconf, "/apps/evolution/mail/junk",
 								(GConfClientNotifyFunc) mail_session_check_junk_notify,
 								session, NULL, NULL);
