@@ -28,14 +28,14 @@
 #include "e-shell-settings-dialog.h"
 
 #include "e-corba-config-page.h"
-
-#include "e-util/e-lang-utils.h"
+#include <e-util/e-icon-factory.h>
 
 #include <gal/util/e-util.h>
 
 #include <bonobo/bonobo-widget.h>
 #include <bonobo/bonobo-exception.h>
-#include <liboaf/liboaf.h>
+
+#include <bonobo-activation/bonobo-activation.h>
 
 #include <string.h>
 
@@ -56,14 +56,26 @@ struct _EShellSettingsDialogPrivate {
 static void
 set_dialog_size (EShellSettingsDialog *dialog)
 {
-	GdkFont *font;
+	PangoLayout *layout;
+	PangoContext *context;
+	PangoFontMetrics *metrics;
 	int width, height;
 
-	font = GTK_WIDGET (dialog)->style->font;
-	width = gdk_string_width (font, "M") * 72;
-	height = (font->ascent + font->descent) * 35;
+	layout = gtk_widget_create_pango_layout (GTK_WIDGET (dialog), "M");
+	context = pango_layout_get_context (layout);
+	metrics = pango_context_get_metrics (context,
+					     gtk_widget_get_style (GTK_WIDGET (dialog))->font_desc,
+					     pango_context_get_language (context));
 
-	gtk_widget_set_usize (GTK_WIDGET (dialog), width, height);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+
+	width *= 60;
+	height = PANGO_PIXELS (pango_font_metrics_get_ascent (metrics)
+			       + pango_font_metrics_get_descent (metrics)) * 30;
+
+	gtk_window_set_default_size((GtkWindow *)dialog, width, height);
+	g_object_unref (layout);
+	pango_font_metrics_unref (metrics);
 }
 
 
@@ -73,7 +85,7 @@ struct _Page {
 	char *title;
 	char *description;
 	GdkPixbuf *icon;
-	OAF_Property *type;
+	Bonobo_ActivationProperty *type;
 	int priority;
 	EConfigPage *page_widget;
 };
@@ -83,18 +95,18 @@ static Page *
 page_new (const char *title,
 	  const char *description,
 	  GdkPixbuf *icon,
-	  OAF_Property *type,
+	  Bonobo_ActivationProperty *type,
 	  int priority,
 	  EConfigPage *page_widget)
 {
 	Page *page;
 
 	if (icon != NULL)
-		gdk_pixbuf_ref (icon);
+		g_object_ref (icon);
 
 	page = g_new (Page, 1);
-	page->title       = g_strdup (_(title));
-	page->description = g_strdup (_(description));
+	page->title       = g_strdup (title);
+	page->description = g_strdup (description);
 	page->icon        = icon;
 	page->type        = type;
 	page->priority    = priority;
@@ -110,7 +122,7 @@ page_free (Page *page)
 	g_free (page->description);
 
 	if (page->icon != NULL)
-		gdk_pixbuf_unref (page->icon);
+		g_object_unref (page->icon);
 
 	g_free (page);
 }
@@ -141,8 +153,9 @@ static void
 load_pages (EShellSettingsDialog *dialog)
 {
 	EShellSettingsDialogPrivate *priv;
-	OAF_ServerInfoList *control_list;
+	Bonobo_ServerInfoList *control_list;
 	CORBA_Environment ev;
+	const GList *l;
 	GSList *language_list;
 	GList *page_list;
 	GList *p;
@@ -152,57 +165,60 @@ load_pages (EShellSettingsDialog *dialog)
 	
 	CORBA_exception_init (&ev);
 
-	control_list = oaf_query ("defined(evolution:config_item:title)", NULL, &ev);
+	control_list = bonobo_activation_query ("defined(evolution2:config_item:title)", NULL, &ev);
 	if (ev._major != CORBA_NO_EXCEPTION || control_list == NULL) {
-		g_warning ("Cannot load configuration pages -- %s", ev._repo_id);
+		g_warning ("Cannot load configuration pages -- %s", BONOBO_EX_REPOID (&ev));
 		CORBA_exception_free (&ev);
 		return;
 	}
 
-	language_list = e_get_language_list ();
+	CORBA_exception_free (&ev);
+
+	/* Great, one uses GList the other GSList (!) */
+	l = gnome_i18n_get_language_list("LC_MESSAGES");
+	for (language_list=NULL;l;l=l->next)
+		language_list = g_slist_append(language_list, l->data);
 
 	page_list = NULL;
 	for (i = 0; i < control_list->_length; i ++) {
 		CORBA_Object corba_object;
-		OAF_ServerInfo *info;
+		Bonobo_ServerInfo *info;
 		const char *title;
 		const char *description;
 		const char *icon_path;
 		const char *priority_string;
-		OAF_Property *type;
+		Bonobo_ActivationProperty *type;
 		int priority;
 		GdkPixbuf *icon;
 
+		CORBA_exception_init (&ev);
+
 		info = & control_list->_buffer[i];
 
-		title       	= oaf_server_info_prop_lookup (info, "evolution:config_item:title", language_list);
-		description 	= oaf_server_info_prop_lookup (info, "evolution:config_item:description", language_list);
-		icon_path   	= oaf_server_info_prop_lookup (info, "evolution:config_item:icon_name", NULL);
-		type            = oaf_server_info_prop_find (info, "evolution:config_item:type");
-		priority_string = oaf_server_info_prop_lookup (info, "evolution:config_item:priority", NULL);
+		title       	= bonobo_server_info_prop_lookup (info, "evolution2:config_item:title", language_list);
+		description 	= bonobo_server_info_prop_lookup (info, "evolution2:config_item:description", language_list);
+		icon_path   	= bonobo_server_info_prop_lookup (info, "evolution2:config_item:icon_name", NULL);
+		type            = bonobo_server_info_prop_find   (info, "evolution2:config_item:type");
+		priority_string = bonobo_server_info_prop_lookup (info, "evolution2:config_item:priority", NULL);
 
 		if (icon_path == NULL) {
 			icon = NULL;
 		} else {
 			if (g_path_is_absolute (icon_path)) {
-				icon = gdk_pixbuf_new_from_file (icon_path);
+				icon = gdk_pixbuf_new_from_file (icon_path, NULL);
 			} else {
-				char *real_icon_path;
-
-				real_icon_path = g_concat_dir_and_file (EVOLUTION_IMAGES, icon_path);
-				icon = gdk_pixbuf_new_from_file (real_icon_path);
-				g_free (real_icon_path);
+				icon = e_icon_factory_get_icon (icon_path, E_ICON_SIZE_DIALOG);
 			}
 		}
 
-		if (type != NULL && type->v._d != OAF_P_STRINGV)
+		if (type != NULL && type->v._d != Bonobo_ACTIVATION_P_STRINGV)
 			type = NULL;
 		if (priority_string == NULL)
 			priority = 0xffff;
 		else
 			priority = atoi (priority_string);
 
-		corba_object = oaf_activate_from_id ((char *) info->iid, 0, NULL, &ev);
+		corba_object = bonobo_activation_activate_from_id ((char *) info->iid, 0, NULL, &ev);
 
 		if (! BONOBO_EX (&ev)) {
 			Page *page;
@@ -212,12 +228,17 @@ load_pages (EShellSettingsDialog *dialog)
 
 			page_list = g_list_prepend (page_list, page);
 		} else {
-			g_warning ("Cannot activate %s -- %s", info->iid, BONOBO_EX_ID (&ev));
+			char *bonobo_ex_text = bonobo_exception_get_text (&ev);
+			g_warning ("Cannot activate %s -- %s", info->iid, bonobo_ex_text);
+			g_free (bonobo_ex_text);
 		}
 
 		if (icon != NULL)
-			gdk_pixbuf_unref (icon);
+			g_object_unref (icon);
+
+		CORBA_exception_free (&ev);
 	}
+	g_slist_free(language_list);
 
 	page_list = sort_page_list (page_list);
 	for (p = page_list, i = 0; p != NULL; p = p->next, i++) {
@@ -232,7 +253,7 @@ load_pages (EShellSettingsDialog *dialog)
 						page->page_widget);
 
 		if (page->type != NULL) {
-			GNOME_stringlist list = page->type->v._u.value_stringv;
+			Bonobo_StringList list = page->type->v._u.value_stringv;
 			
 			for (j = 0; j < list._length; j++) {
 				if (g_hash_table_lookup (priv->types, list._buffer[j]) == NULL)
@@ -246,25 +267,7 @@ load_pages (EShellSettingsDialog *dialog)
 	}
 
 	g_list_free (page_list);
-	e_free_language_list (language_list);
 	CORBA_free (control_list);
-
-	CORBA_exception_free (&ev);
-}
-
-
-/* GtkWidget methods.  */
-
-static void
-impl_realize (GtkWidget *widget)
-{
-	EShellSettingsDialog *dialog;
-
-	dialog = E_SHELL_SETTINGS_DIALOG (widget);
-
-	set_dialog_size (dialog);
-
-	(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
 }
 
 
@@ -279,7 +282,7 @@ destroy_type_entry (gpointer key, gpointer value, gpointer data)
 }
 		
 static void
-impl_destroy (GtkObject *object)
+impl_finalize (GObject *object)
 {
 	EShellSettingsDialog *dialog;
 	EShellSettingsDialogPrivate *priv;
@@ -292,23 +295,19 @@ impl_destroy (GtkObject *object)
 
 	g_free (priv);
 	
-	(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 
 static void
 class_init (EShellSettingsDialog *class)
 {
-	GtkObjectClass *object_class;
-	GtkWidgetClass *widget_class;
+	GObjectClass *object_class;
 
-	object_class = GTK_OBJECT_CLASS (class);
-	object_class->destroy = impl_destroy;
+	object_class = G_OBJECT_CLASS (class);
+	object_class->finalize = impl_finalize;
 
-	widget_class = GTK_WIDGET_CLASS (class);
-	widget_class->realize = impl_realize;
-
-	parent_class = gtk_type_class (PARENT_TYPE);
+	parent_class = g_type_class_ref(PARENT_TYPE);
 }
 
 static void
@@ -322,8 +321,10 @@ init (EShellSettingsDialog *dialog)
 	dialog->priv = priv;
 
 	load_pages (dialog);
-
+	set_dialog_size (dialog);
+	
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Evolution Settings"));
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 }
 
 
@@ -332,7 +333,7 @@ e_shell_settings_dialog_new ()
 {
 	EShellSettingsDialog *new;
 
-	new = gtk_type_new (e_shell_settings_dialog_get_type ());
+	new = g_object_new (e_shell_settings_dialog_get_type (), NULL);
 
 	return GTK_WIDGET (new);
 }
