@@ -31,8 +31,6 @@
 #include "e-shell-marshal.h"
 #include "e-sidebar.h"
 
-#include "e-util/e-lang-utils.h"
-
 #include <gal/util/e-util.h>
 
 #include <gtk/gtkbutton.h>
@@ -84,6 +82,9 @@ struct _EShellWindowPrivate {
 	/* All the ComponentViews.  */
 	GSList *component_views;
 
+	/* The paned widget for the sidebar and component views */
+	GtkWidget *paned;
+
 	/* The sidebar.  */
 	GtkWidget *sidebar;
 
@@ -277,6 +278,10 @@ switch_view (EShellWindow *window, ComponentView *component_view)
 {
 	EShellWindowPrivate *priv = window->priv;
 	GConfClient *gconf_client = gconf_client_get_default ();
+	EComponentRegistry *registry = e_shell_peek_component_registry (window->priv->shell);
+	EComponentInfo *info = e_component_registry_peek_info (registry,
+							       component_view->component_id);
+	char *title;
 
 	if (component_view->sidebar_widget == NULL) {
 		init_view (window, component_view);
@@ -291,6 +296,13 @@ switch_view (EShellWindow *window, ComponentView *component_view)
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->statusbar_notebook), component_view->notebook_page_num);
 	}
 
+	title = g_strdup_printf ("Evolution - %s", info->button_label);
+	gtk_window_set_title (GTK_WINDOW (window), title);
+	g_free (title);
+
+	if (info->button_icon)
+		gtk_window_set_icon (GTK_WINDOW (window), info->button_icon);
+
 	gconf_client_set_string (gconf_client, "/apps/evolution/shell/view_defaults/component_id",
 				 (component_view->component_alias != NULL
 				  ? component_view->component_alias
@@ -321,20 +333,20 @@ update_offline_toggle_status (EShellWindow *window)
 		icon_pixmap = online_pixmap;
 		icon_mask   = online_mask;
 		sensitive   = TRUE;
-		tooltip     = _("Ximian Evolution is currently online.  "
+		tooltip     = _("Evolution is currently online.  "
 				"Click on this button to work offline.");
 		break;
 	case E_SHELL_LINE_STATUS_GOING_OFFLINE:
 		icon_pixmap = online_pixmap;
 		icon_mask   = online_mask;
 		sensitive   = FALSE;
-		tooltip     = _("Ximian Evolution is in the process of going offline.");
+		tooltip     = _("Evolution is in the process of going offline.");
 		break;
 	case E_SHELL_LINE_STATUS_OFFLINE:
 		icon_pixmap = offline_pixmap;
 		icon_mask   = offline_mask;
 		sensitive   = TRUE;
-		tooltip     = _("Ximian Evolution is currently offline.  "
+		tooltip     = _("Evolution is currently offline.  "
 				"Click on this button to work online.");
 		break;
 	default:
@@ -369,7 +381,7 @@ sidebar_button_selected_callback (ESidebar *sidebar,
 				  EShellWindow *window)
 {
 	EShellWindowPrivate *priv = window->priv;
-	ComponentView *component_view;
+	ComponentView *component_view = NULL;
 	GSList *p;
 
 	for (p = priv->component_views; p != NULL; p = p->next) {
@@ -539,22 +551,35 @@ setup_status_bar (EShellWindow *window)
 	g_signal_connect (ui_engine, "remove_hint", G_CALLBACK (ui_engine_remove_hint_callback), window);
 }
 
+static void
+menu_component_selected (BonoboUIComponent *uic,
+			 EShellWindow *window,
+			 const char *path)
+{
+	char *component_id;
+
+	component_id = strchr(path, '-');
+	if (component_id)
+		e_shell_window_switch_to_component (window, component_id+1);
+}
+
 static void
 setup_widgets (EShellWindow *window)
 {
 	EShellWindowPrivate *priv = window->priv;
 	EComponentRegistry *registry = e_shell_peek_component_registry (priv->shell);
-	GtkWidget *paned;
+	GConfClient *gconf_client = gconf_client_get_default ();
 	GtkWidget *contents_vbox;
 	GSList *p;
+	GString *xml;
 	int button_id;
 
-	paned = gtk_hpaned_new ();
+	priv->paned = gtk_hpaned_new ();
 
 	priv->sidebar = e_sidebar_new ();
 	g_signal_connect (priv->sidebar, "button_selected",
 			  G_CALLBACK (sidebar_button_selected_callback), window);
-	gtk_paned_pack1 (GTK_PANED (paned), priv->sidebar, FALSE, FALSE);
+	gtk_paned_pack1 (GTK_PANED (priv->paned), priv->sidebar, FALSE, FALSE);
 
 	priv->sidebar_notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->sidebar_notebook), FALSE);
@@ -564,25 +589,58 @@ setup_widgets (EShellWindow *window)
 	priv->view_notebook = gtk_notebook_new ();
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (priv->view_notebook), FALSE);
 	gtk_notebook_set_show_border (GTK_NOTEBOOK (priv->view_notebook), FALSE);
-	gtk_paned_pack2 (GTK_PANED (paned), priv->view_notebook, TRUE, TRUE);
+	gtk_paned_pack2 (GTK_PANED (priv->paned), priv->view_notebook, TRUE, TRUE);
 
-	gtk_paned_set_position (GTK_PANED (paned), 200);
+	gtk_paned_set_position (GTK_PANED (priv->paned),
+				gconf_client_get_int (gconf_client, "/apps/evolution/shell/view_defaults/folder_bar/width", NULL));
 
 	button_id = 0;
+	xml = g_string_new("");
 	for (p = e_component_registry_peek_list (registry); p != NULL; p = p->next) {
+		char *tmp;
 		EComponentInfo *info = p->data;
 		ComponentView *view = component_view_new (info->id, info->alias, button_id);
 
 		window->priv->component_views = g_slist_prepend (window->priv->component_views, view);
 		e_sidebar_add_button (E_SIDEBAR (priv->sidebar), info->button_label, info->button_icon, button_id);
 
+		g_string_printf(xml, "SwitchComponent-%s", info->alias);
+		bonobo_ui_component_add_verb (e_shell_window_peek_bonobo_ui_component (window),
+					      xml->str,
+					      (BonoboUIVerbFn)menu_component_selected,
+					      window);
+
+		g_string_printf(xml, "<submenu name=\"View\">"
+				"<submenu name=\"Window\">"
+				"<placeholder name=\"WindowComponent\">"
+				"<menuitem name=\"SwitchComponent-%s\" "
+				"verb=\"\" label=\"%s\" accel=\"%s\" tip=\"",
+				info->alias,
+				info->menu_label,
+				info->menu_accelerator);
+		g_string_append_printf(xml, _("Switch to %s"), info->button_label);
+		tmp = bonobo_ui_util_pixbuf_to_xml (info->menu_icon),
+		g_string_append_printf(xml, "\" pixtype=\"pixbuf\" pixname=\"%s\"/>"
+				       "</placeholder></submenu></submenu>\n",
+				       tmp);
+		g_free(tmp);
+		bonobo_ui_component_set_translate (e_shell_window_peek_bonobo_ui_component (window),
+						   "/menu",
+						   xml->str,
+						   NULL);
+		g_string_printf(xml, "<cmd name=\"SwitchComponent-%s\"/>\n", info->alias);
+		bonobo_ui_component_set_translate (e_shell_window_peek_bonobo_ui_component (window),
+						   "/commands",
+						   xml->str,
+						   NULL);
 		button_id ++;
 	}
+	g_string_free(xml, TRUE);
 
 	setup_status_bar (window);
 
 	contents_vbox = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (contents_vbox), paned, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (contents_vbox), priv->paned, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (contents_vbox), priv->status_bar, FALSE, TRUE, 0);
 	gtk_widget_show_all (contents_vbox);
 
@@ -590,6 +648,7 @@ setup_widgets (EShellWindow *window)
 	gtk_widget_hide (priv->menu_hint_label);
 
 	bonobo_window_set_contents (BONOBO_WINDOW (window), contents_vbox);
+	g_object_unref (gconf_client);
 }
 
 
@@ -679,7 +738,7 @@ e_shell_window_new (EShell *shell,
 
 	if (bonobo_window_construct (BONOBO_WINDOW (window),
 				     bonobo_ui_container_new (),
-				     "evolution", "Ximian Evolution") == NULL) {
+				     "evolution", "Evolution") == NULL) {
 		g_object_unref (window);
 		g_object_unref (gconf_client);
 		return NULL;
@@ -733,8 +792,6 @@ e_shell_window_new (EShell *shell,
 				     gconf_client_get_int (gconf_client, "/apps/evolution/shell/view_defaults/width", NULL),
 				     gconf_client_get_int (gconf_client, "/apps/evolution/shell/view_defaults/height", NULL));
 
-	e_user_creatable_items_handler_attach_menus (e_shell_peek_user_creatable_items_handler (shell), window);
-
 	g_object_unref (gconf_client);
 	return GTK_WIDGET (window);
 }
@@ -810,6 +867,9 @@ e_shell_window_save_defaults (EShellWindow *window)
 	gconf_client_set_int (client, "/apps/evolution/shell/view_defaults/height",
 			      GTK_WIDGET (window)->allocation.height, NULL);
 
+	gconf_client_set_int (client, "/apps/evolution/shell/view_defaults/folder_bar/width",
+			      gtk_paned_get_position (GTK_PANED (window->priv->paned)), NULL);
+
 	g_object_unref (client);
 }
 
@@ -818,7 +878,7 @@ e_shell_window_show_settings (EShellWindow *window)
 {
 	g_return_if_fail (E_IS_SHELL_WINDOW (window));
 
-	e_shell_show_settings (window->priv->shell, NULL, window);
+	e_shell_show_settings (window->priv->shell, window->priv->current_view ? window->priv->current_view->component_alias : NULL, window);
 }
 
 
