@@ -27,6 +27,9 @@
 #include "e-shell.h"
 
 #include "e-util/e-dialog-utils.h"
+#include "e-util/e-bconf-map.h"
+#include "e-util/e-fsutils.h"
+#include "widgets/misc/e-error.h"
 
 #include "e-shell-constants.h"
 #include "e-shell-offline-handler.h"
@@ -76,7 +79,6 @@ struct _EShellPrivate {
 
 	GList *windows;
 
-	EUserCreatableItemsHandler *user_creatable_items_handler;
 	/* EUriSchemaRegistry *uri_schema_registry; FIXME */
 	EComponentRegistry *component_registry;
 
@@ -94,6 +96,9 @@ struct _EShellPrivate {
 	/* Settings Dialog */
 	GtkWidget *settings_dialog;
 
+	/* If we're quitting and things are still busy, a timeout handler */
+	guint quit_timeout;
+
 	/* Whether the shell is succesfully initialized.  This is needed during
 	   the start-up sequence, to avoid CORBA calls to do make wrong things
 	   to happen while the shell is initializing.  */
@@ -336,11 +341,6 @@ impl_dispose (GObject *object)
 
 	priv->is_initialized = FALSE;
 
-	if (priv->user_creatable_items_handler != NULL) {
-		g_object_unref (priv->user_creatable_items_handler);
-		priv->user_creatable_items_handler = NULL;
-	}
-
 #if 0				/* FIXME */
 	if (priv->uri_schema_registry != NULL) {
 		g_object_unref (priv->uri_schema_registry);
@@ -348,11 +348,21 @@ impl_dispose (GObject *object)
 	}
 #endif
 
+	if (priv->component_registry != NULL) {
+		g_object_unref (priv->component_registry);
+		priv->component_registry = NULL;
+	}
+
 	if (priv->offline_handler != NULL) {
 		g_object_unref (priv->offline_handler);
 		priv->offline_handler = NULL;
 	}
 
+	if (priv->quit_timeout) {
+		g_source_remove(priv->quit_timeout);
+		priv->quit_timeout = 0;
+	}
+
 	for (p = priv->windows; p != NULL; p = p->next) {
 		EShellWindow *window;
 
@@ -456,11 +466,100 @@ e_shell_init (EShell *shell)
 	priv = g_new0 (EShellPrivate, 1);
 	priv->line_status                  = E_SHELL_LINE_STATUS_OFFLINE;
 	priv->component_registry           = e_component_registry_new ();
-	priv->user_creatable_items_handler = e_user_creatable_items_handler_new (priv->component_registry);
 
 	shell->priv = priv;
 }
 
+static void
+detect_version (GConfClient *gconf, int *major, int *minor, int *revision)
+{
+	char *val, *evolution_dir;
+	struct stat st;
+	
+	evolution_dir = g_build_filename (g_get_home_dir (), "evolution", NULL);
+	
+	val = gconf_client_get_string(gconf, "/apps/evolution/version", NULL);	
+	if (val) {
+		/* Since 1.4.0 We've been keeping the version key in gconf */
+		sscanf(val, "%u.%u.%u", major, minor, revision);
+		g_free(val);
+	} else if (lstat (evolution_dir, &st) != 0 || !S_ISDIR (st.st_mode)) {
+		/* If ~/evolution does not exit or is not a directory it must be a new installation */
+		*major = 0;
+		*minor = 0;
+		*revision = 0;
+	} else {
+		xmlDocPtr config_doc = NULL;
+		xmlNodePtr source;
+		char *tmp;
+
+		tmp = g_build_filename (evolution_dir, "config.xmldb", NULL);
+		if (lstat(tmp, &st) == 0
+		    && S_ISREG(st.st_mode))
+			config_doc = xmlParseFile (tmp);
+		g_free (tmp);
+
+		tmp = NULL;
+		if (config_doc
+		    && (source = e_bconf_get_path (config_doc, "/Shell"))
+		    && (tmp = e_bconf_get_value (source, "upgrade_from_1_0_to_1_2_performed"))
+		    && tmp[0] == '1' ) {
+			*major = 1;
+			*minor = 2;
+			*revision = 0;
+		} else {
+			*major = 1;
+			*minor = 0;
+			*revision = 0;
+		}
+		if (tmp)
+			xmlFree(tmp);
+		if (config_doc)
+			xmlFreeDoc (config_doc);		
+	}
+
+	g_free (evolution_dir);
+}
+
+static void
+attempt_upgrade (EShell *shell)
+{
+	GConfClient *gconf_client;
+	int major = 0, minor = 0, revision = 0;
+	char *version_string;
+
+	gconf_client = gconf_client_get_default ();
+
+	detect_version (gconf_client, &major, &minor, &revision);
+
+	/* if upgrading from < 1.5, we need to copy most data from ~/evolution to ~/.evolution */
+	if (major == 1 && minor < 5) {
+		char *path;
+		long size, space;
+
+		path = g_build_filename(g_get_home_dir(), "evolution", NULL);
+		size = e_fsutils_usage(path);
+		g_free(path);
+		space = e_fsutils_avail(g_get_home_dir());
+		if (size != -1 && space != -1 && space < size) {
+			char *required = g_strdup_printf(_("%ld KB"), size);
+			char *have = g_strdup_printf(_("%ld KB"), space);
+
+			e_error_run(NULL, "shell:upgrade-nospace", required, have, NULL);
+			g_free(required);
+			g_free(have);
+			_exit(0);
+		}
+	}
+
+	if (!e_shell_attempt_upgrade (shell, major, minor, revision))
+		_exit(0);
+
+	version_string = g_strdup_printf ("%s.%s", BASE_VERSION, UPGRADE_REVISION);
+	gconf_client_set_string (gconf_client, "/apps/evolution/version", version_string, NULL);
+	g_object_unref (gconf_client);
+	g_free (version_string);
+}
 
 /**
  * e_shell_construct:
@@ -477,12 +576,11 @@ e_shell_construct (EShell *shell,
 		   const char *iid,
 		   EShellStartupLineMode startup_line_mode)
 {
-	GtkWidget *splash = NULL;
 	EShellPrivate *priv;
 	CORBA_Object corba_object;
 	gboolean start_online;
-
-	g_return_val_if_fail (shell != NULL, E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
+	GSList *component;
+	
 	g_return_val_if_fail (E_IS_SHELL (shell), E_SHELL_CONSTRUCT_RESULT_INVALIDARG);
 	g_return_val_if_fail (startup_line_mode == E_SHELL_STARTUP_LINE_MODE_CONFIG
 			      || startup_line_mode == E_SHELL_STARTUP_LINE_MODE_ONLINE
@@ -501,11 +599,25 @@ e_shell_construct (EShell *shell,
 		return E_SHELL_CONSTRUCT_RESULT_CANNOTREGISTER;
 
 	while (gtk_events_pending ())
-		gtk_main_iteration ();
+		gtk_main_iteration ();	
 	
-	if (splash)
-		gtk_widget_destroy (splash);
+	/* activate all the components */
+	component = e_component_registry_peek_list (shell->priv->component_registry);
+	while (component != NULL) {
+		const EComponentInfo *info = component->data;
+		CORBA_Environment ev;
+		
+		CORBA_exception_init (&ev);
+		
+		e_component_registry_activate (shell->priv->component_registry, info->id, &ev);
+		
+		CORBA_exception_free (&ev);
+		
+		component = component->next;
+	}
 	
+	attempt_upgrade(shell);
+
 	if (e_shell_startup_wizard_create () == FALSE) {
 		bonobo_object_unref (BONOBO_OBJECT (shell));
 		exit (0);
@@ -568,7 +680,6 @@ e_shell_new (EShellStartupLineMode startup_line_mode,
 	return new;
 }
 
-
 /**
  * e_shell_attempt_upgrade:
  * @shell: 
@@ -580,19 +691,18 @@ e_shell_new (EShellStartupLineMode startup_line_mode,
  * upgrade from @from_version is unsupported).
  **/
 gboolean
-e_shell_attempt_upgrade (EShell *shell,
-			 const char *from_version)
+e_shell_attempt_upgrade (EShell *shell, int major, int minor, int revision)
 {
 	GSList *component_infos, *p;
-	int major, minor, revision;
 	int current_major, current_minor, current_revision;
 	gboolean success;
+	int res;
 
 	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
-	g_return_val_if_fail (from_version != NULL, FALSE);
 
-	sscanf (from_version, "%u.%u.%u", &major, &minor, &revision);
-	sscanf (VERSION, "%u.%u.%u", &current_major, &current_minor, &current_revision);
+	sscanf (BASE_VERSION, "%u.%u", &current_major, &current_minor);
+
+	current_revision = atoi (UPGRADE_REVISION);
 
 	if (! (current_major > major
 	       || (current_major == major && current_minor > minor)
@@ -603,39 +713,41 @@ e_shell_attempt_upgrade (EShell *shell,
 	success = TRUE;
 
 	component_infos = e_component_registry_peek_list (shell->priv->component_registry);
-	for (p = component_infos; p != NULL; p = p->next) {
+	for (p = component_infos; success && p != NULL; p = p->next) {
 		const EComponentInfo *info = p->data;
 		CORBA_Environment ev;
 		gboolean component_upgraded;
 
 		CORBA_exception_init (&ev);
-
-		component_upgraded = GNOME_Evolution_Component_upgradeFromVersion (info->iface, major, minor, revision, &ev);
+		
+		GNOME_Evolution_Component_upgradeFromVersion (info->iface, major, minor, revision, &ev);
 		
 		if (BONOBO_EX (&ev)) {
 			char *exception_text;
+			CORBA_char *id = CORBA_exception_id(&ev);
 
-			// Ignore components that do not implement this version, it might just mean that they don't need an
-			// upgrade path.
-			if (strcmp (ev._id, ex_CORBA_NO_IMPLEMENT) == 0) {
-				CORBA_exception_free (&ev);
-				continue;
-			}
+			if (strcmp (id, ex_CORBA_NO_IMPLEMENT) == 0) {
+				/* Ignore components that do not implement this version, it
+				   might just mean that they don't need an upgrade path. */
+			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UpgradeFailed) == 0) {
+				GNOME_Evolution_Component_UpgradeFailed *ex = CORBA_exception_value(&ev);
 
-			exception_text = bonobo_exception_get_text (&ev);
-			g_warning ("Upgrade of component \"%s\" from version %s failed with exception %s",
-				   info->alias, from_version, exception_text);
-			g_free (exception_text);
-			CORBA_exception_free (&ev);
-			success = FALSE;
-		} else {
-			CORBA_exception_free (&ev);
-			if (! component_upgraded) {
-				g_warning ("Component \"%s\" could not upgrade configuration from version \"%s\"",
-					   info->alias, from_version);
-				success = FALSE;
+				res = e_error_run(NULL, "shell:upgrade-failed", ex->what, ex->why, NULL);
+				if (res == GTK_RESPONSE_CANCEL)
+					success = FALSE;
+			} else if (strcmp (id,  ex_GNOME_Evolution_Component_UnsupportedVersion) == 0) {
+				/* This is non-fatal */
+				/* DO WE CARE??? */
+				printf("Upgrade of component failed, unsupported prior version\n");
+			} else {
+				exception_text = bonobo_exception_get_text (&ev);
+				res = e_error_run(NULL, "shell:upgrade-failed", exception_text, _("Uknown system error."), NULL);
+				g_free (exception_text);
+				if (res == GTK_RESPONSE_CANCEL)
+					success = FALSE;
 			}
 		}
+		CORBA_exception_free (&ev);
 	}
 
 	return success;
@@ -690,21 +802,9 @@ e_shell_request_close_window (EShell *shell,
 		return TRUE;
 	}
 
-	if (shell->priv->preparing_to_quit)
-		return FALSE;
-
-	/* If it's the last window, save settings and ask for confirmation before
-	   quitting. */
-
-	e_shell_window_save_defaults (shell_window);
-
-	if (e_shell_prepare_for_quit (shell))
-		return TRUE;
-	else
-		return FALSE;
+	return e_shell_quit(shell);
 }
 
-
 #if 0				/* FIXME */
 /**
  * e_shell_peek_uri_schema_registry:
@@ -797,7 +897,6 @@ e_shell_close_all_windows (EShell *shell)
 	}
 }
 
-
 /**
  * e_shell_get_line_status:
  * @shell: A pointer to an EShell object.
@@ -901,7 +1000,7 @@ e_shell_go_online (EShell *shell,
 	EShellPrivate *priv;
 	GSList *component_infos;
 	GSList *p;
-
+	
 	g_return_if_fail (shell != NULL);
 	g_return_if_fail (E_IS_SHELL (shell));
 	g_return_if_fail (action_window == NULL || E_IS_SHELL_WINDOW (action_window));
@@ -915,15 +1014,15 @@ e_shell_go_online (EShell *shell,
 		GNOME_Evolution_Offline offline_interface;
 
 		CORBA_exception_init (&ev);
-
+		
 		offline_interface = Bonobo_Unknown_queryInterface (info->iface, "IDL:GNOME/Evolution/Offline:1.0", &ev);
 		if (ev._major != CORBA_NO_EXCEPTION || offline_interface == CORBA_OBJECT_NIL) {
-			CORBA_exception_free (&ev);
+				CORBA_exception_free (&ev);
 			continue;
 		}
-
+		
 		CORBA_exception_free (&ev);
-
+		
 		GNOME_Evolution_Offline_goOnline (offline_interface, &ev);
 		if (ev._major != CORBA_NO_EXCEPTION)
 			g_warning ("Error putting component `%s' online.", info->id);
@@ -954,8 +1053,8 @@ e_shell_send_receive (EShell *shell)
 
 		GNOME_Evolution_Component_sendAndReceive (info->iface, &ev);
 
-		// Ignore errors, the components can decide to not implement
-		// this interface.
+		/* Ignore errors, the components can decide to not implement
+		   this interface. */
 
 		CORBA_exception_free (&ev);
 	}
@@ -1011,25 +1110,67 @@ e_shell_construct_result_to_string (EShellConstructResult result)
 	}
 }
 
-
-gboolean
-e_shell_prepare_for_quit (EShell *shell)
+/* timeout handler, so returns TRUE if we can't quit yet */
+static gboolean
+es_run_quit(EShell *shell)
 {
 	EShellPrivate *priv;
 	GSList *component_infos;
-	GList *p;
 	GSList *sp;
-	CORBA_boolean can_quit;
+	CORBA_boolean done_quit;
 
 	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
 
 	priv = shell->priv;
 	priv->preparing_to_quit = TRUE;
 
-	/* Make all the windows insensitive so we have some modal-like
-	   behavior.  */
-	for (p = priv->windows; p != NULL; p = p->next)
-		gtk_widget_set_sensitive (GTK_WIDGET (p->data), FALSE);
+	component_infos = e_component_registry_peek_list (priv->component_registry);
+	done_quit = TRUE;
+	for (sp = component_infos; sp != NULL; sp = sp->next) {
+		EComponentInfo *info = sp->data;
+		CORBA_Environment ev;
+
+		CORBA_exception_init (&ev);
+
+		done_quit = GNOME_Evolution_Component_quit(info->iface, &ev);
+		if (BONOBO_EX (&ev)) {
+			/* The component might not implement the interface, in which case we assume we can quit. */
+			done_quit = TRUE;
+		}
+
+		CORBA_exception_free (&ev);
+
+		if (!done_quit)
+			break;
+	}
+
+	if (done_quit) {
+		if  (priv->quit_timeout) {
+			g_source_remove(priv->quit_timeout);
+			priv->quit_timeout = 0;
+		}
+		e_shell_close_all_windows(shell);
+	} else if (priv->quit_timeout == 0) {
+		priv->quit_timeout = g_timeout_add(500, (GSourceFunc)es_run_quit, shell);
+	}
+
+	return !done_quit;
+}
+
+gboolean
+e_shell_quit(EShell *shell)
+{
+	EShellPrivate *priv;
+	GSList *component_infos;
+	GSList *sp;
+	CORBA_boolean can_quit;
+
+	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
+
+	priv = shell->priv;
+
+	if (priv->preparing_to_quit)
+		return FALSE;
 
 	component_infos = e_component_registry_peek_list (priv->component_registry);
 	can_quit = TRUE;
@@ -1051,22 +1192,18 @@ e_shell_prepare_for_quit (EShell *shell)
 			break;
 	}
 
-	/* Restore all the windows to be sensitive.  */
-	for (p = priv->windows; p != NULL; p = p->next)
-		gtk_widget_set_sensitive (GTK_WIDGET (p->data), TRUE);
+	if (can_quit) {
+		GList *p = shell->priv->windows;
+
+		for (; p != NULL; p = p->next) {
+			gtk_widget_set_sensitive (GTK_WIDGET (p->data), FALSE);
+			if (p == shell->priv->windows)
+				e_shell_window_save_defaults (p->data);
+		}
+		can_quit = !es_run_quit(shell);
+	}
 
-	priv->preparing_to_quit = FALSE;
 	return can_quit;
 }
 
-
-EUserCreatableItemsHandler *
-e_shell_peek_user_creatable_items_handler (EShell *shell)
-{
-	g_return_val_if_fail (E_IS_SHELL (shell), NULL);
-
-	return shell->priv->user_creatable_items_handler;
-}
-
-
 BONOBO_TYPE_FUNC_FULL (EShell, GNOME_Evolution_Shell, PARENT_TYPE, e_shell)
