@@ -28,7 +28,7 @@
 #include "e-util/e-gtk-utils.h"
 #include "e-util/e-bconf-map.h"
 
-#include "e-icon-factory.h"
+#include <e-util/e-icon-factory.h>
 #include "e-shell-constants.h"
 
 #include "e-shell.h"
@@ -40,7 +40,7 @@
 #include <gconf/gconf-client.h>
 
 #include <gtk/gtkalignment.h>
-#include <gtk/gtkbox.h>
+#include <gtk/gtkvbox.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmain.h>
@@ -55,7 +55,6 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <libgnomeui/gnome-ui-init.h>
-#include <libgnomeui/gnome-window-icon.h>
 
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-moniker-util.h>
@@ -69,6 +68,7 @@
 #include "Evolution-DataServer.h"
 
 #include <gal/widgets/e-cursors.h>
+#include "widgets/misc/e-error.h"
 
 #include <fcntl.h>
 #include <signal.h>
@@ -210,7 +210,7 @@ show_development_warning (GtkWindow *parent)
 	g_object_unref (client);
 
 	warning_dialog = gtk_dialog_new ();
-	gtk_window_set_title (GTK_WINDOW (warning_dialog), "Ximian Evolution " VERSION);
+	gtk_window_set_title (GTK_WINDOW (warning_dialog), "Evolution " VERSION);
 	gtk_window_set_modal (GTK_WINDOW (warning_dialog), TRUE);
 	gtk_dialog_add_button (GTK_DIALOG (warning_dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
 	e_dialog_set_transient_for (GTK_WINDOW (warning_dialog), GTK_WIDGET (parent));
@@ -229,9 +229,9 @@ show_development_warning (GtkWindow *parent)
 		/* xgettext:no-c-format */
 		/* Preview/Alpha/Beta version warning message */
 		_("Hi.  Thanks for taking the time to download this preview release\n"
-		  "of the Ximian Evolution groupware suite.\n"
+		  "of the Evolution groupware suite.\n"
 		  "\n"
-		  "This version of Ximian Evolution is not yet complete. It is getting close,\n"
+		  "This version of Evolution is not yet complete. It is getting close,\n"
 		  "but some features are either unfinished or do not work properly.\n"
 		  "\n"
 		  "If you want a stable version of Evolution, we urge you to uninstall\n"
@@ -253,7 +253,7 @@ show_development_warning (GtkWindow *parent)
 	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
 
 	label = gtk_label_new (_("Thanks\n"
-				 "The Ximian Evolution Team\n"));
+				 "The Evolution Team\n"));
 	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
 	gtk_misc_set_alignment(GTK_MISC(label), 1, .5);
 
@@ -388,8 +388,7 @@ idle_cb (void *data)
 	case E_SHELL_CONSTRUCT_RESULT_CANNOTREGISTER:
 		corba_shell = bonobo_activation_activate_from_id (E_SHELL_OAFIID, 0, NULL, &ev);
 		if (ev._major != CORBA_NO_EXCEPTION || corba_shell == CORBA_OBJECT_NIL) {
-			e_notice (NULL, GTK_MESSAGE_ERROR,
-				  _("Cannot access the Ximian Evolution shell."));
+			e_error_run(NULL, "shell:noshell", NULL);
 			CORBA_exception_free (&ev);
 			bonobo_main_quit ();
 			return FALSE;
@@ -397,9 +396,8 @@ idle_cb (void *data)
 		break;
 
 	default:
-		e_notice (NULL, GTK_MESSAGE_ERROR,
-			  _("Cannot initialize the Ximian Evolution shell: %s"),
-			  e_shell_construct_result_to_string (result));
+		e_error_run(NULL, "shell:noshell-reason",
+			    e_shell_construct_result_to_string(result), NULL);
 		CORBA_exception_free (&ev);
 		bonobo_main_quit ();
 		return FALSE;
@@ -521,6 +519,7 @@ main (int argc, char **argv)
 	poptContext popt_context;
 	const char **args;
 	char *evolution_directory;
+	GList *icon_list;
 
 	/* Make ElectricFence work.  */
 	free (malloc (10));
@@ -529,7 +528,7 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
+	program = gnome_program_init (PACKAGE "-" BASE_VERSION, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
 				      GNOME_PROGRAM_STANDARD_PROPERTIES,
 				      GNOME_PARAM_POPT_TABLE, options,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Evolution"),
@@ -570,8 +569,13 @@ main (int argc, char **argv)
 	glade_init ();
 	e_cursors_init ();
 	e_icon_factory_init ();
-
-	gnome_window_icon_set_default_from_file (EVOLUTION_IMAGES "/evolution-inbox.png");
+	
+	icon_list = e_icon_factory_get_icon_list ("stock_mail");
+	if (icon_list) {
+		gtk_window_set_default_icon_list (icon_list);
+		g_list_foreach (icon_list, (GFunc) g_object_unref, NULL);
+		g_list_free (icon_list);
+	}
 
 	/* FIXME We shouldn't be using the old directory at all I think */
 	evolution_directory = g_build_filename (g_get_home_dir (), "evolution", NULL);
@@ -598,6 +602,8 @@ main (int argc, char **argv)
 	g_idle_add (idle_cb, uri_list);
 
 	bonobo_main ();
-
+	
+	e_icon_factory_shutdown ();
+	
 	return 0;
 }
