@@ -1,4 +1,4 @@
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Copyright 2003, Novell Inc.
  *
  * Author(s): Michael Zucchi <notzed@ximian.com>
@@ -18,6 +18,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnome/gnome-i18n.h>
 
 #include <gal/util/e-util.h>
